#include "BlackFish.hpp"

/******************************************************************************

	HawkCreek software

	Notes:

	History of Changes:
    - 23.02.1999: file created by BuschnicK
    - 13.10.1999: removed 'Tiles16x16' etc and put them all in 'Tiles'
    - 02.11.1999: removed member function clip() and reimplemented it as a
                  helper function in Surface.cpp
	- 18.03.2000: fixed destructor (releasing of layers now works correctly)
	- 30.03.2000: _loads_ of changes...
				  finally! worldcoordinate system
				  added map class
				  implemented framerate independant scrolling
	- 31.03.2000: added ChangeMap( FileName )
	- 01.04.2000: added NewMap( Width, Height )
	- 03.04.2000: added Sprites
	- 08.04.2000: fixed cache16x16 (small sprite sizes),
				  fixed clipping sprite to early
				  !! to get rid of the wobbler check the worldtoscreen function,
					 cause sprites don't seem to have that problem ?!
	- 25.05.2000: renamed to BlackFish.cpp
	- 27.09.2000: we seem to have a strange timing problem with the input devices
				  in release/windowed mode the input becomes *very* sluggish
				  release/fullscreen doesn't have this problem
				  adding the center pixel plot in release/windowed (normally debugcode)
				  mode cures the problem ?!
 ******************************************************************************/

#include "General\debug.hpp"
#include "General\HelpMacros.hpp"
#include "General\type.hpp"
#include "IniFile\inifile.hpp"
#include "BlackFish\time.hpp"
#include "BlackFish\paint.hpp"
#include "BlackFish\cache.hpp"
#include "BlackFish\dxdraw.hpp"
#include "BlackFish\Surface.hpp"
#include "BlackFish\tileset.hpp"
#include "BlackFish\sprite.hpp"
#include "Map\map.hpp"
#include <stdio.h>				// sprintf
#include <math.h>

/*
 * constructor, loads all map and map graphic resources
 */
cBLACKFISH::cBLACKFISH(
    cSURFACE *SystemBuffer,
    cCACHE *Cache16x16,
    cCACHE *Cache64x32,
    cCACHE *Cache64x128,
    const char *IniFileName )
{
    ASSERT( DirectDraw != NULL );
    ASSERT( IniFileName != NULL );
    ASSERT( SystemBuffer != NULL );
    ASSERT( Cache16x16 != NULL );
    ASSERT( Cache64x32 != NULL );
    ASSERT( Cache64x128 != NULL );

	Scrolling = FALSE;

	this->Cache16x16   = Cache16x16;
    this->Cache64x32   = Cache64x32;
    this->Cache64x128  = Cache64x128;
    this->SystemBuffer = SystemBuffer;

    cINIFILE *IniFile = NULL;
    IniFile = new cINIFILE( IniFileName );
    ASSERT( IniFile != NULL );

    tBOOL ReturnValue = FALSE;
    ReturnValue = IniFile->ReadNumber( "[BlackFish]", "NrOfTileSetsGround", &NrOfTileSetsGround );
    ASSERT( ReturnValue == TRUE );

    ReturnValue = IniFile->ReadNumber( "[BlackFish]", "NrOfTileSetsOverlay", &NrOfTileSetsOverlay );
    ASSERT( ReturnValue == TRUE );

    ReturnValue = IniFile->ReadNumber( "[BlackFish]", "NrOfTileSetsWall", &NrOfTileSetsWall );
    ASSERT( ReturnValue == TRUE );

    ReturnValue = IniFile->ReadNumber( "[BlackFish]", "NrOfTileSetsSprite", &NrOfTileSetsSprite );
    ASSERT( ReturnValue == TRUE );

    ReturnValue = IniFile->ReadNumber( "[BlackFish]", "NrOfTileSetsMax", &NrOfTileSets );
    ASSERT( ReturnValue == TRUE );

    ReturnValue = IniFile->ReadNumber( "[BlackFish]", "ScrollSpeed", &ScrollSpeed );
    ASSERT( ReturnValue == TRUE );
	if ( ScrollSpeed > 100 ) ScrollSpeed = 1000;	// clamp value to range 0..1000

    char NameString[80];
    Tiles = NULL;
    Tiles = new cTILESET*[NrOfTileSets];
    ASSERT( Tiles != NULL );
	
	tUINT32 i		= 0;
	tUINT32 Offset	= 0;

	for ( i=0; i<NrOfTileSets; i++ )
		Tiles[i] = NULL;
	for ( i=0; i<NrOfTileSetsGround; i++ )
	{
		sprintf( NameString, "[TileSetGround%i]", i);
        Tiles[i] = new cTILESET( IniFileName, NameString );// opens the same inifile again, seems to work
        ASSERT( Tiles[i] != NULL );
        Tiles[i]->SetCache( Cache64x32 );	// ground tiles are always of this size
	}

	Offset += NrOfTileSets / 4;
	for ( i=Offset; i<NrOfTileSetsOverlay+Offset; i++ )
	{
		sprintf( NameString, "[TileSetOverlay%i]", i-Offset );
        Tiles[i] = new cTILESET( IniFileName, NameString );// opens the same inifile again, seems to work
        ASSERT( Tiles[i] != NULL );
        Tiles[i]->SetCache( Cache64x32 );	// overlay (ground) tiles are always of this size
	}

	Offset += NrOfTileSets / 4;
	for ( i=Offset; i<NrOfTileSetsWall+Offset; i++ )
	{
		sprintf( NameString, "[TileSetWall%i]", i-Offset );
        Tiles[i] = new cTILESET( IniFileName, NameString );// opens the same inifile again, seems to work
        ASSERT( Tiles[i] != NULL );
        Tiles[i]->SetCache( Cache64x128 );	// wall tiles are always of this size
	}

	Offset += NrOfTileSets / 4;
	for ( i=Offset; i<NrOfTileSetsSprite+Offset; i++ )
	{
		sprintf( NameString, "[TileSetSprite%i]", i-Offset );
        Tiles[i] = new cTILESET( IniFileName, NameString );// opens the same inifile again, seems to work
        ASSERT( Tiles[i] != NULL );
		tUINT32 Width  = Tiles[i]->GetTileWidth();
		tUINT32 Height = Tiles[i]->GetTileHeight();
		// no exact size checks, cause we might want to have e.g. 13x7 tiles, wich should be cached
		// in Cache16x16
        if ( Width <= 16 )
		{
            Tiles[i]->SetCache( Cache16x16 );
		}
        else if ( Width <= 64 )
		{
            if ( Height <= 32 )
			{
                Tiles[i]->SetCache( Cache64x32 );
			}
/*            else if ( Height <= 64 )				// unused ATM
			{
                Tiles[i]->SetCache( Cache64x64 );
			}*/
            else if ( Height <= 128)
			{
                Tiles[i]->SetCache( Cache64x128 );
			}
		}

	}

	Map = NULL;

	// hack... read current mapfile from inifile, should later be replaced by
	// level initialisation method
    char MapFileName[80] = "";
    ReturnValue = IniFile->ReadString( "[BlackFish]", "MapFileName", MapFileName );
    ASSERT( ReturnValue == TRUE );
	ChangeMap( MapFileName );

	SetRect( &ViewRect, 100, 0, 700, 480 );

    SAFEDELETE( IniFile );
}

/*
 * deletes all tilesets and the map object
 */
cBLACKFISH::~cBLACKFISH( )
{
	// free memory used by graphic tile sets
    for ( tUINT32 i=0; i<NrOfTileSets; i++)
        SAFEDELETE( Tiles[i] );
    SAFEDELETEARRAY( Tiles );
    SAFEDELETE( Map );
}

/*
 * switch current map to map 'FileName'
 */
void
cBLACKFISH::ChangeMap( const char *FileName )
{
	// start at map origin
	WorldPos.Set( 0, 0, 0 );

	// don't start off already scrolling
	ScrollTime		= 0.0;
	Scrolling		= FALSE;
	ScrollDir.Set( 0, 0, 0 );

	// destroy old map
	SAFEDELETE( Map );
	// create the new map
	Map = new cMAP( FileName );

	// adjust worldwidth and height to the new map dimensions
	WorldWidth	= 32 * Map->GetWidth();
	WorldHeight	= 32 * Map->GetHeight();
}

/*
 * start over with a new map initialized with the values in MapHeader and base layer tileset 0 tilenr 0
 */
void
cBLACKFISH::NewMap( tMAPHEADER MapHeader )
{
	ASSERT( MapHeader.Width != 0 );
	ASSERT( MapHeader.Width <= 1024 );	// !! maximum map dimension = 1024x1024
	ASSERT( MapHeader.Height != 0 );		// !! arbitrary defined here good?
	ASSERT( MapHeader.Height <= 1024 );

	// start at map origin
	WorldPos.Set( 0, 0, 0 );

	// don't start off already scrolling
	ScrollTime		= 0.0;
	Scrolling		= FALSE;
	ScrollDir.Set( 0, 0, 0 );

	// destroy old map
	SAFEDELETE( Map );
	// create the new map
	Map = new cMAP( MapHeader );

	// adjust worldwidth and height to the new map dimensions
	WorldWidth	= 32 * Map->GetWidth();
	WorldHeight	= 32 * Map->GetHeight();
}

/*
 * !!
 * Due to the Wobbler Problem the algorithm here isn't correct:
 * - Scroll speed is not equal along the world axes and the diagonals
 * - minimum speed is 1, which means you scroll at least ScrollSpeed
 *   units per second, but with very high framerates it might be more
 *   than that ( you scroll at least 1 pixel per frame )
 * - worldcoordinates are effectively truncated before use ( really
 *   the heart of the problem )
 * !! make both scroll functions inline?
 * !! we can't have this function called from multiple places at the moment
 *    i.e. keyboard and mouse scrolling don't work together because we don't 
 *    have a begin and end scrolling method.
 */
void
cBLACKFISH::Scroll( tDIRECTION Direction )
{
	switch ( Direction )
	{
		case NORTHWEST:
			ScrollDir.Set(  0.0, -2.0,  0.0 );
		break;
		case NORTH:
			ScrollDir.Set(  2.0, -2.0,  0.0 );
		break;
		case NORTHEAST:
			ScrollDir.Set(  2.0,  0.0,  0.0 );
		break;
		case EAST:
			ScrollDir.Set(  2.0,  2.0,  0.0 );
		break;
		case SOUTHEAST:
			ScrollDir.Set(  0.0,  2.0,  0.0 );
		break;
		case SOUTH:
			ScrollDir.Set( -2.0,  2.0,  0.0 );
		break;
		case SOUTHWEST:
			ScrollDir.Set( -2.0,  0.0,  0.0 );
		break;
		case WEST:
			ScrollDir.Set( -2.0, -2.0,  0.0 );
		break;
		default:
			ScrollDir.Set(  0.0,  0.0,  0.0 );
			Scrolling = FALSE;
		break;
	}

	tFLOAT Speed = 0;
	if ( Scrolling )	// we are already scrolling, update timer
	{
		Speed = (( Timer.GetTime( ) - ScrollTime ) / Timer.GetFrequency( )) * ScrollSpeed;
		ScrollTime = Timer.GetTime( );
	}

	if ( Direction != NONE )	// we are starting to scroll, initialize timer, don't scroll, yet
	{
		ScrollTime = Timer.GetTime( );

//		ScrollDir.Normalize();
		ScrollDir.Multiply( ceil ( Speed ));
		WorldPos.Add( ScrollDir );
		Scrolling = TRUE;

		if ( WorldPos.x < 0.0 )
			WorldPos.x = 0.0;
		else if ( WorldPos.x >= WorldWidth )
			WorldPos.x = WorldWidth;
		if ( WorldPos.y < 0.0 )
			WorldPos.y = 0.0;
		else if ( WorldPos.y >= WorldHeight )
			WorldPos.y = WorldHeight;
	}
}

/*
 * shortcut function, allows the translation from mouse-coordinates into scrolling directions
 * should actually be done in game.cpp
 * !! inline this? 
 */
void
cBLACKFISH::Scroll( tSINT32 PosX, tSINT32 PosY )
{
	if (( PosX == 0 ) && ( PosY == 0 ))
		Scroll( NORTHWEST );
	else if (( PosX == 0 ) && ( PosY == 599 ))
		Scroll( SOUTHWEST );
	else if (( PosX == 799 ) && ( PosY == 599 ))
		Scroll( SOUTHEAST );
	else if (( PosX == 799 ) && ( PosY == 0 ))
		Scroll( NORTHEAST );
	else if ( PosX == 0 )
		Scroll( WEST );
	else if ( PosX == 799 )
		Scroll( EAST );
	else if ( PosY == 0 )
		Scroll( NORTH );
	else if ( PosY == 599 )
		Scroll( SOUTH );
	else
		Scroll( NONE ); // !! problem 
}

/*
 * Input: Screen Coordinates
 * OutPut: ArrayIndices of the Cell displayed at these coordinates
 * returns: FALSE if ArrayIndices are out of bounds
 * !! violates Input/Modify/Output parameter order, but it seemed
 * appropriate here, considering the function name
 */
tBOOL
cBLACKFISH::GetCellForScreen( tUINT32 *ArrayX, tUINT32 *ArrayY, tUINT32 ScreenX, tUINT32 ScreenY )
{
	// return false if pixel-position is not in view-rect
	if  (( ScreenX < ViewRect.left )
	  || ( ScreenX >= ViewRect.right ) 
	  || ( ScreenY < ViewRect.top )
	  || ( ScreenY >= ViewRect.bottom ))
	  return FALSE;

	tSINT32 PixelDiffX = (( ViewRect.right + ViewRect.left ) / 2 ) - ScreenX - 12;
	tSINT32 PixelDiffY = (( ViewRect.bottom + ViewRect.top ) / 2 ) - ScreenY;

	tSINT32 WorldDiffX = 0.5 * PixelDiffX - PixelDiffY;
	tSINT32 WorldDiffY = 0.5 * PixelDiffX + PixelDiffY;

	tSINT32 ZeroX = ( WorldPos.x - WorldDiffX );
	if ( ZeroX < 0 ) ZeroX -= 32;
	tSINT32 ZeroY = ( WorldPos.y - WorldDiffY );
	if ( ZeroY < 0 ) ZeroY -= 32;

	*ArrayX =  ZeroX / 32;
	*ArrayY =  ZeroY / 32;

	if (( *ArrayX < 0 ) || ( *ArrayX >= Map->GetWidth( ))) 
		return FALSE;
	if (( *ArrayY < 0 ) || ( *ArrayY >= Map->GetHeight( ))) 
		return FALSE;
	return TRUE;
}

/*
 * Where would a given cell (ArrayIndices) be displayed on-screen?
 * Note: Doesn't perform any out-of-bounds checks
 * !! violates Input/Modify/Output parameter order, but it seemed
 * appropriate here, considering the function name
 */
void
cBLACKFISH::GetBlitRectForCell( RECT *Rect, tUINT32 ArrayX, tUINT32 ArrayY )
{
	tSINT32 WorldDiffX = WorldPos.x - ( ArrayX * 32 );
	tSINT32 WorldDiffY = WorldPos.y - ( ArrayY * 32 );

	tSINT32 PixelDiffX = WorldDiffX + WorldDiffY;
	tSINT32 PixelDiffY = 0.5 * ( WorldDiffY - WorldDiffX );

	Rect->left		= (( ViewRect.right + ViewRect.left ) / 2 ) - PixelDiffX - 12;
	Rect->right		= Rect->left + 64;
	Rect->top		= (( ViewRect.bottom + ViewRect.top ) / 2 ) - PixelDiffY - 16;
	Rect->bottom	= Rect->top + 32;
}

/*
 * !! violates Input/Modify/Output parameter order, but it seemed
 * appropriate here, considering the function name
 */
void
cBLACKFISH::GetScreenForWorld( tSINT32 *ScreenX, tSINT32 *ScreenY, cVECTOR *World )
{
	tSINT32 WorldDiffX = WorldPos.x - World->x;
	tSINT32 WorldDiffY = WorldPos.y - World->y;

	tSINT32 PixelDiffX = WorldDiffX + WorldDiffY;
	tSINT32 PixelDiffY = 0.5 * ( WorldDiffY - WorldDiffX );

	*ScreenX = (( ViewRect.right + ViewRect.left ) / 2 ) - PixelDiffX - 12;
	*ScreenY = (( ViewRect.bottom + ViewRect.top ) / 2 ) - PixelDiffY - World->z - 16;
}

/*
 * return the number of tilesets of the specified type
 * MAX: maximum number of tilesets allowed (default 1024)
 * ALL: number of tilesets allocated
 * GROUND, OVERLAY, WALL, SPRITE: number of allocated tilesets of the respective type
 */
tUINT32
cBLACKFISH::GetNrOfTileSets( tTILESETTYPE Type )
{
	switch ( Type )
	{
	case MAX:
		return NrOfTileSets;
	case ALL:
		return NrOfTileSetsGround + NrOfTileSetsOverlay + NrOfTileSetsWall + NrOfTileSetsSprite;
	case GROUND:
		return NrOfTileSetsGround;
	case OVERLAY:
		return NrOfTileSetsOverlay;
	case WALL:
		return NrOfTileSetsWall;
	case SPRITE:
		return NrOfTileSetsSprite;
	default:
		return 0;
	}
}

/*
 * Renders GroundCells
 * Note: Possible to optimize clipping, since the clipped rectangle results
 *       don't change for the viewrect sides
 */
void
cBLACKFISH::RenderGround( tSINT32 ArrayStartX, tSINT32 ArrayStartY,
						  tSINT32 PixelStartX, tSINT32 PixelStartY )
{
    RECT Dest, Source;
    cCELLINFO *Layer  = NULL;
	tSINT32 ArrayPosX = ArrayStartX;
	tSINT32 ArrayPosY = ArrayStartY;
	tSINT32 PixelPosX = PixelStartX;
	tSINT32 PixelPosY = PixelStartY;

    for ( tUINT32 y = 0; y < TILESVISIBLEY; y++ )
    {
        for ( tUINT32 x = 0; x < TILESVISIBLEX; x++ )
        {
            // don't draw tiles outside of the map array bounds
			SetRect( &Source, 0, 0, 64, 32 );
			SetRect( &Dest, PixelPosX, PixelPosY, PixelPosX + 64, PixelPosY + 32 );

			if ( Clip( ViewRect, &Source, &Dest ))
			{
				if (( ArrayPosX >= 0 ) && ( ArrayPosX < Map->GetWidth()) &&
					( ArrayPosY >= 0 ) && ( ArrayPosY < Map->GetHeight()))
				{
					Layer = Map->GetFirstLayer( ArrayPosX, ArrayPosY );
					while ( Layer != NULL )
					{
						Tiles[Layer->TileSet]->BlitTrans( Layer->TileNr, &Source, SystemBuffer, &Dest );
						Layer = Layer->NextLayer;
					}
				}
				else
				{	// out of map array -> blit black tile
					Tiles[0]->BlitTrans( 1, &Source, SystemBuffer, &Dest );
				}
			}
			
			PixelPosX+=64;
            ArrayPosX++;
            ArrayPosY++;

        }

		PixelPosY += 16;
		PixelPosX = PixelStartX;

		if ( (y % 2) == 1 )
			ArrayStartX--;
		else
		{
			PixelPosX += 32;
			ArrayStartY++;
		}

		ArrayPosX = ArrayStartX;
		ArrayPosY = ArrayStartY;

	}
}

/*
 * Renders Walls and Sprites, algorithmically very similar to RenderGround()
 * so changes to either function should be considered for the other as well.
 * the reason for the seperation is drawing order, since we don't have any
 * per-pixel depth information ( = z-buffer, not practical for our purposes )
 * we have to use the painters algorithm:
 * - draw all the groundlayers, as these never block view on any other items
 *   (this has already been done at this point ( RenderGround()))
 * - Render all sprites for a particular cell, then render the wall for that cell
 * !! note: under some circumstances sprites may pop through north-west walls
 *    this hopefully won't be visible once we've implemented transparency
 *	  for the walls (all walls in front of the 'poppers' will be transparent
 *	  anyway). Other than that I can't think of a solution that'll work
 *    for all cases.
 */
void
cBLACKFISH::RenderWallsAndSprites( tSINT32 ArrayStartX, tSINT32 ArrayStartY,
								   tSINT32 PixelStartX, tSINT32 PixelStartY,
								   tBOOL SmallWalls )
{
    RECT Dest, Source;
	tSINT32 ArrayPosX = ArrayStartX;
	tSINT32 ArrayPosY = ArrayStartY;
	tSINT32 PixelPosX = PixelStartX;
	tSINT32 PixelPosY = PixelStartY;

    // plus x because walls at the bottom of the screen would be clipped out of existance too early otherwise...
    // (Siedler3-like bug ;) )
    for ( tUINT32 y = 0; y < TILESVISIBLEY + 6; y++ )
    {
        for ( tUINT32 x = 0; x < TILESVISIBLEX + 2; x++ )
        {
            if (( ArrayPosX >= 0 ) && ( ArrayPosX < Map->GetWidth( ))
			  &&( ArrayPosY >= 0 ) && ( ArrayPosY < Map->GetHeight( )))
			{				
				cWALLINFO *Wall = Map->GetWall( ArrayPosX, ArrayPosY );
				if ( Wall != NULL )
				{
					if ( SmallWalls )
					{
						SetRect( &Source, 0, 80, 64, 128 );
						SetRect( &Dest, PixelPosX, PixelPosY - 16, PixelPosX + 64, PixelPosY + 32 );
					}
					else
					{
						SetRect( &Source, 0, 0, 64, 128 );
						SetRect( &Dest, PixelPosX, PixelPosY - 96, PixelPosX + 64, PixelPosY + 32 );
					}
/*		
tCACHEDRECTNODE *Node = 
	(tCACHEDRECTNODE *) Tiles[Wall->TileSet]->GetTile( Wall->TileNr )->GetCacheStatus( );

if ( Node )
{
	tUINT32 Temp = Tiles[0]->GetTile( 0 )->GetTransparentColor( );
	Tiles[0]->GetTile( 0 )->SetTransparentColor( 0 );
	Tiles[0]->BlitTrans( 0, &Source, Node->CacheSurface, &Node->CacheRect );
	Tiles[0]->GetTile( 0 )->SetTransparentColor( Temp );
}
*/
					if ( Clip( ViewRect, &Source, &Dest ) )
//Tiles[0]->BlitTrans( 0, &Source, SystemBuffer, &Dest );
						Tiles[Wall->TileSet]->BlitTrans( Wall->TileNr, &Source, SystemBuffer, &Dest );
/*if (Node)
Cache64x128->Invalidate( Tiles[Wall->TileSet]->GetTile( Wall->TileNr ));
*/
				}
			}
			
            // don't draw tiles outside of the map array bounds
            if (( ArrayPosX > 0 ) && ( ArrayPosX <= Map->GetWidth( ))
			  &&( ArrayPosY > 0 ) && ( ArrayPosY <= Map->GetHeight( )))
			{
				
				std::list<cSPRITE *> *SpriteList = Map->GetSpriteList( ArrayPosX - 1, ArrayPosY - 1);
				std::list<cSPRITE *>::iterator Iterator;
				
				//					if ( SpriteList->size( ) > 0 ) // might be slightly faster than entering the loop
				for ( Iterator = SpriteList->begin( ); Iterator != SpriteList->end( ); Iterator++ )
				{
					// !! Sprite origin = where?! in worldcoordinates
					tUINT32 TileSet = (*Iterator)->GetTileSet( );
					tUINT32 TileNr	= (*Iterator)->GetTileNr( );
					
					tSINT32 ScreenX, ScreenY;
					cVECTOR Position;
					(*Iterator)->GetPosition( &Position );
					GetScreenForWorld( &ScreenX, &ScreenY, &Position );
					
					tUINT32 Width	= Tiles[TileSet]->GetTileWidth( );
					tUINT32 Height	= Tiles[TileSet]->GetTileHeight( );
					if ( Height <= 32 ) { ScreenY += Height; } else { ScreenY += 32; }
					SetRect( &Source, 0, 0, Width, Height );
					SetRect( &Dest, ScreenX, ScreenY - Height, ScreenX + Width, ScreenY );
					
					if ( Clip( ViewRect, &Source, &Dest ))
						Tiles[TileSet]->BlitTrans( TileNr, &Source, SystemBuffer, &Dest );
				}
			}
			
			PixelPosX+=64;
			ArrayPosX++;
			ArrayPosY++;
        }

		PixelPosY += 16;
		PixelPosX = PixelStartX;

		if ( (y % 2) == 1 )
			ArrayStartX--;
		else
		{
			PixelPosX += 32;
			ArrayStartY++;
		}

		ArrayPosX = ArrayStartX;
		ArrayPosY = ArrayStartY;

	}
//Cache64x128->Dump( SystemBuffer, 1 );
}

/*
 * renders the whole map (including sprites and walls)
 */
void
cBLACKFISH::Render( tBOOL SmallWalls )
{
	// Calculate array position at the top-left corner of the screen
	// World to array is by a simple scale by 1/32 (see the matrices in readme.txt)
	// this gives the cell at the screen center, an offset is added
	// to get the cell in the corner
	tSINT32 ArrayStartX   = ( WorldPos.x / 32 ); ArrayStartX += 3;
	tSINT32 ArrayStartY   = ( WorldPos.y / 32 ); ArrayStartY -= 13;

	// TempX is used to convert float worldcoordinates into integer coordinates,
	// so i can use the % (modulo) operator
	// TempX and TempY will become pixel-offsets later on, so I divide the world
	// coordinates by 2, loosing precision ( 2 Worldunits correspond to one pixel )
	tSINT32 TempX = WorldPos.x / 2; TempX %= 16;
	tSINT32 TempY = WorldPos.y / 2; TempY %= 16;

	// Matrix to convert from isospace to screenspace ( see readme.txt )
	tSINT32 PixelX = 2 * ( TempX + TempY );
	tSINT32 PixelY = TempY - TempX;

	// adjust pixel offsets to start in our map area and not at 0,0 screen coordinates
	// also, draw half a cell off screen (-32), so we don't see a jiggsaw pattern when
	// scrolling cells off/on screen
	tSINT32 PixelStartX      = -PixelX + ViewRect.left - 32;// + 12; !! 12 pixel off screen center
	tSINT32 PixelStartY		 = -PixelY + ViewRect.top - 32;

	RenderGround( ArrayStartX, ArrayStartY, PixelStartX, PixelStartY );

	// -1 because sprites at the left screen border will disappear too early otherwise
	RenderWallsAndSprites( ArrayStartX - 1, ArrayStartY - 1, PixelStartX - 64, PixelStartY, SmallWalls );

	DEBUGCODE
	(
	// plot a pixel at the viewport center (wrong by 12 pixels)
    cPAINT *Painter = new cPAINT();
    Painter->SetDrawingTarget( SystemBuffer );
    Painter->BeginDrawing();
    Painter->PutPixel( (tFLOAT)(ViewRect.right + ViewRect.left) / 2 - 12,
                       (tFLOAT)(ViewRect.bottom + ViewRect.top) / 2 , 0xFFFF );
    Painter->EndDrawing();
	SAFEDELETE( Painter );
	)
}

