#ifndef __BLACKFISH_HPP__
#define __BLACKFISH_HPP__

/******************************************************************************
    HawkCreek software

    Purpose:

 ******************************************************************************/

#ifndef __TYPE_HPP__
#include "general\type.hpp"
#endif

#ifndef __VECTOR_HPP__
#include "general\vector.hpp"
#endif

#ifndef __MAP_HPP__
#include "Map\Map.hpp"
#endif

#ifndef __WINDOWS_H__
#define __WINDOWS_H__
  #ifndef WIN32_LEAN_AND_MEAN // !! reset this after including windows.h?
  #define WIN32_LEAN_AND_MEAN
  #endif
#include <windows.h> // !! nur fr RECT ?!
#endif

class cSEQUENCER;
class cSURFACE;
class cTILESET;
class cDXDRAW;
class cSPRITE;
class cCACHE;

class
cBLACKFISH
{
protected:								// protected because cEDITFISH inherits from this and needs access to the data

    cSURFACE  *SystemBuffer;            // invisible videomemory surface

    cCACHE    *Cache16x16;
    cCACHE    *Cache64x32;
    cCACHE    *Cache64x128;

    tUINT32	   NrOfTileSetsGround;
    tUINT32	   NrOfTileSetsOverlay;
    tUINT32	   NrOfTileSetsWall;
    tUINT32	   NrOfTileSetsSprite;
    tUINT32	   NrOfTileSets;
    cTILESET **Tiles;

	tSINT32    WorldWidth;
	tSINT32    WorldHeight;
	cVECTOR    WorldPos;

	cMAP	  *Map;

	tUINT32	   ScrollSpeed;
	tDOUBLE	   ScrollTime;
	tBOOL	   Scrolling;
	cVECTOR    ScrollDir;

    enum { TILESVISIBLEX = 11, TILESVISIBLEY = 33 };
	typedef enum tTILESETTYPE { GROUND = 0, OVERLAY, WALL, SPRITE, ALL, MAX };

    void RenderGround( tSINT32 ArrayStartX, tSINT32 ArrayStartY,
  					   tSINT32 PixelStartX, tSINT32 PixelStartY );
	void RenderWallsAndSprites( tSINT32 ArrayStartX, tSINT32 ArrayStartY,
								tSINT32 PixelStartX, tSINT32 PixelStartY,
								tBOOL SmallWalls );

public:
    RECT	   ViewRect;

    cBLACKFISH( cSURFACE *SystemBuffer,
                cCACHE *Cache16x16,
                cCACHE *Cache64x32,
                cCACHE *Cache64x128,
                const char *IniFileName );
    ~cBLACKFISH( );

    void Render( tBOOL SmallWalls = FALSE );
	void Scroll( tDIRECTION Direction );
    void Scroll( tSINT32 PosX, tSINT32 PosY );
	void ChangeMap( const char *FileName );
	void NewMap( tMAPHEADER MapHeader );
	cMAP *GetMap( void ) { return Map; };

	void GetBlitRectForCell( RECT *Rect, tUINT32 ArrayX, tUINT32 ArrayY );
	tBOOL GetCellForScreen( tUINT32 *ArrayX, tUINT32 *ArrayY, tUINT32 ScreenX, tUINT32 ScreenY );
	void GetScreenForWorld( tSINT32 *ScreenX, tSINT32 *ScreenY, cVECTOR *World );
	tUINT32 GetNrOfTileSets( tTILESETTYPE Type );

	inline void SetPosition( cVECTOR *Position )
		{ WorldPos.x = Position->x; WorldPos.y = Position->y; };
	inline tUINT32 WorldX( void ) { return (tUINT32) WorldPos.x; };
	inline tUINT32 WorldY( void ) { return (tUINT32) WorldPos.y; };
};

#endif

