#ifndef __CACHE_HPP__
#define __CACHE_HPP__

/******************************************************************************
    HawkCreek software

    Purpose:
    A Videomemory Surface Cache using the LRU (Least Recently Used) algorithm

******************************************************************************/

#ifndef __DEBUG_HPP__
#include "general\debug.hpp"
#endif

#ifndef __TYPE_HPP__
#include "general\type.hpp"
#endif

#ifndef __WINDOWS_H__
#define __WINDOWS_H__
  #ifndef WIN32_LEAN_AND_MEAN // !! reset this after including windows.h?
  #define WIN32_LEAN_AND_MEAN
  #endif
#include <windows.h> // !! nur fr RECT ?!
#endif

class cSURFACE;
class cDXDRAW;

struct
tCACHEDRECTNODE
{
    RECT CacheRect;                     // what part of the cache surface is mine?
    cSURFACE *CacheSurface;             // what cache surface do I belong to?
    cSURFACE *SourceSurface;            // the source (systemmemory) surface

    tCACHEDRECTNODE *Next;              // doubly linked list, used as a priority queue
    tCACHEDRECTNODE *Prev;
};

class
cCACHE
{
    private:

    tUINT32 NrOfCacheSurfaces;           // how many cache Surfaces = vidmemsurface of size 256*256
    cSURFACE **CacheSurface;          // array of NrOfCachSurfaces pointers to cSURFACEs in vidmem

    tUINT32 CacheRectWidth;
    tUINT32 CacheRectHeight;
    tUINT32 NrOfCacheRects;              // how many CacheRects fit into the cachesurfaces?
    tUINT32 NrOfFreeCacheRects;          // how many CacheRects are currently unused?
                                        // after a couple of seconds run-time this should always be 0
    tCACHEDRECTNODE *First;
    tCACHEDRECTNODE *Last;

    enum { CacheSurfaceWidth = 256 };   // constant, Width = Height, so only one is needed

    void MoveNodeUp( tCACHEDRECTNODE *Node );
    void CacheInSurface( cSURFACE *Surface );

    public:

    cCACHE(
        tUINT32 nCacheRectWidth,
        tUINT32 nCacheRectHeight,
        tUINT32 TransparentColor,
        tUINT32 nNrOfCacheSurfaces );
    ~cCACHE();

    void Invalidate( void );
	void Invalidate( cSURFACE *Surface );

    void Blit( cSURFACE *Source, RECT *SourceRect, cSURFACE *Dest, RECT *DestRect );
    void BlitTrans( cSURFACE *Source, RECT *SourceRect, cSURFACE *Dest, RECT *DestRect );

    DEBUGCODE ( void Dump( cSURFACE *Dest, tUINT32 SurfaceNr ); )
};

#endif
