#include "Dialog.hpp"

/*********************************************************************
		HawkCreek software

		File that contains common dialog class and all heirs

		History of changes:
		- created by Capella, 03/30/2000
***********************************************************************/

#include "BlackFish\Paint.hpp"
#include "BlackFish\Surface.hpp"
#include "general\HelpMacros.hpp"
#include "general\Debug.hpp"
#include "BlackFish\PixelConverter.hpp"
#include "BlackFish\input.hpp"
#include "BlackFish\write.hpp"

#include <stdlib.h>
#include <fstream>

extern cDIALOG* Dialog		= NULL;

cEDITBOX::cEDITBOX(tUINT32 LeftPos, 
				   tUINT32 TopPos, 
				   tUINT32 MaxLength,
				   cPAINT* DialogPainter,
				   tUINT16 Color)
{
	Left			= LeftPos;
	Top				= TopPos;
	Length			= MaxLength;
	Position		= 0;
	EditPainter		= DialogPainter;
	BackgroundColor = Color;
	String			= new char[Length+1];
	memset(String, 0, Length+1);
}

cEDITBOX::~cEDITBOX()
{
	SAFEDELETEARRAY(String);
}

void
cEDITBOX::AddChar(char ASCII)
{
	if (Position < Length)
	{
		EditPainter->Fill(Left + Position * 8, 
					  Top,
					  Left + (Position + 1) * 8,
					  Top + 16,
					  BackgroundColor);
		String[Position] = ASCII;
		Writer->Out(Left + Position * 8, Top, &(String[Position]));
		Position++;
		SetCursor();
	}
}

void
cEDITBOX::DeleteLastChar()
{
	if (Position > 0)
	{
		DeleteCursor();
		Position--;
		String[Position] = 0;
		EditPainter->Fill(Left + Position * 8, 
					  Top,
					  Left + (Position + 2) * 8,
					  Top + 16,
					  BackgroundColor);
		SetCursor();
	}
}

void
cEDITBOX::Reset()
{
	memset(String, 0, Length+1);
	EditPainter->Fill(Left, 
					  Top, 
					  Left + 1 + Position * 8, 
					  Top + 16, 
					  BackgroundColor);
	Position = 0;
}

/*
 * assumes that String is empty and should be called directly after 
 * construction
 */
void
cEDITBOX::Init(const char* InitString)
{
	memcpy(String, InitString, Length+1);
	Position = Length;
	Writer->Out( Left, Top, String); 
}

void
cEDITBOX::SetCursor()
{
	EditPainter->Fill(Left + Position * 8, Top, 
					  Left + 1 + Position * 8, Top + 16, 
					  0xffff);
	// hier mte dann der Timer gestartet werden etc.
}

void
cEDITBOX::DeleteCursor()
{
	EditPainter->Fill(Left + Position * 8, Top, 
					  Left + 1 + Position * 8, Top + 16, 
					  BackgroundColor);
}

cDIALOG::cDIALOG()
{
	FileName = "graphics\\testdialog.bmp";
	PositionX = 100;
	PositionY = 100;
	Initialize();
}

// construction, set to default if no values are passed
cDIALOG::cDIALOG( char* BmpFileName ,
				 tUINT32 PosX = 100,
				 tUINT32 PosY = 100 )
{
	FileName = BmpFileName;
	PositionX = PosX;
	PositionY = PosY;
	Initialize();
}

cDIALOG::~cDIALOG()
{
	SAFEDELETE (BmpSurface);
	SAFEDELETE( PixelConverter );
	*LOG << "cDIALOG destroyed \n";
}

void
cDIALOG::Initialize()
{
	DialogActive = TRUE;
	cPAINT *Painter = new cPAINT();

	tBOOL Check;
	Check = Painter->GetBMPDimensions(FileName, &Width, &Height);
	ASSERT (Check);

    BmpSurface = new cSURFACE();
	BmpSurface->Create(Width, Height, FALSE);

	Painter->SetDrawingTarget(BmpSurface);
	Painter->LoadBMP(FileName);

	PixelConverter = NULL;
    PixelConverter = new cPIXELCONVERTER( BmpSurface );
    ASSERT( PixelConverter != NULL );
    tUINT32 TransparentColor = PixelConverter->RGBToColor( 255, 0, 255 );
    BmpSurface->SetTransparentColor(TransparentColor);


	// bye, bye Painter
	SAFEDELETE (Painter);

	ReturnData = NULL;
}

tBOOL
cDIALOG::SetPosition( tUINT32 PosX, tUINT32 PosY )
{
	// if new Position is valid for Dialog, move Dialog there
	if ((PosX < 800 - Width) && (PosY < 600 - Height) &&
		(PosX >= 0) && (PosY >= 0))
	{
		PositionX = PosX;
		PositionY = PosY;
		return TRUE;
	}
	else
	{
		return FALSE;
	}

}


void
cDIALOG::MouseClick( tSINT32 MouseX, tSINT32 MouseY, tUINT32 Click)
{
	// detect left mouse click
	if (Click & cMOUSE::Button1)
	{

	}
}

void
cDIALOG::CharMessage(char ASCII)
{
	*LOG << ASCII ;
}

void
cDIALOG::Show( cSURFACE* BlitSurface )
{
	tUINT32 PosX, PosY;
	GetPosition( &PosX, &PosY );
	tUINT32 Width, Height;
	GetSize(&Width, &Height);

	RECT Dest;
	Dest.left	= PosX;
	Dest.top	= PosY;
	Dest.right  = Dest.left + Width;
	Dest.bottom = Dest.top + Height;

	RECT Source;
	Source.left   = 0;
	Source.top    = 0;
	Source.right  = Source.left + Width;
	Source.bottom = Source.top + Height;

	BlitSurface->BlitTrans( GetSurface(), &Source, &Dest);
}

tUINT32 
cDIALOG::GetState()
{
	if (DialogActive)
		return ACTIVE;
	else
		return FINISH;
};


cRETURNDATABASE*
cDIALOG::GetDataPackage()
{
	return ReturnData;
}

//////////////////////////////////////////////////////////////////////////////////


cNEWMAPDIALOG::cNEWMAPDIALOG() : cDIALOG("graphics\\NewMapDialog.bmp")
{
	Writer->SetDrawingTarget(BmpSurface);
	ReturnData = NULL;

	ReturnData = new cRETURNDATA();
	ReturnData->Name = NULL; 
	ReturnData->Author = NULL; 
//	ReturnData->DialogID = D_NEWMAP;
	ReturnData->Width = 20;
	ReturnData->Height = 20;

	Painter = new cPAINT();
	Painter->SetDrawingTarget(BmpSurface);

	EditName   = new cEDITBOX(32, 102, 55, Painter, 0);
	EditAuthor = new cEDITBOX(32, 140, 55, Painter, 0);
	EditWidth  = new cEDITBOX(120, 180, 3, Painter, 0);
	EditWidth->Init("20");
	WidthChanged = FALSE;
	EditHeight = new cEDITBOX(272, 180, 3, Painter, 0);
	EditHeight->Init("20");
	HeightChanged = FALSE;

	InputFocus = NONE;
	ChangeInputFocus(NAME);
}

cNEWMAPDIALOG::~cNEWMAPDIALOG()
{
	SAFEDELETE( EditName );
	SAFEDELETE( EditAuthor );
	SAFEDELETE( EditWidth );
	SAFEDELETE( EditHeight );
	SAFEDELETE( Painter );
	SAFEDELETE( ReturnData );
}

void
cNEWMAPDIALOG::CharMessage(char ASCII)
{
	Writer->SetDrawingTarget( BmpSurface );
	if (ASCII == '\t')
	{
		ChangeInputFocus();
	}
	else
	{
		switch (InputFocus)
		{
			case NAME :
				if (ASCII == 13)
				{
					EditName->DeleteLastChar();
				}
				else
				{
					EditName->AddChar(ASCII);
				}
			break;
			case AUTHOR :
				if (ASCII == 13)
				{
					EditAuthor->DeleteLastChar();
				}
				else
				{
					EditAuthor->AddChar(ASCII);
				}
			break;
			case WIDTH :
				if (ASCII == 13)
				{
					EditWidth->DeleteLastChar();
				}
				else if ((ASCII >= 48) && (ASCII <= 57))
				{
					EditWidth->AddChar(ASCII);
				}
			break;
			case HEIGHT :
				if (ASCII == 13)
				{
					EditHeight->DeleteLastChar();
				}
				else if ((ASCII >= 48) && (ASCII <= 57))
				{
					EditHeight->AddChar(ASCII);
				}
			break;
			case OK :
				if (ASCII == '\n')
				{
					OnOk();
				}
			break;
			case CANCEL :
				if (ASCII == '\n')
				{
					OnCancel();
				}
			break;
		}
	}
}

void
cNEWMAPDIALOG::MouseClick(tSINT32 MouseX, tSINT32 MouseY, tUINT32 Click)
{
	// all left clicks handled here
	if (Click & cMOUSE::Button1)
	{
		// check if MousePointer is over OK Button
		if ((MouseX >= 400 + PositionX) && (MouseX <= 456 + PositionX) &&
			(MouseY >= 173 + PositionY) && (MouseY <= 198 + PositionY))
		{
			OnOk();
		}

		//check if Mouse is over Cancel button
		if ((MouseX >= 460 + PositionX) && (MouseX <= 516 + PositionX) &&
			(MouseY >= 174 + PositionY) && (MouseY <= 196 + PositionY))
		{
			OnCancel();
		}

		// check if Mouse is over Name Field
		if ((MouseX >= 28 + PositionX) && (MouseX <= 516 + PositionX) &&
			(MouseY >= 100 + PositionY) && (MouseY <= 120 + PositionY))
		{
			ChangeInputFocus(NAME);
		}
		// check if Mouse is over Author Field
		else if ((MouseX >= 20 + PositionX) && (MouseX <= 512 + PositionX) &&
				 (MouseY >= 138 + PositionY) && (MouseY <= 160 + PositionY))
		{
			ChangeInputFocus(AUTHOR);
		}
		// check if Mouse is over Width Field
		else if ((MouseX >= 110 + PositionX) && (MouseX <= 174 + PositionX) &&
				 (MouseY >= 178 + PositionY) && (MouseY <= 196 + PositionY))
		{
			
			ChangeInputFocus(WIDTH);
		}
		// check if Mouse is over Height Field
		else if ((MouseX >= 262 + PositionX) && (MouseX <= 320 + PositionX) &&
				 (MouseY >= 174 + PositionY) && (MouseY <= 194 + PositionY))
		{
			
			ChangeInputFocus(HEIGHT);
		}
	}
}

void
cNEWMAPDIALOG::ChangeInputFocus(tFOCUS NewFocus)
{
	if (NewFocus == InputFocus)
		return;

	tUINT16 Green = PixelConverter->RGBToColor(0,69,0);
	
	switch (InputFocus)
	{
		case NAME :
			EditName->DeleteCursor();
			if (NewFocus == NONE)
				NewFocus = AUTHOR;
		break;
		case AUTHOR :
			EditAuthor->DeleteCursor();
			if (NewFocus == NONE)
				NewFocus = WIDTH;
		break;
		case WIDTH :
			EditWidth->DeleteCursor();
			if (NewFocus == NONE)
				NewFocus = HEIGHT;
		break;
		case HEIGHT :
			EditHeight->DeleteCursor();
			if (NewFocus == NONE)
				NewFocus = OK;
		break;
		case OK :
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect(401, 173, 457, 198, Green);
			Painter->EndDrawing();
			if (NewFocus == NONE)
				NewFocus = CANCEL;
		break;
		case CANCEL :
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect(461, 173, 516, 198, Green);
			Painter->EndDrawing();
			if (NewFocus == NONE)
				NewFocus = NAME;
		break;
	}

	switch (NewFocus)
	{
		case NAME :
			InputFocus = NAME;
			EditName->SetCursor();
		break;
		case AUTHOR :
			InputFocus = AUTHOR;
			EditAuthor->SetCursor();
		break;
		case HEIGHT :
			if(!HeightChanged)
			{
				EditHeight->Reset();
				HeightChanged = TRUE;
			}
			InputFocus = HEIGHT;
			EditHeight->SetCursor();
		break;
		case WIDTH :
			if(!WidthChanged)
			{
				EditWidth->Reset();
				WidthChanged = TRUE;
			}
			InputFocus = WIDTH;
			EditWidth->SetCursor();
		break;
		case OK :
			InputFocus = OK;
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect( 401, 173, 457, 198, 0 );
			Painter->EndDrawing();
		break;
		case CANCEL :
			InputFocus = CANCEL;
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect( 461, 173, 516, 198, 0 );
			Painter->EndDrawing();
		break;
	}
}

void
cNEWMAPDIALOG::OnOk()
{
	tUINT32 Width = atoi(EditWidth->GetString());
	tUINT32 Height = atoi(EditHeight->GetString());
	if((Width > 0) && (Width <= 512))
	{
		ReturnData->Width = Width;
	}
	if((Height > 0) && (Height <= 512))
	{
		ReturnData->Height = Height;
	}
	ReturnData->Name = EditName->GetString();
	ReturnData->Author = EditAuthor->GetString();
	DialogActive = FALSE;
}

void
cNEWMAPDIALOG::OnCancel()
{
	SAFEDELETE(ReturnData);
	DialogActive = FALSE;
}

cRETURNDATABASE*
cNEWMAPDIALOG::GetDataPackage()
{
	return ReturnData;
}

cLOADMAPDIALOG::cLOADMAPDIALOG() : cDIALOG("graphics\\LoadMapDialog.bmp")
{
	Painter = new cPAINT();
	Painter->SetDrawingTarget(BmpSurface);
	EditFileName = new cEDITBOX( 32, 63, 50, Painter, 0);
	ReturnData = new cRETURNDATA();
	InputFocus = NONE;
	ChangeInputFocus(FILE);
}

cLOADMAPDIALOG::~cLOADMAPDIALOG()
{
	SAFEDELETE(EditFileName);
	SAFEDELETE(ReturnData);
	SAFEDELETE(Painter);
}

cRETURNDATABASE*
cLOADMAPDIALOG::GetDataPackage()
{
	return ReturnData;
}

void
cLOADMAPDIALOG::CharMessage(char ASCII)
{
	Writer->SetDrawingTarget(BmpSurface);
	if (ASCII == '\t')
	{
		ChangeInputFocus();
	}
	else
	{
		switch (InputFocus)
		{
			case FILE :
				if (ASCII == 13)
				{
					EditFileName->DeleteLastChar();
				}
				else
				{
					EditFileName->AddChar(ASCII);
				}
			break;
			case OK :
				if (ASCII == '\n')
				{
					OnOk();
				}
			break;
			case CANCEL :
				if (ASCII == '\n')
				{
					OnCancel();
				}
			break;
		}
	}
}

void
cLOADMAPDIALOG::MouseClick(tSINT32 MouseX, tSINT32 MouseY, tUINT32 Click)
{
	// detect left click on ok button
	if ((MouseX >= 360 + PositionX) && (MouseX < 416 + PositionY) && 
		(MouseY >= 84  + PositionX) && (MouseY < 108 + PositionY) && 
		(Click & cMOUSE::Button1))
	{
		OnOk();
	}
	//detect left click on cancel button
	if ((MouseX >= 420 + PositionX) && (MouseX <= 476 + PositionX) &&
		(MouseY >= 84 + PositionY)  && (MouseY <= 108 + PositionY) &&
		(Click & cMOUSE::Button1))
	{
		OnCancel();
	}
}

void
cLOADMAPDIALOG::OnOk()
{
	std::string FileName = "maps\\";
	FileName.append(EditFileName->GetString());	
	if (FileName.find(".map") == -1)
	{
		FileName.append(".map");
	}
*LOG << FileName.c_str() << "\n";

	std::fstream TestStream;
	TestStream.open(FileName.c_str(), std::ios::in|std::ios::binary);
	if(TestStream.is_open())
	{
		ReturnData->FileName = FileName;
		DialogActive = FALSE;
		TestStream.close();
	}
	else
	{
		EditFileName->Reset();
		ChangeInputFocus(FILE);
	}
}

void
cLOADMAPDIALOG::OnCancel()
{
	SAFEDELETE(ReturnData);
	DialogActive = FALSE;
}

void
cLOADMAPDIALOG::ChangeInputFocus(tFOCUS NewFocus)
{
	if (InputFocus == NewFocus)
		return;

	tUINT16 Green = PixelConverter->RGBToColor(0,69,0);
	switch (InputFocus)
	{
		case FILE :
			if (NewFocus == NONE)
				NewFocus = OK;
			EditFileName->DeleteCursor();
		break;
		case OK :
			if (NewFocus == NONE)
				NewFocus = CANCEL;
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect(360, 84, 416, 109, Green);
			Painter->EndDrawing();
		break;
		case CANCEL :
			if (NewFocus == NONE)
				NewFocus = FILE;
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect(420, 84, 476, 109, Green);
			Painter->EndDrawing();
		break;
	}

	switch (NewFocus)
	{
		case FILE :
			InputFocus = FILE;
			EditFileName->SetCursor();
		break;
		case OK :
			InputFocus = OK;
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect(360, 84, 416, 109, 0);
			Painter->EndDrawing();
		break;
		case CANCEL :
			InputFocus = CANCEL;
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect(420, 84, 476, 109, 0);
			Painter->EndDrawing();
		break;
	}
}

cSAVEMAPDIALOG::cSAVEMAPDIALOG(const char* MapName, const char* Author, std::string FileName) 
	: cDIALOG("graphics\\SaveMapDialog.bmp")
{
	Writer->SetDrawingTarget(BmpSurface);
	Painter = new cPAINT();
	Painter->SetDrawingTarget(BmpSurface);
	EditName = new cEDITBOX(32, 80, 55, Painter, 0);
	EditName->Init(MapName);
	NameChanged = FALSE;
	EditAuthor = new cEDITBOX(32, 126, 55, Painter, 0);
	EditAuthor->Init(Author);
	AuthorChanged = FALSE;
	EditFile = new cEDITBOX(32, 170, 55, Painter, 0);
	if(FileName != "")
	{
		//desect file name string so that only file title remains
		tSINT32 StartPos = FileName.find_last_of("\\") + 1;
		tSINT32 EndPos	 = FileName.find_last_of(".");
		FileName = FileName.substr(StartPos, EndPos - StartPos);
	}
	EditFile->Init(FileName.c_str());
	FileChanged = FALSE;
	InputFocus = NONE;
	ChangeInputFocus(OK);
	ReturnData = new cRETURNDATA();
	ReturnData->MapName = MapName;	
	ReturnData->Author  = Author;
	ReturnData->FileName = "maps\\";
}

cSAVEMAPDIALOG::~cSAVEMAPDIALOG()
{
	SAFEDELETE(Painter);
	SAFEDELETE(EditName);
	SAFEDELETE(EditAuthor);
	SAFEDELETE(EditFile);
	SAFEDELETE(ReturnData);
}

void
cSAVEMAPDIALOG::CharMessage(char ASCII)
{
	Writer->SetDrawingTarget(BmpSurface);

	if (ASCII == '\t')
	{
		ChangeInputFocus();
	}
	else
	{
		switch(InputFocus)
		{
			case NAME :
				if (ASCII == 13)
				{
					EditName->DeleteLastChar();
				}
				else
				{
					EditName->AddChar(ASCII);
				}
			break;
			case AUTHOR :
				if (ASCII == 13)
				{
					EditAuthor->DeleteLastChar();
				}
				else
				{
					EditAuthor->AddChar(ASCII);
				}
			break;
			case FILE :
				if (ASCII == 13)
				{
					EditFile->DeleteLastChar();
				}
				else
				{
					EditFile->AddChar(ASCII);
				}
			break;
			case OK :
				if (ASCII == '\n')
				{
					OnOk();
				}
			break;
			case CANCEL :
				if (ASCII == '\n')
				{
					OnCancel();
				}
			break;
		}
	}
}

void
cSAVEMAPDIALOG::MouseClick(tSINT32 MouseX, tSINT32 MouseY, tUINT32 Click)
{
	// all left clicks
	if (Click & cMOUSE::Button1)
	{
		// Map Name field
		if ((MouseX >= 22 + PositionX) && (MouseX <= 484 + PositionX) && 
			(MouseY >= 76 + PositionY) && (MouseY <= 96  + PositionY))
		{
			ChangeInputFocus(NAME);
		}
		// Author field
		else if ((MouseX >= 22  + PositionX) && (MouseX <= 484 + PositionX) && 
			     (MouseY >= 122 + PositionY) && (MouseY <= 142 + PositionY))
		{
			ChangeInputFocus(AUTHOR);
		}
		// FileName field
		else if ((MouseX >= 22  + PositionX) && (MouseX <= 484 + PositionX) && 
			     (MouseY >= 166 + PositionY) && (MouseY <= 186 + PositionY))
		{
			ChangeInputFocus(FILE);	
		}
		// OK Button
		else if ((MouseX >= 352 + PositionX) && (MouseX <= 408 + PositionX) && 
			     (MouseY >= 196 + PositionY) && (MouseY <= 220 + PositionY))
		{
			OnOk();
		}
		// Cancel Button
		else if ((MouseX >= 412 + PositionX) && (MouseX <= 468 + PositionX) && 
			     (MouseY >= 196 + PositionY) && (MouseY <= 220 + PositionY))
		{
			OnCancel();
		}
	}
}

void
cSAVEMAPDIALOG::OnOk()
{
	if(NameChanged)
	{
		ReturnData->MapName.assign(EditName->GetString(), 64);
	}
	if(AuthorChanged)
	{
		ReturnData->Author.assign(EditAuthor->GetString(), 64);
	}
	ReturnData->FileName.append(EditFile->GetString());
	if (ReturnData->FileName.find(".map") == -1)
	{
		ReturnData->FileName.append(".map");
	}
	
*LOG << "try to open" << ReturnData->FileName << "\n";
	std::ofstream TestStream;
	TestStream.open(ReturnData->FileName.c_str(), std::ios_base::binary);
	if (TestStream.is_open())
	{
*LOG << "TestStream opened succesfully \n";
		DialogActive = FALSE;
		TestStream.close();
*LOG << "TestStream closed \n";
	}
	else // invalid outputfile
	{
*LOG << "Opening of TestStream failed \n";
		ChangeInputFocus(FILE);
	}
}

void
cSAVEMAPDIALOG::OnCancel()
{
	SAFEDELETE(ReturnData);
	DialogActive = FALSE;
}

void
cSAVEMAPDIALOG::ChangeInputFocus(tFOCUS NewFocus)
{
	if (NewFocus == InputFocus)
		return;

	tUINT16 Green = PixelConverter->RGBToColor(0, 69, 0);

	switch (InputFocus)
	{
		case NAME :
			EditName->DeleteCursor();
			if(NewFocus == NONE)
				NewFocus = AUTHOR;
		break;
		case AUTHOR :
			EditAuthor->DeleteCursor();
			if(NewFocus == NONE)
				NewFocus = FILE;
		break;
		case FILE :
			EditFile->DeleteCursor();
			if (NewFocus == NONE)
				NewFocus = OK;
		break;
		case OK :
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect(352, 197, 409, 222, Green);
			Painter->EndDrawing();
			if (NewFocus == NONE)
				NewFocus = CANCEL;
		break;
		case CANCEL :
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect(413, 197, 468, 222, Green);
			Painter->EndDrawing();
			if (NewFocus == NONE)
				NewFocus = NAME;
		break;
	}

	switch (NewFocus)
	{
		case NAME :
			InputFocus = NAME;
			if(!NameChanged)
			{
				EditName->Reset();
				NameChanged = TRUE;
			}
			EditName->SetCursor();
		break;
		case AUTHOR :
			InputFocus = AUTHOR;
			if(!AuthorChanged)
			{
				EditAuthor->Reset();
				AuthorChanged = TRUE;
			}
			EditAuthor->SetCursor();
		break;
		case FILE :
			InputFocus = FILE;
			if(!FileChanged)
			{
				EditFile->Reset();
				FileChanged = TRUE;
			}
			EditFile->SetCursor();
		break;
		case OK :
			InputFocus = OK;
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect(352, 197, 409, 222, 0);
			Painter->EndDrawing();
		break;
		case CANCEL :
			InputFocus = CANCEL;
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect(413, 197, 468, 222, 0);
			Painter->EndDrawing();
		break;
	}
}

cRETURNDATABASE*
cSAVEMAPDIALOG::GetDataPackage()
{
	return ReturnData;
}