#ifndef __DIALOG_HPP__
#define __DIALOG_HPP__

/*********************************************************************
		HawkCreek software

		Header for all Dialog classes

		History of changes:
		- File created by Capella 03/30/2000
***********************************************************************/
#ifndef __TYPE_HPP__
#include "general\type.hpp"
#endif

#include <string>

class cSURFACE;
class cWRITE;
class cPAINT;
class cPIXELCONVERTER;

class cEDITBOX
{
private:
	//members
	tUINT32 Left;
	tUINT32 Top;
	tUINT32 Length;
	tUINT32 Position;
	cPAINT* EditPainter;
	tUINT16 BackgroundColor;
	char*	String;

public:
	cEDITBOX(tUINT32 LeftPos, 
			 tUINT32 TopPos, 
			 tUINT32 MaxLength,
			 cPAINT* DialogPainter,
			 tUINT16 Color);
	~cEDITBOX();
	void AddChar(char ASCII);
	void DeleteLastChar();
	char* GetString() {return String;};
	void Reset();
	void Init(const char* InitString);
	void SetCursor();
	void DeleteCursor();
};

class cRETURNDATABASE
{
public :
	virtual ~cRETURNDATABASE() {};
};

class cDIALOG
{
protected:
	//members
	char* FileName;
    cSURFACE* BmpSurface;
	tUINT32 PositionX, PositionY;
	tUINT32 Width, Height;
	tBOOL DialogActive;
	//tUINT8 ReturnData;
	cRETURNDATABASE* ReturnData;
	cPIXELCONVERTER* PixelConverter;
	
	//methods
	void Initialize();

public:
	cDIALOG();
	cDIALOG( char* BmpFileName, tUINT32 PosX, tUINT32 PosY);
	virtual ~cDIALOG();

	enum {FINISH = 0, ACTIVE = 1};

	//methods
    inline cSURFACE* GetSurface() {return BmpSurface;};
	inline void GetPosition(tUINT32* PosX, tUINT32* PosY) {*PosX = PositionX; *PosY = PositionY;};
	inline void GetSize(tUINT32* SizeX, tUINT32* SizeY) {*SizeX = Width; *SizeY = Height;};

    void Show(cSURFACE* BlitSurface);
	tBOOL SetPosition( tUINT32 PosX, tUINT32 PosY);
	tUINT32 GetState();
	virtual void MouseClick ( tSINT32 MouseX, tSINT32 MouseY, tUINT32 Click);
	virtual void CharMessage ( char ASCII );
	virtual cRETURNDATABASE* GetDataPackage();
};


/*
 *	input dialog for the creation of new map, shall get name, author and
 *  dimensions from the User
 */
class cNEWMAPDIALOG : public cDIALOG
{
private:
	//members
	cEDITBOX* EditName;
	cEDITBOX* EditAuthor;
	cEDITBOX* EditWidth;
	cEDITBOX* EditHeight;
	cPAINT*   Painter;
	tBOOL	  WidthChanged;
	tBOOL	  HeightChanged;
	
	enum tFOCUS 
	{ 
		NONE, 
		NAME, 
		AUTHOR, 
		WIDTH, 
		HEIGHT, 
		OK, 
		CANCEL 
	} InputFocus;
	
	void ChangeInputFocus( tFOCUS NewFocus = NONE );
	void OnOk();
	void OnCancel();

public:
	cNEWMAPDIALOG();
	~cNEWMAPDIALOG();

	class cRETURNDATA : public cRETURNDATABASE
	{
	public :
		char* Name;
		char* Author;
		tUINT32 Width;
		tUINT32 Height;
	};
	
	//methods
	void CharMessage(char ASCII);	//overwrites cDIALOG method (I hope!)
	cRETURNDATABASE* GetDataPackage();
	void MouseClick(tSINT32 MouseX, tSINT32 MouseY, tUINT32 Click);

private:
	cNEWMAPDIALOG::cRETURNDATA* ReturnData;
};

class cLOADMAPDIALOG : public cDIALOG
{
private:
	cEDITBOX* EditFileName;
	cPAINT* Painter;
	
	enum tFOCUS 
	{
		NONE,
		FILE,
		OK,
		CANCEL
	} InputFocus;

	void ChangeInputFocus (tFOCUS NewFocus = NONE);
	void OnOk();
	void OnCancel();

public:
	cLOADMAPDIALOG();
	~cLOADMAPDIALOG();

	class cRETURNDATA : public cRETURNDATABASE
	{
	public :
		std::string FileName;
	};
	
	//methods
	void CharMessage(char ASCII);	
	cRETURNDATABASE* GetDataPackage();
	void MouseClick(tSINT32 MouseX, tSINT32 MouseY, tUINT32 Click);

private :
	cRETURNDATA* ReturnData;

};


class cSAVEMAPDIALOG : public cDIALOG
{
private :
	cEDITBOX* EditName;
	cEDITBOX* EditAuthor;
	cEDITBOX* EditFile;
	cPAINT*   Painter;
	tBOOL	  NameChanged;
	tBOOL	  AuthorChanged;
	tBOOL	  FileChanged;
	enum tFOCUS
	{
		NONE,
		NAME, 
		AUTHOR, 
		FILE,
		OK,
		CANCEL
	} InputFocus;

	void ChangeInputFocus(tFOCUS NewFocus = NONE);
	void OnOk();
	void OnCancel();

public :
	cSAVEMAPDIALOG(const char* MapName, const char* Author, std::string FileName);
	~cSAVEMAPDIALOG();

	class cRETURNDATA : public cRETURNDATABASE
	{
	public :
		std::string MapName;
		std::string Author;
		std::string FileName;
	};

	//methods
	void CharMessage(char ASCII);	
	cRETURNDATABASE* GetDataPackage();
	void MouseClick(tSINT32 MouseX, tSINT32 MouseY, tUINT32 Click);

private :
	cRETURNDATA* ReturnData;
};

extern cDIALOG* Dialog;


#endif
