#include "dxdraw.hpp"

/******************************************************************************
    HawkCreek software
    Notes:

    History of Changes:
    - 27.02.1999: file created by BuschnicK
    - 28.02.1999: use the Singleton Pattern here to make sure only one instance
                  of this object can ever be created? (see "Design Patterns")
    - 01.03.1999: rename this to 'dxdraw.hpp'? Cause it is actually only DDraw
                  and not DX...
	- 19.03.2000: added ability to run windowed/non-exclusive
    - 05.03.1999: renamed files to 'dxdraw.*', added 'GetDirectDrawInterface()'
	- 01.04.2000: added global DirectDraw Object, very dangerous though, cause
				  it has to be initialized and destroyed somewhere else and many
                  other modules (cSURFACE, cCACHE etc) depend on it
    - 11.06.1999: made 'GetDirectDrawInterface()' an inline function

******************************************************************************/

#include "general\debug.hpp"
#include "general\helpmacros.hpp"
#include <ddraw.h>

cDXDRAW *DirectDraw = NULL;

/*
 * initializes the default 800*600*16bit Resolution, either windowed or fullscreen mode
 */
cDXDRAW::cDXDRAW( const HWND &GameWindow, tBOOL FullScreen )
{
    Initialize( GameWindow, 800, 600, 16, FullScreen );
}

/*
 * either initializes to the default resolution and bitdepth in fullscreen mode or 
 * to the mode described by the supplied parameters
 */
cDXDRAW::cDXDRAW( 
	const HWND &GameWindow, 
	tUINT32 XResolution = 800, 
	tUINT32 YResolution = 600, 
	tUINT32 BitDepth	= 16, 
	tBOOL FullScreen	= TRUE )
{
    Initialize( GameWindow, XResolution, YResolution, BitDepth, FullScreen );
}

/*
 *
 */
cDXDRAW::~cDXDRAW()
{
    RELEASE( DirectDraw );
}

/*
 * provate function, only called from the constructors
 */
void
cDXDRAW::Initialize(
	const HWND &GameWindow,
	tUINT32 XResolution,
	tUINT32 YResolution,
	tUINT32 BitDepth,
	tBOOL FullScreen )
{
    HRESULT ReturnValue = DD_OK;

	this->FullScreen = FullScreen;
	this->Window = GameWindow;

	ReturnValue = DirectDrawCreateEx( NULL, ( void ** ) &DirectDraw, IID_IDirectDraw7, NULL);
	ASSERT( ReturnValue == DD_OK );

	if ( FullScreen )
	{
	    ReturnValue = DirectDraw->SetCooperativeLevel( GameWindow, DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN );
		ASSERT( ReturnValue == DD_OK );

		ReturnValue = DirectDraw->SetDisplayMode( (DWORD) XResolution, (DWORD) YResolution, (DWORD) BitDepth, 0, 0 );
		ASSERT( ReturnValue == DD_OK );
	}
	else
	{
        ReturnValue = DirectDraw->SetCooperativeLevel( GameWindow, DDSCL_NORMAL );
        ASSERT( ReturnValue == DD_OK );
	}
}

/*
 * returns the total videomemory on the graphics card
 */
tUINT32
cDXDRAW::GetTotalVideoMemory( void )
{
    ASSERT( DirectDraw != NULL );

    DDCAPS DeviceCaps;

    memset( &DeviceCaps, 0, sizeof( DeviceCaps ));
    DeviceCaps.dwSize = sizeof( DeviceCaps );

    HRESULT ReturnValue = DD_OK;
    ReturnValue = DirectDraw->GetCaps( &DeviceCaps, NULL );
    ASSERT( ReturnValue == DD_OK );

    return ( DeviceCaps.dwVidMemTotal );
}

/*
 * returns the total available videomemory on the graphics card
 */
tUINT32
cDXDRAW::GetFreeVideoMemory( void )
{
    ASSERT( DirectDraw != NULL );

    DDCAPS DeviceCaps;

    memset( &DeviceCaps, 0, sizeof( DeviceCaps ));
    DeviceCaps.dwSize = sizeof( DeviceCaps );

    HRESULT ReturnValue = DD_OK;
    ReturnValue = DirectDraw->GetCaps( &DeviceCaps, NULL );
    ASSERT( ReturnValue == DD_OK );

    return ( DeviceCaps.dwVidMemFree );
}
