#ifndef __GAME_HPP__
#define __GAME_HPP__

/******************************************************************************
    HawkCreek software

    Purpose:


    !!! the ratio of message system maintenance code to real code is too high
	can I have a function in cEVENT that returns the correctly cast type?
	i.e. cEVENT->GetEvent( cMOUSEMOVEEVENT ) returns the correct child class or 
	NULL if it can't be cast? what would the return value have to be without
	requiring another cast? are templates usefull here?
******************************************************************************/

#ifndef __TYPE_HPP__
#include "general\type.hpp"
#endif

#ifndef __VECTOR_HPP__
#include "general\vector.hpp"
#endif

#include <vector>
#include <list>

#ifndef __WINDOWS_H__
#define __WINDOWS_H__
  #ifndef WIN32_LEAN_AND_MEAN // !! reset this after including windows.h?
  #define WIN32_LEAN_AND_MEAN
  #endif
#include <windows.h> // !! nur fr RECT ?!
#endif

class cSURFACE;
class cTILESET;
class cDXDRAW;
class cSPRITE;
class cCACHE;
class cMAP;
class cPAINT;
class cINPUT;
class cKEYBOARD;
class cMOUSE;
class cBLACKFISH;
class cMARINESPRITE;

#include "general\debug.hpp"

class
cEVENT
{
private:
	tBOOL IsPersistant;
public:
	cEVENT( tBOOL IsPersistant = FALSE ) : IsPersistant( IsPersistant ) { };
	virtual ~cEVENT( ) { };	// must be virtual so class is polymorph and dynamic_cast can be used

	tBOOL Persistant( void ) { return IsPersistant; };
};

class
cMOUSECLICKEVENT : public cEVENT
{
public:
	tSINT32 PositionX;
	tSINT32 PositionY;
	tSINT32 MouseWheel;
	tUINT32 Buttons;
	tUINT32 LastClick;

	cMOUSECLICKEVENT( 
		tSINT32 PositionX, 
		tSINT32 PositionY, 
		tSINT32 MouseWheel, 
		tUINT32 Buttons,
		tUINT32 LastClick )
		: cEVENT( FALSE ),
		  PositionX( PositionX ),
		  PositionY( PositionY ),
		  MouseWheel( MouseWheel ),
		  Buttons( Buttons ),
		  LastClick( LastClick ) 
	{ };
};

class 
cMOUSEMOVEEVENT : public cEVENT
{
public:
	tSINT32 PositionX;
	tSINT32 PositionY;

	cMOUSEMOVEEVENT(
		tSINT32 PositionX,
		tSINT32 PositionY )
		: cEVENT( FALSE ),
		  PositionX( PositionX ),
		  PositionY( PositionY )
	{ };
};

// key events, trivial as classes so this style of compressed writing is hopefully ok...
// actual key hits (presses and released)
class cKEY_ESC_EVENT		: public cEVENT {  };

//////////////////////////////////////////////

class
cEVENTHANDLER
{
private:
	cEVENTHANDLER *NextHandler;
public:
	cEVENTHANDLER( cEVENTHANDLER *NextHandler );
	virtual ~cEVENTHANDLER( );

	virtual void HandleEvent( cEVENT *Event );
};

class 
cMAPEVENTHANDLER : public cEVENTHANDLER
{
private:
	cBLACKFISH *BlackFish;
	cMARINESPRITE *SelectedMarine;
	tUINT32 OldX, OldY;
public:
	cMAPEVENTHANDLER( cBLACKFISH *BlackFish, cEVENTHANDLER *NextHandler );

	void HandleEvent( cEVENT *Event );
};

class
cMAINKEYEVENTHANDLER : public cEVENTHANDLER
{
private:
	HWND Window;
public:
	cMAINKEYEVENTHANDLER( HWND Window, cEVENTHANDLER *NextHandler );

	void HandleEvent( cEVENT *Event );

};
//////////////////////////////////////////////

class
cGAME
{
private:
	tBOOL ApplicationActive;
	cSURFACE	*Primary;
	cSURFACE	*BackBuffer;
	cSURFACE	*RenderTarget;

	cTILESET    *Font1;
	cTILESET    *MouseCursor;
	
	cCACHE      *Cache16x16;
	cCACHE      *Cache64x32;
	cCACHE      *Cache64x128;
	
	cPAINT      *Painter;

	cINPUT      *Input;
	cKEYBOARD	*KeyBoard;
	cMOUSE      *Mouse;

	cBLACKFISH  *BlackFish;

	cEVENTHANDLER			*RootEventHandler;
	cMAPEVENTHANDLER		*MapEventHandler;
	cMAINKEYEVENTHANDLER	*MainKeyEventHandler;

	std::vector<cSPRITE *> Marine;

	void ShowDebugInfo( void );

public:
	cGAME( HWND WindowHandle, HINSTANCE Instance, tBOOL FullScreen );
	~cGAME( );

	void Run( void );

	void RetrieveFocus( void );
};

#endif