#include "PixelConverter.hpp"

/******************************************************************************
    HawkCreek software
    Notes:
    History of Changes:
    - 24.07.1999: file created by BuschnicK
******************************************************************************/

#include "general\debug.hpp"
#include "general\helpmacros.hpp"
#include "BlackFish\Surface.hpp"
#include <ddraw.h>

/*
 * initialize the pixelformat, extract the format from a valid surface
 * which has already been created in this format, i.e. the primary
 */
cPIXELCONVERTER::cPIXELCONVERTER( cSURFACE *Surface )
{
    ASSERT( Surface != NULL );

    HRESULT ReturnValue = DD_OK;

    DDPIXELFORMAT PixelFormat;
    ZeroMemory( &PixelFormat, sizeof( PixelFormat ) );
    PixelFormat.dwSize = sizeof( PixelFormat );

    IDirectDrawSurface7 *DXSurface = Surface->GetDXSurface();
    ReturnValue = DXSurface->GetPixelFormat( &PixelFormat );
    ASSERT( ReturnValue == DD_OK );

    ASSERT( PixelFormat.dwFlags == DDPF_RGB );
    ASSERT( PixelFormat.dwRGBBitCount == 16 );

    ProcessBits( PixelFormat.dwRBitMask, &RBitPos, &RBitCount );
    ProcessBits( PixelFormat.dwGBitMask, &GBitPos, &GBitCount );
    ProcessBits( PixelFormat.dwBBitMask, &BBitPos, &BBitCount );
}

/*
 */
cPIXELCONVERTER::~cPIXELCONVERTER( )
{
}

/*
 * Given three 8bit RGB values compose the 16bit color value
 */
tUINT16
cPIXELCONVERTER::RGBToColor( tUINT8 Red, tUINT8 Green, tUINT8 Blue )
{
    tUINT16 RedValue   = (tUINT16)(( Red   * ( 1 << RBitCount )) >> 8);// = / 256
    tUINT16 GreenValue = (tUINT16)(( Green * ( 1 << GBitCount )) >> 8);
    tUINT16 BlueValue  = (tUINT16)(( Blue  * ( 1 << BBitCount )) >> 8);

    RedValue   <<= RBitPos;
    GreenValue <<= GBitPos;
    BlueValue  <<= BBitPos;

    return ((tUINT16)(RedValue | GreenValue | BlueValue ));
}

/*
 * private Funktion zum ermitteln der Bits-pro-Pixel und Bitpositionen
 * liefert die Anzahl ('NumBits') gesetzter Bits und die Position des ersten
 * signifikanten Bits ('LowBit') in 'Mask'. Mask mu aus Null-Bits und einem
 * zusammenhngenden Block gesetzter Bits bestehen.
 */
void
cPIXELCONVERTER::ProcessBits( tUINT32 Mask, tUINT16 *LowBit, tUINT16 *NumBits )
{
    tUINT32 TestMask = 1;

    for ( *LowBit = 0; *LowBit < 32; ( *LowBit )++ )
    {
        if ( Mask & TestMask )
            break;
        TestMask <<= 1;
    }

    TestMask <<= 1;

    for ( *NumBits = 1; *NumBits < 32; ( *NumBits )++ )
    {
        if( !( Mask & TestMask ))
            break;
        TestMask <<= 1;
    }
}
