#include "sprite.hpp"

/******************************************************************************
    HawkCreek software
    Purpose:

    History of Changes:
    - 12.10.1999: file created by BuschnicK
    - 25.10.1999: this file doesn't do anything, yet - I'm planning a complete
      rewrite/restructure
	- 08.10.2000: this is da remix, yo! ;)
 ******************************************************************************/

#include "general\debug.hpp"
#include "general\helpmacros.hpp"
#include "inifile\inifile.hpp"
#include "BlackFish\time.hpp"
#include "map\map.hpp"

/*
 * constructor, registers the sprite with the appropriate map cell
 */
cSPRITE::cSPRITE(
	cMAP *Map,
	tUINT32 TileSet,
	cVECTOR Position )
{
	ASSERT( Map != NULL );
	this->Map		= Map;
	this->TileSet	= TileSet;
	this->Position.Set( Position );
	Map->InsertSprite( &this->Position, this );
}

/*
 *
 */
cSPRITE::~cSPRITE( )
{
	Map->RemoveSprite( &this->Position, this );
}

/*
 * return the sprite's tileset
 */
tUINT32
cSPRITE::GetTileSet( )
{
	return TileSet;
}

/*
 * returns the current frame's tile nr, base class always returns 0 since it's 
 * not animated
 */
tUINT32
cSPRITE::GetTileNr( )
{
	return 0; // base class type is not animated, always assumes to be tilesetnr, tile 0
}

/*
 * change the sprite's position, registers the new position with the associated map
 */
tBOOL
cSPRITE::SetPosition( cVECTOR Position )
{
	tBOOL Moved = FALSE;

	tUINT32 NewX = Position.x / 32;
	tUINT32 NewY = Position.y / 32;

	if ( Map->IsValidCell( NewX, NewY ) 
		&& (Position.x >= 0.0) && (Position.y >= 0.0))	// !! find a better way, only necessary
	{													// because negative numbers are truncated to 0
		Moved = TRUE;
		if (( NewX != (tUINT32)(this->Position.x / 32)) 
		  ||( NewY != (tUINT32)(this->Position.y / 32)))
		{
			Map->RemoveSprite( &(this->Position), this );
			this->Position.Set( Position );
			Map->InsertSprite( &(this->Position), this );
		}
		else
		{
			this->Position.Set( Position );
		}			
	}
	return Moved;
}

/*
 * return sprite position in world-coordinates
 */
void
cSPRITE::GetPosition( cVECTOR *Position )
{
	ASSERT( Position != NULL );
	Position->Set( this->Position );
}

/*
 * perform AI, base class does nothing (override)
 */
void
cSPRITE::Think( )
{
}

/**********************************************************************************/

/*
 *
 */
cMARINESPRITE::cMARINESPRITE( cMAP *Map, tUINT32 TileSet, cVECTOR Position )
	: cSPRITE( Map, TileSet, Position ), 
	  LastAnimTime( Timer.GetTime( )),
	  LastFrameTime( Timer.GetTime( )), 
	  FrameTime( Timer.GetTime( )),
	  FrameDelta( 0 ),
	  TileNr( 0 ),  
	  Direction( EAST ), 
	  Speed( 0.0, 0.0, 0.0 ) 
{
//	SequenceLength	= 8;	// walk cycles
//	TileNrBase		= (Direction - 1) * SequenceLength;
	SequenceLength = 1;
	TileNrBase = 64;
}

/*
 *
 */
void
cMARINESPRITE::Move( cVECTOR To )
{
	if ( Path.empty( )) // don't accept a new path while still on one
		Map->FindPath( Position, To, &Path );
}

/*
 * !! this function will get long -> split
 * !! do something against those sharp turn animations, i.e. a turn from N to SE should
 *    display the NE, E animations in between
 */
void 
cMARINESPRITE::Think( )
{
	FrameTime		= Timer.GetTime( );	// in ticks
	FrameDelta		= ( FrameTime - LastFrameTime ) / Timer.GetFrequency( ); // in seconds
	LastFrameTime	= FrameTime;		// in ticks

	if ( !Path.empty( ))		// we haven't arrived, yet
	{
		if ( !Position.Equals( Path.front( ))) // we are between two map cells
		{
			Speed.Normalize( );		// !! don't renormalize each frame (sqrt()!) but save the unit
									//    vector add a multiply of that to Position (->modify vector class)
			Speed.Multiply( FrameDelta * UnitsPerSecond );

			cVECTOR NewPos;
			NewPos.Set( Position );

			cVECTOR Distance;
			Distance.Set( Path.front( ));  // instead of calculating Position from scratch each frame 
			Distance.Subtract( Position ); // I could save it and just subtract Speed each frame

			if ( Speed.GetLength( ) > Distance.GetLength( ))	// make sure we don't overshoot our goal
				NewPos.Add( Distance );
			else
				NewPos.Add( Speed );

			SetPosition( NewPos );
		}
		else	// we are at the goal position
		{			
			cVECTOR NewPos;
			NewPos.Set( Path.front( ));
			SetPosition( NewPos );	// set our position to the goal position (to prevent 
									// floating point errors from accumulating)
			Path.pop_front( );
			if ( !Path.empty( ))	// steer towards our new goal if any
			{
				Speed.Set( Path.front( ));
				Speed.Subtract( Position );

				if ( Speed.x < 0.0 )
					if ( Speed.y < 0.0 )
						Direction = WEST;
					else if ( Speed.y > 0.0 )
						Direction = SOUTH;
					else
						Direction = SOUTHWEST;
				else if ( Speed.x > 0.0 )
					if ( Speed.y < 0.0 )
						Direction = NORTH;
					else if ( Speed.y > 0.0 )
						Direction = EAST;
					else
						Direction = NORTHEAST;
				else 
					if ( Speed.y < 0.0 )
						Direction = NORTHWEST;
					else
						Direction = SOUTHEAST;

				SequenceLength  = 8;	// 8 walk frames in each direction
				TileNrBase		= (Direction - 1) * SequenceLength;
			}
		}
	}
	else // no goal, use stop and stand animations...
	{
		TileNr = 0;
		SequenceLength = 1;
		TileNrBase = 63 + Direction;
	}	
}

/*
 *
 */
tUINT32
cMARINESPRITE::GetTileNr( )
{
	tDOUBLE AnimDelta = ( FrameTime - LastAnimTime ) / Timer.GetFrequency( );
	if ( AnimDelta >= ( 1.0 / Frequency ))
	{
		TileNr += AnimDelta * Frequency;
		TileNr %= SequenceLength;
		LastAnimTime = FrameTime;
	}

	return TileNr + TileNrBase;
}
