#include "Surface.hpp"

/******************************************************************************
    HawkCreek software
    Notes:

    History of Changes:
    - 05.03.1999: file created by BuschnicK
    - 09.03.1999: brauchen wir ein 'Create', das die surface aus einem Bitmap
                  erstellt?
    - 14.03.1999: pitch and surfacemem are now refreshed by every 'lock()'
    - 25.03.1999: !! find out correct way to pass parameters ( const int &Hello ?), Pointers?
    - 13.06.1999: split the pixel format convert functions into a seperate class cPIXELCONVERTER
    - 23.07.1999: Trennung von cSURFACE in cSURFACE, cPAINT und cPIXELCONVERTER
    - 02.08.1999: write a constructor for each create (just for convenience)
    - 27.09.1999: added the two cache accessors, see notes about possible problems with that
    - 28.09.1999: added Getsize to return the surface size as a rect
    - 02.11.1999: added Clip()
    - 05.11.1999: added a DEBUGCODE safeguard, every blit is clipped in debugmode
                  and a message is logged if it completly clips a tile.
                  This is just to prevent crashes during debugging since blits are
                  unclipped in release mode and every tile that requires clipping
                  should be clipped before blitting
                  Things that still crash the program and fail an assertion:
                    - passing null surface pointers to blit()
                    - passing rects wich don't obey the rules:
                        left < right, top < bottom
    - 06.11.1999: added RestoreAllIfLost() to restore lost surfaces
	- 19.03.2000: added ability to run in windowed/non-exclusive mode
				  ( primary surface constructor changed )
******************************************************************************/

#include "general\helpmacros.hpp"
#include "BlackFish\dxdraw.hpp"
#include <ddraw.h>

/*
 * Constructor, initialisiert alle Variablen auf ihre Default Werte,
 * die class ist in diesem Zustand nicht zu gebrauchen, 'Create' mu
 * aufgerufen werden
 */
cSURFACE::cSURFACE( )
	: Surface( NULL ),                  // surface doesn't exist, yet - call create()
      SurfaceMem( NULL ),               // } call lock() to set these values
      Pitch( 0 ),                       // }
      ImCachedHere( NULL ),             // surface is not yet cached
      Width( 0 ),
      Height( 0 ),
	  Locked( FALSE )
{
    DEBUGCODE
    (
		SetRect( &ScreenRect, 0, 0, 800, 600 );
    )
}

/*
 * Destructor, releases all allocated memory, i.e. the Surface
 */
cSURFACE::~cSURFACE( )
{
	ASSERT( !Locked ); // don't destroy locked surfaces

    if ( Surface != NULL )
        RELEASE( Surface );
}

/*
 * Erstellt die primary Surface mit einem Backbuffer. Die Gre und Farbtiefe
 * der Surface entspricht der aktuellen Bildschirmeinstellung
 * - DirectDraw mu ein gltiges DirectDraw Objekt sein
 * - die Auflsung mu bereits mit 'DirectDraw->SetMode' gesetzt worden sein
 * - es sollte nur eine Primary Surface im gesamten Programm geben
 *      !! use a design pattern for that requirement?
 *      !! use a design pattern that the user can only call one Create function
 *         once for every instance of this class
 * - die backbuffer-surface erhlt man, indem man in einem zweiten cSURFACE
 *   Objekt 'Create( PrimarySurface )' aufruft
 * - !! untested: both surfaces are created in video-memory if possible, else ???
 */
void
cSURFACE::Create( void )
{
    ASSERT( DirectDraw != NULL );
    ASSERT( Surface == NULL );  // make sure surface hasn't been created, yet

    HRESULT ReturnValue = DD_OK;

    DDSURFACEDESC2 SurfaceDescriptor;
    ZeroMemory( &SurfaceDescriptor, sizeof( SurfaceDescriptor ) );

	if ( DirectDraw->IsFullScreen( ))
	{
*LOG << "cSURFACE::Create() \n ...in fullscreen mode.\n";
	    SurfaceDescriptor.dwSize            = sizeof( SurfaceDescriptor );
		SurfaceDescriptor.dwFlags           = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
		SurfaceDescriptor.ddsCaps.dwCaps    = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP | DDSCAPS_COMPLEX;
		SurfaceDescriptor.dwBackBufferCount = 1;

		ReturnValue = DirectDraw->GetDirectDrawInterface()->CreateSurface( &SurfaceDescriptor, &Surface, NULL);
		ASSERT( ReturnValue == DD_OK );

	    ZeroMemory( &SurfaceDescriptor, sizeof( SurfaceDescriptor ) );
	    SurfaceDescriptor.dwSize            = sizeof( SurfaceDescriptor );
	}
	else
	{
*LOG << "cSURFACE::Create() \n ...in windowed mode.\n";
		SurfaceDescriptor.dwSize            = sizeof( SurfaceDescriptor );
		SurfaceDescriptor.dwFlags           = DDSD_CAPS;
		SurfaceDescriptor.ddsCaps.dwCaps    = DDSCAPS_PRIMARYSURFACE;

		ReturnValue = DirectDraw->GetDirectDrawInterface()->CreateSurface( &SurfaceDescriptor, &Surface, NULL);
		ASSERT( ReturnValue == DD_OK );

		IDirectDrawClipper *Clipper = NULL;
        ReturnValue = DirectDraw->GetDirectDrawInterface()->CreateClipper( 0, &Clipper, NULL );
		ASSERT( ReturnValue == DD_OK );

        Clipper->SetHWnd( 0, DirectDraw->GetWindow( ));
        Surface->SetClipper( Clipper );
        Clipper->Release( );
        Clipper = NULL;
	}

	ReturnValue = Surface->GetSurfaceDesc( &SurfaceDescriptor );
	ASSERT( ReturnValue == DD_OK );

	SetTransparentColor( 0 );			// default transparent color, so blittrans calls don't crash

	Width	= SurfaceDescriptor.dwWidth;
	Height	= SurfaceDescriptor.dwHeight;
}

/*
 * Erstellt einen pointer auf die Backbuffer surface zu der angegebenen Primary Surface
 * - Die Primary mu gltig sein, i.e. 'Primary->Create( DirectDraw )'
 *   mu vorher aufgerufen worden sein
 */
void
cSURFACE::Create( cSURFACE *Primary )
{
    ASSERT( Primary != NULL );
    ASSERT( Surface == NULL );

    HRESULT ReturnValue = DD_OK;

    DDSCAPS2 SurfaceCaps;
    ZeroMemory( &SurfaceCaps, sizeof( SurfaceCaps ) );

    SurfaceCaps.dwCaps = DDSCAPS_BACKBUFFER;

    ReturnValue = Primary->GetDXSurface()->GetAttachedSurface( &SurfaceCaps, &Surface );
    ASSERT( ReturnValue == DD_OK );

	SetTransparentColor( 0 );			// default transparent color, so blittrans calls don't crash

	Width	= Primary->GetWidth( );
	Height	= Primary->GetHeight( );
}

/*
 * Erstellt eine Surface der angegebenen Ausmae im Systemspeicher
 * !! ist die Farbtiefe wirklich immer = der DirectDraw Tiefe?
 */
void
cSURFACE::Create( tUINT32 Width, tUINT32 Height, tBOOL VideoMemory )
{
    ASSERT( DirectDraw != NULL );
    ASSERT( Surface == NULL );  // make sure surface hasn't been created, yet

    HRESULT ReturnValue = DD_OK;

    DDSURFACEDESC2 SurfaceDescriptor;
    ZeroMemory( &SurfaceDescriptor, sizeof( SurfaceDescriptor ) );

    SurfaceDescriptor.dwSize            = sizeof( SurfaceDescriptor );
    SurfaceDescriptor.dwFlags           = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
    if ( VideoMemory )
        SurfaceDescriptor.ddsCaps.dwCaps    = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_VIDEOMEMORY;
    else
        SurfaceDescriptor.ddsCaps.dwCaps    = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_SYSTEMMEMORY;
    SurfaceDescriptor.dwWidth           = (DWORD) Width;
    SurfaceDescriptor.dwHeight          = (DWORD) Height;

    ReturnValue = DirectDraw->GetDirectDrawInterface()->CreateSurface( &SurfaceDescriptor, &Surface, NULL);
    ASSERT( ReturnValue == DD_OK );

	SetTransparentColor( 0 );			// default transparent color, so blittrans calls don't crash

    this->Width   = Width;
    this->Height  = Height;
}

/*
 * checks whether this surface has been lost (overwritten in videomem, sysmem surfaces can't
 * get lost) and calls DirectDraw->RestoreAllSurfaces( ), which restores not only this, but
 * all directdraw surfaces
 */
void
cSURFACE::RestoreAllIfLost( void )
{
    ASSERT( Surface != NULL );

    if ( Surface->IsLost() == DDERR_SURFACELOST )
    {
        *LOG << "Restoring all surfaces! \n";
        IDirectDraw4 *DirectDraw;
        Surface->GetDDInterface( ( void **)&DirectDraw );
        DirectDraw->RestoreAllSurfaces();
    }
}

/*
 * copy the Region specified by 'SourceRect' from the surface 'Source' onto the
 * calling (this) surface in the Region specified by 'DestRect'
 * - if SourceRect and DestRect are of different sizes, stretching/shrinking is automatically
 *   performed
 * - if SourceRect or DestRect are partially or fully outside of surface bounds, the result
 *   is undefined, i.e. clipping is *not* performed
 */
void
cSURFACE::Blit( cSURFACE *Source, RECT *SourceRect, RECT *DestRect )
{
    ASSERT( Source != NULL );
    ASSERT( Surface != NULL );

    // !! warning, debug may clip the source and dest rects although
    // this might not be desired
    DEBUGCODE
    (
        if (( SourceRect != NULL ) && ( DestRect != NULL ))
            if ( !Clip( ScreenRect, SourceRect, DestRect ))
            {
                *LOG << "cSURFACE::Blit(): DEBUGCODE Clipped a tile out of existance! \n";
                return;
            }
    )

    HRESULT ReturnValue;

    RestoreAllIfLost();

    for ( ;; )
    {
        ReturnValue = Surface->Blt( DestRect, Source->GetDXSurface(), SourceRect, DDBLT_ASYNC, NULL );

        if ( ReturnValue == DD_OK )     // blit() ok, stop trying
            break;

        if ( ReturnValue != DDERR_WASSTILLDRAWING )  // still drawing last frame, keep trying
            break;
    }

    ASSERT( ReturnValue == DD_OK );
}

/*
 * same as 'Blit()', but uses the SrcColorKey, meaning that the specified color is not
 * copied from the source surface, i.e. appears transparent
 */
void
cSURFACE::BlitTrans( cSURFACE *Source, RECT *SourceRect, RECT *DestRect )
{
    ASSERT( Source != NULL );
    ASSERT( Surface != NULL );

    // !! warning, debug may clip the source and dest rects although
    // this might not be desired
    DEBUGCODE
    (
        if (( SourceRect != NULL ) && ( DestRect != NULL ))
            if ( !Clip( ScreenRect, SourceRect, DestRect ))
            {
                *LOG << "cSURFACE::BlitTrans(): DEBUGCODE Clipped a tile out of existance! \n";
                return;
            }
    )

    HRESULT ReturnValue;

    RestoreAllIfLost();

    for ( ;; )
    {
        ReturnValue = Surface->Blt( DestRect, Source->GetDXSurface(), SourceRect,
                                    DDBLT_ASYNC | DDBLT_KEYSRC, NULL );

        if ( ReturnValue == DD_OK )
            break;

        if ( ReturnValue != DDERR_WASSTILLDRAWING )
            break;
    }

    ASSERT( ReturnValue == DD_OK );
}

/*
 * Flips the Primary surface and the Backbuffer
 * - this function must only be called on the Primary Surface, i.e.
 *   on the surface created with 'Create( DirectDraw )', else
 *   strange things will happen
 */
void
cSURFACE::Flip( void )
{
    ASSERT( Surface != NULL );

    HRESULT ReturnValue = DD_OK;

    RestoreAllIfLost();

    for ( ;; )
    {
        ReturnValue = Surface->Flip( NULL, 0 );

        if ( ReturnValue == DD_OK )
            break;

        if ( ReturnValue != DDERR_WASSTILLDRAWING )
            break;
    }

    ASSERT( ReturnValue == DD_OK ); // if it isn't DD_OK we couldn't flip
}

/*
 * specifies the transparent color (source-colorkey)
 * note: pixel format is important here, so if the transparent color is neither
 *		 black or white you should use a cPIXELCONVERTER to calculate the value
 */
void
cSURFACE::SetTransparentColor( tUINT32 Color )
{
    DDCOLORKEY ColorKey;

    ColorKey.dwColorSpaceLowValue   = (DWORD) Color;
    ColorKey.dwColorSpaceHighValue  = ColorKey.dwColorSpaceLowValue;

    HRESULT ReturnValue = DD_OK;
    ReturnValue = Surface->SetColorKey( DDCKEY_SRCBLT, &ColorKey );
    ASSERT( ReturnValue == DD_OK );
}

/*
 * returns the transparent color (source-colorkey)
 */
tUINT32
cSURFACE::GetTransparentColor( void )
{
    DDCOLORKEY ColorKey;

    ZeroMemory( &ColorKey, sizeof ( ColorKey ));

    HRESULT ReturnValue = DD_OK;
    ReturnValue = Surface->GetColorKey( DDCKEY_SRCBLT, &ColorKey );
    ASSERT( ReturnValue == DD_OK );

    ASSERT( ColorKey.dwColorSpaceLowValue == ColorKey.dwColorSpaceHighValue );

    return (tUINT32) ( ColorKey.dwColorSpaceLowValue );
}

/*
 * initializes the Rect parameter with the surface dimensions
 * !! kinda redundant, since it's easy for the caller to do this himself
 */
void
cSURFACE::GetSize( RECT *Rect )
{
    ASSERT( Rect != NULL );

    Rect->top       = 0;
    Rect->left      = 0;
    Rect->bottom    = Height;
    Rect->right     = Width;
}

/*
 * Locks the surface's memory for direct access afterwards
 * - must only be called on systemmory surfaces, i.e. not the primary or backbuffer
 * - you must call 'UnLock()' after locking a surface
 * - only use functions in between a 'Lock()' 'UnLock()' pair that don't attempt
 *   to lock the surface memory themselves
 */
void
cSURFACE::Lock( void )
{
    ASSERT( Surface != NULL );
	ASSERT( !Locked );

	Locked = TRUE;

    HRESULT ReturnValue = DD_OK;
    DDSURFACEDESC2 SurfaceDescriptor;

    SurfaceDescriptor.dwSize = sizeof ( SurfaceDescriptor );

    do
    {
        ReturnValue = Surface->Lock( NULL, &SurfaceDescriptor, 0, NULL );
    }
    while ( ReturnValue == DDERR_WASSTILLDRAWING );

    ASSERT( ReturnValue == DD_OK );

    SurfaceMem = (tUINT16 *) SurfaceDescriptor.lpSurface;
    Pitch = (tUINT32) SurfaceDescriptor.lPitch >> 1;
    ASSERT( SurfaceMem != NULL );
    ASSERT( Pitch != 0 );
}

/*
 * Unlocks the surface memory
 * - 'Lock()' must have been called before
 */
void
cSURFACE::UnLock( void )
{
    ASSERT( Surface != NULL );
	ASSERT( Locked );

	Locked = FALSE;

    HRESULT ReturnValue = DD_OK; // !! DEBUGCODE ?

    ReturnValue = Surface->Unlock( NULL );
    ASSERT( ReturnValue == DD_OK );

    SurfaceMem = NULL; // !! DEBUGCODE this? it's just here for extra safety
    Pitch = 0;

}

/*			non-class-member helper functions		*/	
/*
 * adjust the Source rect so it fits into the ClipRect, return the result in Dest
 * (Source is unaltered)
 * return TRUE if at least part of the rect is still visible
 */
tBOOL Clip( RECT ClipRect, RECT *Source, RECT *Dest )
{
    ASSERT( Source != NULL );
	ASSERT( Dest != NULL );

    // trivially reject if tile is completely out of the visible rect
	if (( Dest->left >= ClipRect.right ) || ( Dest->right <= ClipRect.left ) ||
		( Dest->top >= ClipRect.bottom ) || ( Dest->bottom <= ClipRect.top ))
		return FALSE;

	tSINT32 Difference       = 0;
	if (( Difference = Dest->left - ClipRect.left) < 0 )   // clip left
	{
		Dest->left       = ClipRect.left;
		Source->left    -= Difference;
	}
	else if (( Difference = Dest->right - ClipRect.right ) > 0 )   // clip right
	{
		Dest->right      = ClipRect.right;
		Source->right   -= Difference;
	}
	if (( Difference = Dest->top - ClipRect.top ) < 0 )   // clip top
	{
		Dest->top        = ClipRect.top;
		Source->top     -= Difference;
	}
	else if (( Difference = Dest->bottom - ClipRect.bottom ) > 0 )   // clip bottom
	{
		Dest->bottom     = ClipRect.bottom;
		Source->bottom  -= Difference;
	}
	return TRUE;
}
