#ifndef __SURFACE_HPP__
#define __SURFACE_HPP__

/******************************************************************************
    HawkCreek software

    Purpose:
    provide access to a DirectXSurface, be it in system or in videomemory

******************************************************************************/

#ifndef __DEBUG_HPP__
#include "general\debug.hpp"
#endif

#ifndef __TYPE_HPP__
#include "general\type.hpp"
#endif

#ifndef __WINDOWS_H__
#define __WINDOWS_H__
  #ifndef WIN32_LEAN_AND_MEAN // !! reset this after including windows.h?
  #define WIN32_LEAN_AND_MEAN
  #endif
#include <windows.h> // !! nur fr RECT ?!
#endif

class cDXDRAW;
struct IDirectDrawSurface7;

class
cSURFACE
{
private:

    IDirectDrawSurface7 *Surface;
    tUINT16 *SurfaceMem;
    tUINT32  Pitch;
    void    *ImCachedHere;
    tUINT32  Width;
    tUINT32  Height;
    tBOOL	 Locked;

    DEBUGCODE ( RECT ScreenRect )

public:

    cSURFACE( );
    ~cSURFACE( );

    void Create( void );
    void Create( cSURFACE *Primary );
    void Create( tUINT32 Width, tUINT32 Height, tBOOL VideoMemory );

    void Flip( void );
    void Blit( cSURFACE *Source, RECT *SourceRect, RECT *DestRect );
    void BlitTrans( cSURFACE *Source, RECT *SourceRect, RECT *DestRect );

    void SetTransparentColor( tUINT32 Color ); // !! 16bit Farbtiefe?
    tUINT32 GetTransparentColor( void );

    void RestoreAllIfLost( void );

    void Lock( void );
    void UnLock( void );

    IDirectDrawSurface7 *GetDXSurface( void )	{ return Surface; };
    tUINT16 *GetSurfaceMemPointer( void )		{ return SurfaceMem; };
	tBOOL IsLocked( void ) const				{ return Locked; };
    tUINT32 GetPitch( void ) const					{ return Pitch; };
    tUINT32 GetWidth( void ) const					{ return Width; };
    tUINT32 GetHeight( void ) const					{ return Height; };
    void GetSize( RECT *Rect );
    /* !! the following two accessors create some ugly cyclic dependancy between cSURFACE and cCACHE
     * cSURFACE still doesn't need to include cCACHE, because I use a void* Pointer to hide the dependancy
     * and cast that. The pointer is needed here to quickly identify the requested surface in the cache, i.e.
     * instead of searching the whole cache for the requested (for a blit) surface, the surface itself knows
     * it's already in the cache and where */
    void SetCacheStatus( void *CachePosition )	{ ImCachedHere = CachePosition; };
    void *GetCacheStatus( void )				{ return ImCachedHere; };
};

tBOOL Clip( RECT ClipRect, RECT *Source, RECT *Dest );

#endif

