#include "write.hpp"

/******************************************************************************
    HawkCreek software
    Notes:
    History of Changes:
    - 05.08.1999: file created by BuschnicK
    - !! write more constructors
    - 13.10.1999: removed the cache variable, as it is assumed that the font
      tileset already is associated with the cache
******************************************************************************/

#include "general\helpmacros.hpp"
#include "BlackFish\Surface.hpp"
#include "BlackFish\Paint.hpp"
#include "BlackFish\tileset.hpp"
#include <ddraw.h>

#ifndef __WINDOWS_H__
#define __WINDOWS_H__
  #ifndef WIN32_LEAN_AND_MEAN // !! reset this after including windows.h?
  #define WIN32_LEAN_AND_MEAN
  #endif
#include <windows.h>
#endif

cWRITE *Writer = NULL;

/*
 * constructor, object is unusable in this state, a font has to be selected first
 */
cWRITE::cWRITE( void )
	: Font( NULL ),
	  CharacterWidth( 0 ),
	  CharacterHeight( 0 ),
	  Spacing( 0 ),
	  RenderTarget( NULL )
{
}

/*
 * destructor
 */
cWRITE::~cWRITE( )
{
}

/*
 * the font is a cTILESET (see included example font.ini and font bitmap
 * for the layout and organization)
 */
void
cWRITE::SetFont( cTILESET *NewFont )
{
    ASSERT( NewFont != NULL );

    Font = NewFont;

    CharacterWidth	= Font->GetTileWidth();
    CharacterHeight = Font->GetTileHeight();
}

/*
 * write 'Text' to the 'Destination' surface, top left text corner is at
 * DestX/DestY
 *
 * Note: Output is unclipped, resulting in undefined behaviour if text is
 * written out of the destination surface's bounds
 */
void
cWRITE::Out( tUINT32 DestX, tUINT32 DestY, const char *Text )
{
    ASSERT( Font != NULL );
    ASSERT( RenderTarget != NULL );
    ASSERT( Text != NULL );

    const char *TextPtr = Text;

    RECT DestRect;
	SetRect( &DestRect, DestX, DestY, DestX + CharacterWidth, DestY + CharacterHeight );

    RECT SourceRect;
	SetRect( &SourceRect, 0, 0, CharacterWidth, CharacterHeight );

    while ( *TextPtr != '\0' )
    {
        // the first 32 ASCII characters are unprintable
        Font->BlitTrans( (tUINT32) *TextPtr - 32, &SourceRect, RenderTarget, &DestRect );

        TextPtr++;
        DestRect.left += CharacterWidth;
        DestRect.left += Spacing;
        DestRect.right = DestRect.left + CharacterWidth;
    }
}

/*
 * sets the surface text is drawn to, returns the previous drawing target surface, so
 * callers may restore that after they are done
 */
cSURFACE *
cWRITE::SetDrawingTarget( cSURFACE *Target )
{ 
	ASSERT( Target != NULL ); 
	cSURFACE *ReturnValue = RenderTarget;
	RenderTarget = Target; 
	return ReturnValue;
}

