#include "editfish\EditFish.hpp"
/***********************************************************************************************************
            HawkCreek software

   Notes: Nachkomme von BlackFish mit zustzlicher MenuFunktionalitt

   History of Changes:
   - 9.10.1999 file created by Capella
   - 11.10.1999 first functionality reached, draws floortiles on map
   - 15.10.1999 now also draws walls and overlays, shows complete tile after right click
   - 18.10.1999 map optimizer added
   - 30.10.1999 tiles are now deletable
   - 03.11.1999 save function added, writes map to binary file
   - 17.03.2000 ShowTile updates on left click
   - 19.03.2000 Map functionality now in extra class
   - 24.06.2000 BuschnicK
				search for !!! a delete missing?
				crashes when trying to acces tileset nr 9 -> the one after the character anim (wrap error?)
				modified functions to use the new map interface
				commented out the newmapdialog, since it's nonfunctional at the moment anyways
				8 sign mismatch warnings?
				feature wishlist:
				when modifying a cell in the editor make that the active showtile
				implement 'drawing' = holding down the mouse button to place tiles
				implement delete key to erase walls or tiles
				make an option to just draw on the top layer automatically
				implement a 'place random tile in range x to y'
************************************************************************************************************/

#include "BlackFish\cache.hpp"
#include "BlackFish\tileset.hpp"
#include "BlackFish\write.hpp"
#include "general\debug.hpp"
#include "general\HelpMacros.hpp"
#include "map\Map.hpp"
#include "BlackFish\surface.hpp"
#include "BlackFish\paint.hpp"
#include "BlackFish\time.hpp"
#include "BlackFish\dialog.hpp"
#include "BlackFish\input.hpp"
#include "EditFish\Selection.hpp"
#include "EditFish\MiniMap.hpp"
#include "IniFile\IniFile.hpp"

#define MAXLAYER    4                   // max count of overlays

cEDITFISH::cEDITFISH(cSURFACE *nSystemBuffer,
                     cCACHE *nCache16x16,
                     cCACHE *nCache64x32,
                     cCACHE *nCache64x128,
					 const char *IniFileName)
         : cBLACKFISH( nSystemBuffer, nCache16x16, nCache64x32, nCache64x128, IniFileName)

{
    // here the Tileset that contains the Menubuttons is defined and loaded
    MenuTiles = new cTILESET( "graphics\\edit.ini", "[Edit1]" );
    MenuTiles->SetCache ( Cache64x32 );

	PaintSelection = new cSELECTION();
	LeftButtonDown = FALSE;

	MiniMap = new cMINIMAP();
	MiniMap->Refresh(Map);

    Buttons = new cBUTTONARRAY("graphics\\edit.ini");

    ShowTile = new cWALLINFO[6];  //Tile that is selected with right click and shown on bottom of screen

    tUINT32 i;
    for ( i = 0; i < 6; i++ )
    {
        ShowTile[i].TileSet = 0;
        ShowTile[i].TileNr  = 0;
    }

    ShowTilePosX = 0;
    ShowTilePosY = 0;

    //beim Start erstes GroundTileset anzeigen
	ActiveTilesKind = GROUND;
	ActiveNrOfTileSets = GetNrOfTileSets(ActiveTilesKind);
	CurrentIndexOffset = (NrOfTileSets / 4) * ActiveTilesKind;
	ActiveTilesNr = 1;
    ActiveTiles   = Tiles[CurrentIndexOffset + ActiveTilesNr];
    VisibleTile   = 0;
	WasInSelection = FALSE;
	PaintSelection->Select(VisibleTile);

    //start on floor layer
    WorkingLayerNr = 0;

	//start not in eraser mode
	EraseMode = E_NONE;

	//start with big walls
	SmallWalls = FALSE;
}

cEDITFISH::~cEDITFISH()
{
    SAFEDELETE( Buttons );
    SAFEDELETE( MenuTiles );
	SAFEDELETE( PaintSelection );
	SAFEDELETE( MiniMap );
	SAFEDELETEARRAY( ShowTile );
}

void
cEDITFISH::Menu(void)
{
    RECT Source;
    RECT Dest;

    /*
     * Show buttons and edit frames
     */
    Source.left     = 0;
    Source.right    = Source.left + 64;
    Source.top      = 0;
    Source.bottom   = Source.top + 32;

    MenuTiles->Blit( 0, &Source, SystemBuffer, &(Buttons->GetButton(0)->Rect) );
    MenuTiles->Blit( 1, &Source, SystemBuffer, &(Buttons->GetButton(1)->Rect) );
	//OptimizeButton
    MenuTiles->BlitTrans( 7, &Source, SystemBuffer, &(Buttons->GetButton(2)->Rect) );
	//NewButton
	MenuTiles->BlitTrans( 11, &Source, SystemBuffer, &(Buttons->GetButton(7)->Rect) );
	//LoadButton
	MenuTiles->BlitTrans(9, &Source, SystemBuffer, &(Buttons->GetButton(8)->Rect) );
	//SaveButton
	MenuTiles->BlitTrans(10, &Source, SystemBuffer, &(Buttons->GetButton(9)->Rect) );

    /*
     * show left and right buttons
     * !!not very nicely done, since BlitRectangle much bigger than actual button
     */
    Dest.left       = Buttons->GetButton(3)->Rect.left;
    Dest.right      = Dest.left + 64;
    Dest.top        = Buttons->GetButton(3)->Rect.top;
    Dest.bottom     = Dest.top + 32;

    MenuTiles->BlitTrans( 3, &Source, SystemBuffer, &Dest );
    MenuTiles->BlitTrans( 4, &Source, SystemBuffer, &Dest );

    // should write the number of the active tile set
    char String[4];
    itoa( ActiveTilesNr, String, 10);
	Writer->SetDrawingTarget( SystemBuffer );
    Writer->Out( 736, 4, String );

    /*
     * show layer buttons
     */
    Dest.left       = Buttons->GetButton(5)->Rect.left;
    Dest.right      = Dest.left + 64;
    Dest.top        = Buttons->GetButton(5)->Rect.top;
    Dest.bottom     = Dest.top + 32;

    MenuTiles->BlitTrans( 3, &Source, SystemBuffer, &Dest );
    MenuTiles->BlitTrans( 4, &Source, SystemBuffer, &Dest );

    // should write the number of the active layer
    itoa( WorkingLayerNr, String, 10);
    Writer->Out( 724, 480, "Layer");
    Writer->Out( 736, 500, String );

    tUINT32 Height;
    Height = ActiveTiles->GetTileHeight() ;
    if ( Height == 32 )
    {
        // blits EditFrame for FloorTiles
        MenuTiles->BlitTrans( 2, &Source, SystemBuffer, &(Buttons->GetButton(17)->Rect) );

        tSINT32 i;
        tSINT32 TileCount;
        if ( ActiveTiles->NrOfTiles >= 12)
        {
            for ( i = -6; i <=5 ; i++ )
            {
                TileCount = VisibleTile + i;
                if ( TileCount < 0 )
                {
                    TileCount =  ActiveTiles->NrOfTiles + TileCount;
                }
                else if ( TileCount >= (tSINT32) ActiveTiles->NrOfTiles )
                {
                    TileCount = abs( ActiveTiles->NrOfTiles - TileCount );
                }

                // should blit 12 tiles, 17 is the position of the tile in edit frame (VisibleTile)
                ActiveTiles->BlitTrans( TileCount, &Source, SystemBuffer, &(Buttons->GetButton(17+i)->Rect) );
				Buttons->GetButton(17+i)->Active = TRUE;
		
				// blit green frame for selected tiles
				if((PaintSelection->IsSelected(TileCount))&&(TileCount != (tSINT32) VisibleTile))
				{
					MenuTiles->BlitTrans( 12, &Source, SystemBuffer, &(Buttons->GetButton(17+i)->Rect) );
				}
            }
        }
        else
        {
            for ( i= -6; i <= 5; i++ )
            {
                TileCount = VisibleTile + i;
                if (( TileCount >= 0 ) && (TileCount < (tSINT32) ActiveTiles->NrOfTiles ))
                {
                        ActiveTiles->BlitTrans( TileCount, 
												&Source, 
												SystemBuffer, 
												&(Buttons->GetButton(17+i)->Rect) );
                        Buttons->GetButton(17+i)->Active = TRUE;
							// blit green frame for selected tiles
					if((PaintSelection->IsSelected(TileCount))&&(TileCount != (tSINT32) VisibleTile))
					{
						MenuTiles->BlitTrans( 12, &Source, SystemBuffer, &(Buttons->GetButton(17+i)->Rect) );
					}
                }
                else
                {
                    if ( TileCount < 0 )
                    {
                        if ( ( 12 + TileCount ) < (tSINT32) ActiveTiles->NrOfTiles )
                        {
                            ActiveTiles->BlitTrans( 12 + TileCount, 
													&Source, 
													SystemBuffer, 
													&(Buttons->GetButton(17+i)->Rect) );
                            Buttons->GetButton(17+i)->Active = TRUE;
								// blit green frame for selected tiles
							if(PaintSelection->IsSelected(TileCount + 12))
							{
								MenuTiles->BlitTrans( 12, &Source, SystemBuffer, &(Buttons->GetButton(17+i)->Rect) );
							}
                        }
                        else
                        {
                            Buttons->GetButton(17+i)->Active = FALSE;
                        }
                    }
                    else
                    {
                        if (( TileCount - 12 ) >= 0 )
                        {
                            ActiveTiles->BlitTrans( TileCount - 12, 
													&Source, 
													SystemBuffer, 
													&(Buttons->GetButton(17+i)->Rect));
                            Buttons->GetButton(17+i)->Active = TRUE;
								// blit green frame for selected tiles
							if(PaintSelection->IsSelected(TileCount - 12))
							{
								MenuTiles->BlitTrans( 12, &Source, SystemBuffer, &(Buttons->GetButton(17+i)->Rect) );
							}
                        }
                        else
                        {
                            Buttons->GetButton(17+i)->Active = FALSE;
                        }
                    }
                }
            }
        }
    }
    else
    {
        //show two parts of edit frame for WallTiles
        MenuTiles->BlitTrans( 5, &Source, SystemBuffer, &(Buttons->GetButton(15)->Rect) );
        MenuTiles->BlitTrans( 6, &Source, SystemBuffer, &(Buttons->GetButton(18)->Rect) );

        tSINT32 i;
        tSINT32 TileCount;

        for ( i = -1; i <=1 ; i++ )
        {
            TileCount = VisibleTile + i;
            if ( TileCount < 0 ) 
            {
				if(ActiveTiles->NrOfTiles > 2)
				{
					TileCount = ActiveTiles->NrOfTiles + TileCount;
				}
				else
				{
					TileCount = 3;
				}
            }
			else if ( TileCount >= (tSINT32) ActiveTiles->NrOfTiles )
            {
				if(ActiveTiles->NrOfTiles > 2)
				{
					TileCount = abs( ActiveTiles->NrOfTiles - TileCount );
				}
				else
				{
					TileCount = 3;
				}
            }

            // 15 is the middle of the tile in edit frame (VisibleTile), hence:
            Dest.left    = Buttons->GetButton(17)->Rect.left ;
            Dest.right   = Buttons->GetButton(17)->Rect.right ;
            Dest.top     = Buttons->GetButton(15+(4*i))->Rect.top;
            Dest.bottom  = Buttons->GetButton(18+(4*i))->Rect.bottom ;

            Source.left   = 0 ;
            Source.right  = Source.left + 64;
            Source.top    = 0 ;
            Source.bottom = Source.top + 128;

            // should blit only tiles that really exist
            if(TileCount < ActiveTiles->NrOfTiles)
			{
				ActiveTiles->BlitTrans( TileCount, &Source, SystemBuffer, &Dest );
				Buttons->GetButton(15+(4*i))->Active = TRUE;
				Buttons->GetButton(16+(4*i))->Active = TRUE;
				Buttons->GetButton(17+(4*i))->Active = TRUE;
				Buttons->GetButton(18+(4*i))->Active = TRUE;
			}
			else
			{
				Buttons->GetButton(15+(4*i))->Active = FALSE;
				Buttons->GetButton(16+(4*i))->Active = FALSE;
				Buttons->GetButton(17+(4*i))->Active = FALSE;
				Buttons->GetButton(18+(4*i))->Active = FALSE;
			}
	    }
	}

    /*
     * show all layers of a selected showtile
     */
    tUINT32 i;
    tUINT32 ShowX, ShowY;
    ShowX = 100;
    ShowY = 500;

    Source.left   = 0;
    Source.right  = Source.left + 64;
    Source.top    = 0;
    Source.bottom = Source.top + 32;

    for ( i = 0; i <=5; i++)
    {
        Dest.left   = ShowX;
        Dest.right  = Dest.left + 64;
        Dest.top    = ShowY;
        Dest.bottom = Dest.top + 32;

        Tiles[ShowTile[i].TileSet]->BlitTrans( ShowTile[i].TileNr, &Source, SystemBuffer, &Dest );
        ShowX = ShowX +  64;
    }

	/*
	 *	show eraser
	 */
	MenuTiles->Blit(13, &Source, SystemBuffer, &(Buttons->GetButton(10)->Rect));
	WriteEraseMode( Buttons->GetButton(10)->Rect.left + 8,
					Buttons->GetButton(10)->Rect.top  + 8);
	if(EraseMode == E_NONE)
	{
		MenuTiles->BlitTrans(14, &Source, SystemBuffer, &(Buttons->GetButton(10)->Rect));
	}

	/*
	 * blit small walls button
	 */
	MenuTiles->Blit(15, &Source, SystemBuffer, &(Buttons->GetButton(30)->Rect));
	
    /*
     * blit yellow frame around show tile on map
	 * note that source rect can be clipped after this!!
     */
    GetBlitRectForCell (&Dest, ShowTilePosX, ShowTilePosY);

    if ( Clip ( ViewRect, &Source, &Dest ))
    {
        MenuTiles->BlitTrans( 8, &Source, SystemBuffer, &Dest );
    }

	/*
	 *	for debug reasons: show cache dump
	 */
//	DEBUGCODE ( Cache64x128->Dump(SystemBuffer, 0)  );

	/*
	 *	show MiniMap
	 */
	MiniMap->Blit(SystemBuffer, 0, 500);
	
	char CellString[20];
	Writer->Out(3, 16, "CellX:");
	Writer->Out(3, 32, "CellY:");
	if(Map->IsValidCell(MouseCellX, MouseCellY))
	{
		itoa(MouseCellX, CellString, 10);
		Writer->Out(64, 16, CellString);
		itoa(MouseCellY, CellString, 10);
		Writer->Out(64, 32, CellString);
	}
	else
	{
		Writer->Out(64, 16, "-");
		Writer->Out(64, 32, "-");
	}
}

void
cEDITFISH::MouseClick( tSINT32 MouseX, tSINT32 MouseY, tUINT32 MouseButtonNr )
{
    /*
     * check if a tile is hit
     */
    if ( ( ( MouseX >= ViewRect.left ) && ( MouseX <= ViewRect.right) ) &&
         ( ( MouseY >= ViewRect.top )  && ( MouseY <= ViewRect.bottom ) ) )
    {
        // Variables for ArrayTile that has been hit
        tUINT32 HitCellX    = 0 ;
        tUINT32 HitCellY    = 0 ;

        GetCellForScreen(&HitCellX, &HitCellY, MouseX, MouseY);

        /*
         * right click on tile makes it ShowTile
         */
        if ( MouseButtonNr == 2 )
        {
			UpdateShowTile(HitCellX, HitCellY);    
        }
    }

    /*
     * check if a menu button is hit
     */
    else
    {
        for ( tUINT32 i = 0; i < Buttons->GetNrOfButtons(); i++)
        {
			tBUTTON* ThisButton = Buttons->GetButton(i);
            if ((( MouseX >= ThisButton->Rect.left ) && ( MouseX <= ThisButton->Rect.right )) &&
                (( MouseY >= ThisButton->Rect.top ) && ( MouseY <= ThisButton->Rect.bottom )))
            {
                if ( ThisButton->Active )
                {
                   ActivateButton ( i, MouseButtonNr );
                }
            }
        }
    }
}

/*
 *	takes care of dragging moves
 */
void
cEDITFISH::MouseState(tSINT32 PosX, tSINT32 PosY, tUINT32 Buttons)
{
	// everything over the array area
	 if ((( PosX >= ViewRect.left ) && ( PosX <= ViewRect.right )) &&
         (( PosY >= ViewRect.top ) && ( PosY <= ViewRect.bottom )))
	 {
		 tUINT32 LocalMouseCellX;
		 tUINT32 LocalMouseCellY;
		 GetCellForScreen(&LocalMouseCellX, &LocalMouseCellY, PosX, PosY);
		 
		 if (Buttons&cMOUSE::Button1)
		 {
			 if(Map->IsValidCell(LocalMouseCellX, LocalMouseCellY))
			 {
				 if(!LeftButtonDown)
				 {
					 LeftButtonDown = TRUE;
					 MouseCellX = LocalMouseCellX;
					 MouseCellY = LocalMouseCellY;
					 if(EraseMode == E_NONE)
					 {
						 tSINT32 NewTile = PaintSelection->GetRandomSelection();
						 if (NewTile != -1)
						 {
							 PutNewTile(MouseCellX, MouseCellY, NewTile, CurrentIndexOffset + ActiveTilesNr);
							 StartTime = Timer.GetTime();
						 }
					 }
					 else
					 {
						 EraseTile(MouseCellX, MouseCellY);
					 }
				 }
				 else if ((MouseCellX == LocalMouseCellX) &&
						  (MouseCellY == LocalMouseCellY))
				 {
					 if(EraseMode == E_NONE)
					 {
						 // timed replacement of tile 
						 double Time = Timer.GetTime() - StartTime;
						 Time = (Timer.GetFrequency ? Time/Timer.GetFrequency() : 0.0);
						 if (Time > 0.2)
						 {
				 			tSINT32 NewTile = PaintSelection->GetRandomSelection();
							if (NewTile != -1)
							{
								PutNewTile(MouseCellX, MouseCellY, NewTile, CurrentIndexOffset + ActiveTilesNr);
								StartTime = Timer.GetTime();
							}
						 }
					 }
				 }
				 else
				 {
					 MouseCellX = LocalMouseCellX;
					 MouseCellY = LocalMouseCellY;
					 if(EraseMode == E_NONE)
					 {
						 tSINT32 NewTile = PaintSelection->GetRandomSelection();
						 if (NewTile != -1)
						 {
							 PutNewTile ( MouseCellX, MouseCellY, NewTile, CurrentIndexOffset + ActiveTilesNr);
							 StartTime = Timer.GetTime();
						 }
					 }
					 else
					 {
						 EraseTile(MouseCellX, MouseCellY);
					 }
				 }
			 }
		 }
		 else
		 {
			 MouseCellX = LocalMouseCellX;
			 MouseCellY = LocalMouseCellY;			
		 }		 
	 }
	 else
	 {
		 MouseCellX = -1;
		 MouseCellY = -1;
	 }
	

	 if(!Buttons&cMOUSE::Button1)
	 {
		 LeftButtonDown = FALSE;
	 }
}

void
cEDITFISH::HandleKeyStroke(tUINT8 KeyMessage)
{
	switch (KeyMessage)
	{
		case KEY_HOME:
			//SetPosition( 0, 0 );
		break;

		case KEY_LEFT:
			//ActivateButton ( 3, 1);
			ChangeTileSet(FALSE);
		break;

		case KEY_RIGHT:
			//ActivateButton( 4, 1);
			ChangeTileSet(TRUE);
		break;

		case KEY_UP:
			ChangeTile (VisibleTile - 1 );
		break;

		case KEY_DOWN:
			ChangeTile ( VisibleTile + 1 );
		break;

		case KEY_0:
			SetWorkingLayer(0);
		break;

		case KEY_1:
			SetWorkingLayer(1);
		break;

		case KEY_2:
			SetWorkingLayer(2);
		break;

		case KEY_3:
			SetWorkingLayer(3);
		break;

		case KEY_4:
			SetWorkingLayer(4);
		break;

		case KEY_R:
			PaintSelection->ReleaseSelection();
			PaintSelection->Select(VisibleTile);
		break;
	}

}

/*
 * Activates all MenuButtons after they have been hit with the mouse or called for with shortcut
 */
void
cEDITFISH::ActivateButton ( tUINT32 ButtonNr, tUINT32 MouseButtonNr )
{
    tUINT32 Height;
    Height = ActiveTiles->GetTileHeight();
    switch ( MouseButtonNr )
    {
        //all left clicks handled here
        case 1 :
            switch ( ButtonNr )
            {
                case 0 :
                    ChangeTile ( VisibleTile - 1 );
                break;

                case 1 :
                    ChangeTile ( VisibleTile + 1 );
                break;

                case 2 :
                    Map->Optimize();
					UpdateShowTile(ShowTilePosX, ShowTilePosY);
                break;

                case 3 :
                    ChangeTileSet(FALSE);
                break;

                case 4 :
                    ChangeTileSet(TRUE);
                break;

                case 5 :
                    if ( WorkingLayerNr >0 )
                    {
                        WorkingLayerNr = WorkingLayerNr - 1;
                    }
                    else
                    {
                        WorkingLayerNr = MAXLAYER;
                    }
                break;

                case 6 :
                    if ( WorkingLayerNr < MAXLAYER )
                    {
                        WorkingLayerNr = WorkingLayerNr + 1;
                    }
                    else
                    {
                        WorkingLayerNr = 0;
                    }
                break;

				//NewButton
				case 7:
					if (Dialog == NULL)
						Dialog = new cNEWMAPDIALOG();
				break;

				//LoadButton
				case 8:
					if (Dialog == NULL)
						Dialog = new cLOADMAPDIALOG();
				break;

				//SaveButton
				case 9:
					if (Dialog == NULL)
						Dialog = new cSAVEMAPDIALOG(Map->GetName(), Map->GetAuthor(), Map->GetFileName());
				break;

				// switch eraser 
				case 10:
					ChangeEraseMode(TRUE);
				break;

                case 11 :
                case 12 :
                case 13 :
                case 14 :
                    if ( Height == 32)
                    {
                        ChangeTile( VisibleTile + ButtonNr - 17 ) ;
                    }
                    else
                    {
                        ChangeTile( VisibleTile - 1 );
                    }
                break;

                case 15 :
                case 16 :
                case 17 :
                case 18 :
                    if ( Height == 32)
                    {
                        ChangeTile( VisibleTile + ButtonNr - 17 ) ;
                    }
                break;

                case 19 :
                case 20 :
                case 21 :
                case 22 :
                    if ( Height == 32)
                    {
                        ChangeTile( VisibleTile + ButtonNr - 17 ) ;
                    }
                    else
                    {
                        ChangeTile( VisibleTile + 1 );
                    }
                break;

                // click in ShowTile changes the ActiveTile
                case 23 :
                case 24 :
                case 25 :
                case 26 :
                case 27 :
                case 28 :
					{
						tUINT32 NewTileSet = ShowTile[ButtonNr - 23].TileSet;
						switch (NewTileSet / 256)
						{
							case 0 : 
								ActiveTilesKind = GROUND;
							break;
							case 1 :
								ActiveTilesKind = OVERLAY;
							break;
							case 2 :
								ActiveTilesKind = WALL;
							break;
							case 3 :
								ActiveTilesKind = SPRITE;
							break;
						}
						ActiveNrOfTileSets = GetNrOfTileSets(ActiveTilesKind);
						CurrentIndexOffset = (GetNrOfTileSets(MAX) / 4) * ActiveTilesKind; 
						ActiveTiles = Tiles[NewTileSet];
						PaintSelection->ReleaseSelection();
						PaintSelection->Select(VisibleTile);	           
						ChangeTile(ShowTile[ButtonNr - 23].TileNr);
					}
                break;
				case 29 :
					ChangeKind();
				break;
				case 30 :
					if(SmallWalls)
						SmallWalls = FALSE;
					else
						SmallWalls = TRUE;
				break;
            }
         break;
			
		 //all right clicks handled here
         case 2 :
			
            switch (ButtonNr)
            {
				//switch erase mode
				case 10:
					ChangeEraseMode(FALSE);
				break;

				// for all visible tiles, check if one is to be selected
				// or deselected
				case 11 :
				case 12 :
				case 13 :
				case 14 :
					tSINT32 ClickedTile;
					if ( Height == 32)
                    {
                        ClickedTile = VisibleTile + ButtonNr - 17;
                    }
                    else
                    {
						ClickedTile = VisibleTile - 1;
					} 
					
					if (ClickedTile >= 0)
					{
						ClickedTile = ClickedTile % ActiveTiles->NrOfTiles;
					}
					else
					{
						ClickedTile = __max(12, ActiveTiles->NrOfTiles) - (abs(ClickedTile) % ActiveTiles->NrOfTiles);
					}
					
					if (PaintSelection->IsSelected(ClickedTile))
					{
						PaintSelection->Deselect(ClickedTile);
					}
					else
					{
						PaintSelection->Select(ClickedTile);
					}
				break;
				case 15 :
				case 16 :
//				case 17 :
				case 18 :
					if ( Height == 32 )
                    {
                        ClickedTile = VisibleTile + ButtonNr - 17;
						if (ClickedTile >= 0)
						{
							ClickedTile = ClickedTile % ActiveTiles->NrOfTiles;
						}
						else
						{
							ClickedTile = __max(12, ActiveTiles->NrOfTiles) - (abs(ClickedTile) % ActiveTiles->NrOfTiles);
						}

						if (PaintSelection->IsSelected(ClickedTile))
						{
							PaintSelection->Deselect(ClickedTile);
						}
						else
						{
							PaintSelection->Select(ClickedTile);
						}
                    }
              
				break;
				case 19 :
				case 20 :
				case 21 :
				case 22 :
					if ( Height == 32)
                    {
                        ClickedTile = VisibleTile + ButtonNr - 17;
                    }
                    else
                    {
						ClickedTile = VisibleTile + 1;
					}
					
					if (ClickedTile >= 0)
					{
						ClickedTile = ClickedTile % ActiveTiles->NrOfTiles;
					}
					else
					{
						ClickedTile = __max(12, ActiveTiles->NrOfTiles) - (abs(ClickedTile) % ActiveTiles->NrOfTiles);
					}

					if (PaintSelection->IsSelected(ClickedTile))
						 {
							 PaintSelection->Deselect(ClickedTile);
						 }
						 else
						 {
							 PaintSelection->Select(ClickedTile);
						 }
				break;

                // fill layer with empty tile
                case 23 :
                    DeleteShowTileLayer(0);
                break;
                case 24 :
                    DeleteShowTileLayer(1);
                break;
                case 25 :
                    DeleteShowTileLayer(2);
                break;
                case 26 :
                    DeleteShowTileLayer(3);
                break;
                case 27 :
                    DeleteShowTileLayer(4);
                break;
                // delete wall
                case 28 :
					DeleteWall(ShowTilePosX, ShowTilePosY);
                    UpdateShowTile(ShowTilePosX, ShowTilePosY);
                break;
				case 29 :
					ChangeKind(FALSE);
				break;
            }
         break;

         case 3 :
         break;
    }
}


void
cEDITFISH::PutNewTile ( tSINT32 NewTilePosX, tSINT32 NewTilePosY, tUINT32 NewTileNr, tUINT32 NewTileSetNr )
{
    // security check, do not write outside of array
    if ( Map->IsValidCell(NewTilePosX, NewTilePosY) )
    {
        tUINT32 Height;
        Height = Tiles[NewTileSetNr]->GetTileHeight();
        if ( Height == 32)
        {
            cCELLINFO *Layer;
            Layer = GetWorkingLayer ( NewTilePosX, NewTilePosY );
            Layer->TileSet = NewTileSetNr;
            Layer->TileNr = NewTileNr ;
			if(WorkingLayerNr == 0)
			{
				// ground has been changed, update MiniMap
				MiniMap->RefreshPixel(Map, NewTilePosX, NewTilePosY);

			}
        }
        else
        {
            if ( Map->GetWall( NewTilePosX, NewTilePosY ) == NULL )
            {
                Map->SetWall( NewTilePosX, NewTilePosY, NewTileSetNr, NewTileNr );
				//Map->SetFlags( NewTilePosX, NewTilePosY, FLAGBLOCKING );
            }
			else
			{
				Map->GetWall( NewTilePosX, NewTilePosY )->TileNr = NewTileNr;
				Map->GetWall( NewTilePosX, NewTilePosY )->TileSet = NewTileSetNr;
			}
			// change Minimap
			MiniMap->RefreshPixel(Map, NewTilePosX, NewTilePosY);
        }

		//update ShowTile if neccessary
		if((NewTilePosX == ShowTilePosX) && (NewTilePosY == ShowTilePosY))
		{
			UpdateShowTile(ShowTilePosX, ShowTilePosY);
		}
    }
}

/*
 *	forces the new layer to exist
 */
cCELLINFO *
cEDITFISH::GetWorkingLayer ( tSINT32 NewTilePosX, tSINT32 NewTilePosY)
{
    tSINT32 i, j;
    tSINT32 Counter = -1;
    cCELLINFO *Layer = Map->GetFirstLayer(NewTilePosX, NewTilePosY);
    while ( Layer != NULL )
    {
        Counter++;
        Layer = Layer->NextLayer;
    }
    if ( (tSINT32) WorkingLayerNr <= ( Counter ))
    {
        Layer = Map->GetFirstLayer(NewTilePosX, NewTilePosY);
        for ( i=0; i < (tSINT32) WorkingLayerNr; i++)
        {
            Layer = Layer->NextLayer;
        }
    }
    else
    {
        Layer = Map->GetFirstLayer(NewTilePosX, NewTilePosY);
        for ( i = 0; i < Counter ; i++ )
        {
            Layer = Layer->NextLayer;
        }
        for ( j = Counter; j < (tSINT32) WorkingLayerNr; j++ )
        {
            Layer->NextLayer = new cCELLINFO();
            Layer = Layer->NextLayer;

            Layer->NextLayer    = NULL;
            Layer->TileNr       = 2;
            Layer->TileSet      = 0;
        }
    }
    return Layer;
}

/*
 *	like GetWorkingLayer, but returns NULL if layer does not exist
 */
cCELLINFO*
cEDITFISH::GetLayer(tSINT32 NewTilePosX, tSINT32 NewTilePosY, tUINT32 LayerNr)
{
	tSINT32 Counter = 0;
    cCELLINFO *Layer = Map->GetFirstLayer(NewTilePosX, NewTilePosY);
    while (( Layer != NULL ) && (Counter < LayerNr))
    {
        Counter++;
        Layer = Layer->NextLayer;
    }
	
	return Layer;
}

void
cEDITFISH::NewMapHeader(cRETURNDATABASE* NewMapData)
{
	cNEWMAPDIALOG::cRETURNDATA* MapData = // !!! check for NULL return and return;
		dynamic_cast<cNEWMAPDIALOG::cRETURNDATA*> ( NewMapData );
	tMAPHEADER MapHeader;

	memcpy( MapHeader.Name, MapData->Name, sizeof( MapHeader.Name ));
	memcpy( MapHeader.Author, MapData->Author, sizeof( MapHeader.Author ));
	MapHeader.Width   = MapData->Width;
	MapHeader.Height  = MapData->Height;
	MapHeader.Version = DATAVERSION;

	//now that everything has been copied, destroy Dialog
	SAFEDELETE (Dialog);
	
	// make new Map from Header
	NewMap ( MapHeader);
	MiniMap->Refresh(Map);
}

void 
cEDITFISH::LoadMap(cRETURNDATABASE* Data)
{
	cLOADMAPDIALOG::cRETURNDATA* ReturnData =
		dynamic_cast<cLOADMAPDIALOG::cRETURNDATA*> (Data);
	
	if (ReturnData)
	{
		ChangeMap( ReturnData->FileName.c_str( ));
		MiniMap->Refresh(Map);
	}
	
	SAFEDELETE(Dialog);
};

void
cEDITFISH::SaveMap(cRETURNDATABASE* Data)
{
	cSAVEMAPDIALOG::cRETURNDATA* ReturnData =
		dynamic_cast<cSAVEMAPDIALOG::cRETURNDATA*> (Data);

	Map->SetName(ReturnData->MapName.c_str());
	Map->SetAuthor(ReturnData->Author.c_str());
	Map->Save(ReturnData->FileName.c_str());

	SAFEDELETE(Dialog);
}

/*
 *	toggle between different kinds of tiles (ground, overlay, wall, sprite)
 */
void
cEDITFISH::ChangeKind(tBOOL Up)
{
	if(Up)
	{
		if (ActiveTilesKind == SPRITE)
		{
			ActiveTilesKind = GROUND;
		}
		else
		{
			ActiveTilesKind = (tTILESETTYPE)((tSINT32) ActiveTilesKind + 1);
		}
	}
	else
	{
		if (ActiveTilesKind == GROUND)
		{
			ActiveTilesKind = SPRITE;
		}
		else
		{
			ActiveTilesKind = (tTILESETTYPE) ((tSINT32) ActiveTilesKind - 1);
		}
	}

	ActiveTilesNr = 1;
	ChangeTile(0);
	ActiveNrOfTileSets = GetNrOfTileSets(ActiveTilesKind);
	CurrentIndexOffset = (GetNrOfTileSets(MAX) / 4) * ActiveTilesKind; 
	ActiveTiles = Tiles[CurrentIndexOffset + ActiveTilesNr];
	PaintSelection->ReleaseSelection();
	PaintSelection->Select(VisibleTile);	
}

/*
 *	switch to next/prev tile set of same kind
 */
void 
cEDITFISH::ChangeTileSet(tBOOL Up)
{
	if (Up)
	{
		if ( ActiveTilesNr < ( ActiveNrOfTileSets - 1 ))
        {
            ActiveTilesNr = ActiveTilesNr + 1;
            ActiveTiles = Tiles[CurrentIndexOffset + ActiveTilesNr];
            ChangeTile(0);
        }
        else
        {
            ActiveTilesNr = 1;
			ActiveTiles = Tiles[CurrentIndexOffset + ActiveTilesNr];
            ChangeTile(0);
        }
	}
	else
	{
		if ( ActiveTilesNr > 1 ) 
        {
            ActiveTilesNr = ActiveTilesNr - 1;
            ActiveTiles = Tiles[CurrentIndexOffset + ActiveTilesNr];
            ChangeTile(0);
        }
        else
        {
            ActiveTilesNr = ActiveNrOfTileSets - 1 ;
            ActiveTiles = Tiles[CurrentIndexOffset + ActiveTilesNr];
            ChangeTile(0);
        }
	}
	PaintSelection->ReleaseSelection();
	PaintSelection->Select(VisibleTile);
}

void 
cEDITFISH::ChangeTile(tSINT32 TileNr)
{
	if(!WasInSelection)
		PaintSelection->Deselect(VisibleTile);
	
	if(TileNr >= 0)
	{
		VisibleTile = TileNr % ActiveTiles->NrOfTiles;
	}
	else
	{
		if(ActiveTilesKind == WALL)
		{
			VisibleTile = ActiveTiles->NrOfTiles - (abs(TileNr) % ActiveTiles->NrOfTiles);
		}
		else
		{
			VisibleTile = ActiveTiles->NrOfTiles - (abs(TileNr) % ActiveTiles->NrOfTiles);
		}
	}

	if(PaintSelection->IsSelected(VisibleTile))
	{
		WasInSelection = TRUE;
	}
	else
	{
		WasInSelection = FALSE;
		PaintSelection->Select(VisibleTile);
	}
}

void
cEDITFISH::DeleteWall(tUINT32 PosX, tUINT32 PosY)
{
	Map->DeleteWall( PosX, PosY );
	MiniMap->RefreshPixel(Map, PosX, PosY);
}

void
cEDITFISH::DeleteShowTileLayer(tUINT32 LayerNr)
{
	tUINT32 BackUpLayerNr; 
	BackUpLayerNr = WorkingLayerNr;
    WorkingLayerNr = LayerNr;
	// Blankotile has to be 0 if it is a base tile (LayerNr = 0), 
	// 2 otherwise
	tUINT32 Blanko = (LayerNr ? 2 : 0);
    PutNewTile ( ShowTilePosX, ShowTilePosY, Blanko, 0 );
    WorkingLayerNr = BackUpLayerNr;
    UpdateShowTile(ShowTilePosX, ShowTilePosY);
}

/*
 *	sets ActiveTilesNr, ActiveTilesKind and CurrentIndexOffset
 *  for an absolute TileSetNr
 */
void 
cEDITFISH::SetTileSet(tUINT32 TileSetNr)
{
	ActiveTilesNr = TileSetNr % (GetNrOfTileSets(MAX) / 4); 
	ActiveTilesKind = (tTILESETTYPE) (TileSetNr / (GetNrOfTileSets(MAX) / 4));
	CurrentIndexOffset = (GetNrOfTileSets(MAX)/4) * (tUINT32) ActiveTilesKind;
	ActiveTiles = Tiles[TileSetNr];
}

void
cEDITFISH::WriteEraseMode(tUINT32 ScreenX, tUINT32 ScreenY)
{
	Writer->SetDrawingTarget(SystemBuffer);
	switch (EraseMode)
	{
		case E_ALL:
			Writer->Out(ScreenX, ScreenY, " All");
		break;
		case E_LAYER:
			Writer->Out(ScreenX, ScreenY, "Layer");
		break;
		case E_WALL:
			Writer->Out(ScreenX, ScreenY, " Wall");
		break;
	}
}

/*
 *	switches through the different erase modi in both directions
 */
void
cEDITFISH::ChangeEraseMode(tBOOL Up)
{
	if(Up)
	{
		switch (EraseMode)
		{
			case E_NONE:
				EraseMode = E_ALL;
			break;
			case E_ALL:
				EraseMode = E_LAYER;
			break;
			case E_LAYER:
				EraseMode = E_WALL;
			break;
			case E_WALL:
				EraseMode = E_NONE;
			break;
		}
	}
	else
	{
		switch (EraseMode)
		{
			case E_NONE:
				EraseMode = E_WALL;
			break;
			case E_ALL:
				EraseMode = E_NONE;
			break;
			case E_LAYER:
				EraseMode = E_ALL;
			break;
			case E_WALL:
				EraseMode = E_LAYER;
			break;
		}
	}
}

void
cEDITFISH::EraseTile(tSINT32 CellX, tSINT32 CellY)
{
	switch(EraseMode)
	{
		case E_ALL:
			{
				DeleteWall(CellX, CellY);
				Map->DeleteAllLayers(CellX, CellY);
				tUINT32 Backup = WorkingLayerNr;
				WorkingLayerNr = 0;
				PutNewTile(CellX, CellY, 0, 0);
				WorkingLayerNr = Backup;
				MiniMap->RefreshPixel(Map, CellX, CellY);
			}
		break;
		case E_LAYER:
			if(GetLayer(CellX, CellY, WorkingLayerNr))
			{
				// Blankotile has to be 0 if it is a base tile (LayerNr = 0), 
				// 2 otherwise
				tUINT32 Blanko = (WorkingLayerNr ? 2 : 0);
				PutNewTile(CellX, CellY, Blanko, 0);
				MiniMap->RefreshPixel(Map, CellX, CellY);
			}
		break;
		case E_WALL:
			DeleteWall(CellX, CellY);
		break;
	}
}

cBUTTONARRAY::cBUTTONARRAY(const char* IniFileName)
{
	Initialize(IniFileName);
}

cBUTTONARRAY::~cBUTTONARRAY()
{
	for( tUINT32 i = 0; i < NrOfButtons; i++ )
	{
		SAFEDELETE(Buttons[i]);
	}
	SAFEDELETEARRAY( Buttons );
}

void
cBUTTONARRAY::Initialize(const char* IniFileName)
{
	cINIFILE* IniFile = new cINIFILE(IniFileName);

	IniFile->ReadNumber("[Buttonarray]", "NrOfButtons", &NrOfButtons);
	Buttons = new tBUTTON*[NrOfButtons];

	char NameString[16];
	for (tUINT32 i = 0; i < NrOfButtons; i++)
	{
		Buttons[i] = new tBUTTON;
		sprintf( NameString, "[Button%i]", i);
		IniFile->ReadNumber ( NameString, "Left", &(Buttons[i]->Rect.left) );
		IniFile->ReadNumber ( NameString, "Right", &(Buttons[i]->Rect.right) );
		IniFile->ReadNumber ( NameString, "Top", &(Buttons[i]->Rect.top) );
		IniFile->ReadNumber ( NameString, "Bottom", &(Buttons[i]->Rect.bottom) );
		IniFile->ReadNumber ( NameString, "Active", &(Buttons[i]->Active) );
	}

	SAFEDELETE(IniFile);
}

void
cEDITFISH::UpdateShowTile(tUINT32 PosX, tUINT32 PosY)
{
	// security check, do not write outside of array
    if ( ( ( PosX >= 0 ) && ( PosX < Map->GetWidth() ) ) &&
        ( ( PosY >= 0 ) && ( PosY < Map->GetHeight()) ) )
    {
        // "empty" showtile
        tUINT32 i;
        for ( i = 0; i <= 5; i++ )
        {
            ShowTile[i].TileSet = 0;
            ShowTile[i].TileNr  = 0;
        }

        ShowTilePosX = PosX;
        ShowTilePosY = PosY;

        tUINT32 Counter = 0;
        cCELLINFO *ThisCell= Map->GetFirstLayer( PosX, PosY );

        ShowTile[Counter].TileSet = ThisCell->TileSet;
        ShowTile[Counter].TileNr  = ThisCell->TileNr;

        if ( Map->GetWall( PosX, PosY ) != NULL )
        {
            ShowTile[5].TileSet = Map->GetWall( PosX, PosY )->TileSet;
            ShowTile[5].TileNr = Map->GetWall( PosX, PosY )->TileNr;
            Buttons->GetButton(28)->Active = TRUE;
        }

        while ( ThisCell->NextLayer != NULL )
        {
            ThisCell   = ThisCell->NextLayer;
            Counter++;
            ShowTile[Counter].TileSet = ThisCell->TileSet;
            ShowTile[Counter].TileNr = ThisCell->TileNr;
        }

        //enable all buttons which are not empty
        for ( i = 0; i <= Counter; i++ )
        {
			// assumes that all tiles in Blankfloor are invalid
            if (!( ShowTile[i].TileSet == 0 ))
            {
                Buttons->GetButton(23+i)->Active = TRUE;
            }
			else
			{
				Buttons->GetButton(23+i)->Active = FALSE;
			}
        }
    }
}



