/******************************************************************************
    HawkCreek software

    Purpose:
    Testumgebung fr Editor

    Notes:

    History of Changes:
    - 09.10.1999: file created by Capella (mainly stolen from BuschnicK)
	- 17.03.2000: project now compiles on VisualC++ 6.0

******************************************************************************/

#include "general\debug.hpp"
#include "general\helpmacros.hpp"
#include "general\type.hpp"
#include "BlackFish\Blackfish.hpp"
#include "BlackFish\Cache.hpp"
#include "BlackFish\DXDraw.hpp"
#include "BlackFish\Input.hpp"
#include "BlackFish\Surface.hpp"
#include "BlackFish\PixelConverter.hpp"
#include "BlackFish\paint.hpp"
#include "BlackFish\TileSet.hpp"
#include "BlackFish\write.hpp"
#include "BlackFish\Dialog.hpp"
#include "BlackFish\Time.hpp"
#include "EditFish\EditFish.hpp"
#include "resource.h"


#ifndef __WINDOWS_H__
#define __WINDOWS_H__
  #ifndef WIN32_LEAN_AND_MEAN // !! reset this after including windows.h?
  #define WIN32_LEAN_AND_MEAN
  #endif
#include <windows.h>
#endif
#include <stdlib.h>                     // rand, itoa
#include <ddraw.h>                      // Clipper
#include <mmsystem.h>                   // timeGetTime()

// !! Should be read from a resource or so, whatever...
#define NAME        "Editor"
#define TITLE       "BlackFish Map Editor"
#define WHEELUP     120
#define WHEELDOWN   -120

tBOOL ApplicationActive         = FALSE;
const tBOOL FullScreen			= TRUE;

cSURFACE    *Primary            = NULL;
cSURFACE    *BackBuffer         = NULL;


cTILESET    *Font1              = NULL;
cTILESET    *MouseCursor        = NULL;
cCACHE      *Cache16x16         = NULL;
cCACHE      *Cache64x32         = NULL;
cCACHE      *Cache64x128        = NULL;
cPAINT      *Painter            = NULL;

cINPUT      *DXInput            = NULL;
cMOUSE		*Mouse              = NULL;
cKEYBOARD	*KeyBoard			= NULL;

cEDITFISH   *EditFish           = NULL;


// global for the posting of messages in WinMain
HWND        WindowHandle;

tSINT32 MouseX = 0, MouseY = 0, WheelPos = 0;
tUINT32 Button = 0;
tBOOL  Click1 = FALSE, Click2 = FALSE, Click3 = FALSE;
tSINT32 ScrollRate = 1;


void
ShowMiscInfo( void )
{
    static tUINT32 FrameTime = 10000, OldFrameTime = 0, FrameRate = 0;
    static tUINT32 Frames;

    Frames++;
    FrameTime = timeGetTime();
    if ( (FrameTime - OldFrameTime) >= 1000 )
    {
        FrameRate = ( Frames * 1000 ) / (FrameTime - OldFrameTime);
        FrameTime = timeGetTime();
        OldFrameTime = FrameTime;
        Frames = 0;
    }
    char MiscString[20];
    itoa( FrameRate, MiscString, 10 );
	Writer->SetDrawingTarget( BackBuffer );
    Writer->Out(  3, 0, "FPS:" );
    Writer->Out(  48, 0, MiscString );
}


/*
 *
 */
long FAR PASCAL
WindowProcedure( HWND WindowHandle, UINT Message, WPARAM wParam, LPARAM lParam )
{
    switch ( Message )
    {

		case WM_ACTIVATEAPP:
            ApplicationActive = (wParam == WA_ACTIVE) || (wParam == WA_CLICKACTIVE);
            if ( ApplicationActive )
            {
                // if () to avoid crash on first activate app, when the objects haven't been created, yet
                if ( Cache16x16 ) Cache16x16->Invalidate();
                if ( Cache64x32 ) Cache64x32->Invalidate();
                if ( Cache64x128 ) Cache64x128->Invalidate();
                //*LOG << "Cache Invalidated! \n";
            }

        return 0L;


        case WM_DESTROY:
            SAFEDELETE( DXInput );
            SAFEDELETE( Mouse );
            SAFEDELETE( Painter );
            SAFEDELETE( EditFish );
            SAFEDELETE( BackBuffer );
            SAFEDELETE( Primary );
            SAFEDELETE( DirectDraw );
            SAFEDELETE( Font1 );
            SAFEDELETE( Writer );
            SAFEDELETE( Cache16x16 );
            SAFEDELETE( Cache64x32 );
            SAFEDELETE( Cache64x128 );
			SAFEDELETE( KeyBoard );
			SAFEDELETE( MouseCursor );
            ShutDownLogging ();
            PostQuitMessage(0);
        return 0L;

        case WM_SETCURSOR:
            SetCursor( NULL );
        return TRUE;

    }
    return DefWindowProc( WindowHandle, Message, wParam, lParam );
}

/*
 *
 */
void
InitApplication( HINSTANCE hInstance, int nCmdShow )
{

    WNDCLASS    WindowClass;

    // Set up and register window class
    WindowClass.style            = CS_HREDRAW | CS_VREDRAW;
    WindowClass.lpfnWndProc      = WindowProcedure;
    WindowClass.cbClsExtra       = 0;
    WindowClass.cbWndExtra       = 0;
    WindowClass.hInstance        = hInstance;
    WindowClass.hIcon            = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON1));
    WindowClass.hCursor          = LoadCursor(NULL, IDC_ARROW);
    WindowClass.hbrBackground    = (HBRUSH )GetStockObject(BLACK_BRUSH);
    WindowClass.lpszMenuName     = NAME;
    WindowClass.lpszClassName    = NAME;
    RegisterClass( &WindowClass );

	RECT Client;
	SetRect( &Client, 64, 64, 800 + 64, 600 + 64 );
	AdjustWindowRectEx(
		&Client,
		WS_OVERLAPPEDWINDOW,
		FALSE,
		0 );



    // Now that windows knows about our new type of window create one
    WindowHandle = CreateWindowEx(
        0,//WS_EX_TOPMOST,
        NAME,
        TITLE,
        FullScreen ? WS_POPUP : WS_OVERLAPPEDWINDOW,
        Client.left,
        Client.top,
        Client.right - Client.left,
        Client.bottom - Client.top,
        NULL,
        NULL,
        hInstance,
        NULL );

    ASSERT( WindowHandle != NULL );

    ShowWindow( WindowHandle, nCmdShow );
    UpdateWindow( WindowHandle );
    SetFocus( WindowHandle );

    InitLogging ( "EditLog.txt" );

	*LOG << "Init";

    DXInput = new cINPUT( hInstance );

    Mouse = new cMOUSE( WindowHandle, DXInput );

	KeyBoard = new cKEYBOARD( WindowHandle, DXInput );

    DirectDraw = new cDXDRAW( WindowHandle, FullScreen );

    Primary = new cSURFACE();
    Primary->Create( );

    BackBuffer = new cSURFACE( );
	if ( DirectDraw->IsFullScreen( ))
		BackBuffer->Create( Primary ); // We're fullscreen, create a DX flipping chain
	else
		BackBuffer->Create( 800, 600, TRUE ); // We're windowed, create offcreen vidmem backbuffer

    Painter = new cPAINT();
    Painter->SetDrawingTarget( BackBuffer );

    cPIXELCONVERTER *PixelConverter = NULL;
    PixelConverter = new cPIXELCONVERTER( BackBuffer );
    ASSERT( PixelConverter != NULL );
    tUINT32 TransparentColor = PixelConverter->RGBToColor( 255, 0, 255 );
    SAFEDELETE( PixelConverter );

    Cache64x128 = new cCACHE( 64,128, TransparentColor, 4 );
    Cache64x32  = new cCACHE( 64, 32, TransparentColor, 4 );
    Cache16x16  = new cCACHE( 16, 16, TransparentColor, 1 );

    EditFish = new cEDITFISH( BackBuffer,
							  Cache16x16,
							  Cache64x32,
							  Cache64x128,
							  "graphics\\BlackFish.ini");

    Font1 = new cTILESET( "graphics\\font.ini", "[Font1]" );
    Font1->SetCache( Cache16x16 );

    MouseCursor = new cTILESET( "graphics\\mouse.ini", "[MouseCursor]" );
    MouseCursor->SetCache( Cache16x16 );

    Writer = new cWRITE();
    Writer->SetFont( Font1 );
    Writer->SetSpacing( 0 );
	*LOG << "Writer initialized \n";
    //EditFish->SetWriter(Writer);

	srand((tUINT32)Timer.GetTime());
}

void
Transpatch (cRETURNDATABASE* DataPackage)
{
	cNEWMAPDIALOG::cRETURNDATA* NewMapData = 
		dynamic_cast<cNEWMAPDIALOG::cRETURNDATA*> (DataPackage);
	if(NewMapData)
	{
		EditFish->NewMapHeader(DataPackage);
		return;
	}

	cLOADMAPDIALOG::cRETURNDATA* LoadMapData =
		dynamic_cast<cLOADMAPDIALOG::cRETURNDATA*> (DataPackage);
	if(LoadMapData)
	{
		EditFish->LoadMap(DataPackage);
		return;
	}

	cSAVEMAPDIALOG::cRETURNDATA* SafeMapData =
		dynamic_cast<cSAVEMAPDIALOG::cRETURNDATA*> (DataPackage);
	if(SafeMapData)
	{
		EditFish->SaveMap(DataPackage);
		return;
	}
	// and so on

	//if we get to this point, a wrong or empty DataPackage (Cancel)
	//was returned, destroy dialog anyway
	SAFEDELETE(Dialog);
}

/*
 *
 */
int PASCAL
WinMain(
    HINSTANCE Instance,
    HINSTANCE PreviousInstance,
    LPSTR CommandLine,
    int CommandShow)
{
    // !! Dummies to avoid the 'Not used' warning
    PreviousInstance = PreviousInstance;
    CommandLine      = CommandLine;

    InitApplication( Instance, CommandShow );

    MSG Message;
    while ( TRUE )
    {
        if ( PeekMessage( &Message, NULL, 0, 0, PM_NOREMOVE ))
        {
            if ( !GetMessage( &Message, NULL, 0, 0 ))
                return Message.wParam;
            TranslateMessage( &Message );
            DispatchMessage( &Message );
        }
        else if ( ApplicationActive )
        {
			Mouse->UpdateState();
			Mouse->GetState( &MouseX, &MouseY, &WheelPos, &Button );
			
			tUINT32 Click = Mouse->GetLastClick();

			KeyBoard->UpdateState();
			tUINT8 Key = KeyBoard->GetLastKey();
			if (Dialog == NULL)
			{
				EditFish->MouseState(MouseX, MouseY, Button);
				/*
				 * MouseButtonCheck
				 */
				if ( Click & cMOUSE::Button1 )
				{
					EditFish->MouseClick( MouseX, MouseY, 1 );
				}

				if ( Click & cMOUSE::Button2 )
				{
					EditFish->MouseClick( MouseX, MouseY, 2 );
				}

				if ( Click & cMOUSE::Button3 )
				{
					EditFish->MouseClick( MouseX, MouseY, 3 );
				}

				// !! not encapsuled, EditMain should not need to know the button no.
				if ( WheelPos == WHEELUP)
				{
					EditFish->ActivateButton( 0 , 1 );  // same as left click on UpArrow
				}
				// !! dito
				if ( WheelPos == WHEELDOWN)
				{
					EditFish->ActivateButton( 1 , 1 );  // same as left click on DownArrow
				}

				EditFish->Scroll(MouseX, MouseY);

				switch (Key)
				{
					case KEY_ESCAPE:
						PostMessage( WindowHandle, WM_CLOSE, 0, 0 );
					break;

					case KEY_F12:
						PostMessage( WindowHandle, WM_CLOSE, 0, 0 );
					break;

					default :
						EditFish->HandleKeyStroke(Key);
					break;
				}

			}
			else
			{
				// Escape and F12 are the only keys treated normally,
				// but destroy dialog before leaving program
				if (( Key == KEY_ESCAPE ) || ( Key == KEY_F12 ))
				{
					SAFEDELETE (Dialog);
					PostMessage( WindowHandle, WM_CLOSE, 0, 0 );
				}
			    // all other input is treated by the dialog itself
				else
				{
					Dialog->MouseClick(MouseX, MouseY, Click);
					tBOOL Shift;
					Shift = KeyBoard->GetState(KEY_LSHIFT) | KeyBoard->GetState(KEY_RSHIFT);
					char ASCII = KeyBoard->KeyToASCII( Key, Shift);
					if (ASCII)
					{
						Dialog->CharMessage(ASCII);
					}
					
							
					if ( Dialog->GetState() == cDIALOG::FINISH )
					{
						Transpatch(Dialog->GetDataPackage());
					}
				}
			}

            Painter->Fill( 0 );
            EditFish->Render(EditFish->GetWallState());
            EditFish->Menu();
            ShowMiscInfo ();

			// check for dialog, if active, blit
			if (Dialog != NULL)
			{
				Dialog->Show(BackBuffer);
			}

			RECT Source, Dest, Screen;
			SetRect( &Screen, 0, 0, 799, 699 );
			MouseCursor->GetTileRect( &Source );
			MouseCursor->GetTileRect( &Dest );
			Dest.left	+= MouseX;
			Dest.right	+= MouseX;
			Dest.top	+= MouseY;
			Dest.bottom	+= MouseY;
			Clip( Screen, &Source, &Dest );
			MouseCursor->BlitTrans( 1, &Source, BackBuffer, &Dest );

			if ( DirectDraw->IsFullScreen( ))
				Primary->Flip( );
			else
			{
				RECT Screen;
				GetClientRect( DirectDraw->GetWindow( ), &Screen);
				ClientToScreen( DirectDraw->GetWindow( ), (POINT*)&Screen.left);
				ClientToScreen( DirectDraw->GetWindow( ), (POINT*)&Screen.right);
				Primary->Blit( BackBuffer, NULL, &Screen );
			}
        }
        else
        {
            WaitMessage( );
        }
    }
}

