#include "Editfish/Minimap.hpp"

/*********************************************************************
	HawkCreek Software


	Minimap		class that maintains and blits a surface containing
			    information about wall and floor tiles of a map

	History of Changes:
	- 29.09.2000 file created by Capella
**********************************************************************/

#include "Blackfish\Surface.hpp"
#include "Blackfish\Paint.hpp"
#include "Map\Map.hpp"
#include "Blackfish\Pixelconverter.hpp"
#include "General\Debug.hpp"
#include "General\Helpmacros.hpp"


cMINIMAP::cMINIMAP()
{
	Surface = new cSURFACE();
	Surface->Create(MM_SIZE_X, MM_SIZE_Y, FALSE);
}

cMINIMAP::~cMINIMAP()
{
	SAFEDELETE(Surface);
}

/*
 *	repaints the whole map
 */
void
cMINIMAP::Refresh(cMAP* NewMap)
{
	tUINT32 Width, Height;
	if(NewMap->GetWidth() >= MM_SIZE_X)
	{
		Width = MM_SIZE_X;
	}
	else
	{
		Width = NewMap->GetWidth();
	}
	if(NewMap->GetHeight() >= MM_SIZE_X)
	{
		Height = MM_SIZE_X;
	}
	else
	{
		Height = NewMap->GetHeight();
	}

	cPIXELCONVERTER Converter(Surface);
	cPAINT* Painter = new cPAINT();
	Painter->SetDrawingTarget(Surface);
	//Painter->Fill(Converter.RGBToColor(20,20,20));
	Painter->Fill( 0, 0, Width, Height, Converter.RGBToColor(20,20,20));
	Painter->BeginDrawing();

	for (tUINT32 y = 0; y < Height; y++)
	{
		for (tUINT32 x = 0; x < Width; x++)
		{
			if(NewMap->GetWall( x, y ) != NULL)
			{
				Painter->PutPixel(x, y, Converter.RGBToColor(0,192,192));
			}
			else if (( NewMap->GetFirstLayer(x,y)->TileNr != 0 ) ||
					 ( NewMap->GetFirstLayer(x,y)->TileSet != 0 ))
			{
				Painter->PutPixel(x, y, Converter.RGBToColor(64,64,64));
			}
		}
	}
*LOG << "Complete MiniMap refreshed!\n";
	Painter->EndDrawing();
	SAFEDELETE( Painter );
}

void
cMINIMAP::Blit(cSURFACE* Dest, tUINT32 XKoord, tUINT32 YKoord )
{
	RECT SourceRect;
	SourceRect.left   = 0;
	SourceRect.right  = MM_SIZE_X;
	SourceRect.top    = 0;
	SourceRect.bottom = MM_SIZE_Y;
	RECT DestRect;
	DestRect.left	= XKoord;
	DestRect.right	= DestRect.left + MM_SIZE_X;
	DestRect.top	= YKoord;
	DestRect.bottom	= DestRect.top + MM_SIZE_Y;

	Dest->Blit(Surface, &SourceRect, &DestRect);
}

void
cMINIMAP::RefreshPixel(cMAP* Map, tUINT32 PosX, tUINT32 PosY)
{
	if(( PosX < MM_SIZE_X ) && ( PosY < MM_SIZE_Y ))
	{
		cPIXELCONVERTER Converter(Surface);
		cPAINT* Painter = new cPAINT();
		Painter->SetDrawingTarget(Surface);
		Painter->BeginDrawing();
		if(Map->GetWall( PosX, PosY ) != NULL)
		{
			Painter->PutPixel(PosX, PosY, Converter.RGBToColor(0,192,192));
		}
		else if (( Map->GetFirstLayer(PosX, PosY)->TileNr != 0 ) ||
				 ( Map->GetFirstLayer(PosX, PosY)->TileSet != 0 ))
		{
			Painter->PutPixel(PosX, PosY, Converter.RGBToColor(64,64,64));
		}
		else
		{
			Painter->PutPixel(PosX, PosY, Converter.RGBToColor(20,20,20));
		}
		Painter->EndDrawing();
		SAFEDELETE(Painter);
	}
}