#ifndef __DEBUG_HPP__
#define __DEBUG_HPP__

/******************************************************************************
    HawkCreek software

    Purpose:
    debugging macros

    Notes:
    - 17.11.1998:
      ASSERT is used for all and every possible error, the code has to be revised
      later to decide how the specific error should be handled in the release version.
      i.e. file not found errors etc. should be properly reported to the user etc.
      as it is now it is an all or nothing approach (programmer version ;) )

    History of Changes:
    - 08.11.1998: file created by BuschnicK
    - 21.01.1999: polished for Aliens Project
    - 11.03.1999: ASSERT() now outputs to the file "c:\project\assert.txt"
                  Die Datei kann ziemlich lang werden, also von Zeit zu Zeit mal lschen.
                  Die Funktion schreibt an den Anfang jeder Zeile einen Counter, die wie
                  vielte assertion im aktuellen Programmdurchlauf gerade schiefgegangen
                  ist - daran kann man auch erkennen, wann das Programm neu gestartet wurde
                  (der Counter startet wieder bei 0).
                  Wichtiger Unterschied zu der alten ASSERT() Funktion: das Programm wird
                  bei einem Fehler *nicht* abgebrochen, sondern der Fehler wird ausgegeben
                  und das Programm luft weiter.
                  (12.03.99) Programm lt jetzt maximal 'MaxAssertionFailures' 'asserts()'
                             fehlschlagen, dann bricht die Funktion das Programm brutal
                             mit 'exit( EXIT_FAILURE )' ab.
                  assert.lib mu jetzt ins Projekt integriert werden
    - 05.11.1999: removed the LOG macro and associated functions and replaced them with a
                  global LOG ofstream. This provides a *lot* more flexibility in the log
                  message format, i.e. you can now write numbers and pointers etc...
                  The global has to be initialized and killed with the InitLogging and
                  ShutDownLogging functions. It can then be used just like a normal ofstream,
                  for example:
                  *LOG << "Variable Test in " << __FILE__ << " has a value of " << Test << " ! \n";
                  One disantvantage is that all logging related calls are still there in
                  a release build and have to be removed manually.
*****************************************************************************/

#include "general\type.hpp"
#include <fstream>

void FileAssert( unsigned int Expression, const char *ErrorFileName, unsigned int ErrorLineNr );
void InitLogging( const char *LogFileName );
void ShutDownLogging( void );
extern std::ofstream *LOG;

#ifdef _DEBUG
#define ASSERT( BooleanExpression ) { FileAssert( BooleanExpression, __FILE__, __LINE__ ); }
#else
#define ASSERT( BooleanExpression )
#endif

#ifdef _DEBUG
#define DEBUGCODE( CodeFragment ) CodeFragment;
#else
#define DEBUGCODE( CodeFragment )
#endif

#endif
