#include "IniFile.hpp"

/******************************************************************************
    HawkCreek software
    Notes:
    History of Changes:
    - 08.11.1998: file created by BuschnicK
    - 06.02.1999: modified by BuschnicK
    - 20.02.1999: modified by BuschnicK, reorganized 'ReadInteger()' and added
                  new helper functions: 'FindSection()' and 'FindKeyWord()'
    - 27.02.1999: updated variable declarations according to the new names in
                  type.hpp
    - 25.07.1999: added 'ReadString()'
	- 16.03.2000: hopefully compatible to the new cpp headers and VisualC++ now
				  lots of tweaks and little hacks done
	- 16.10.2000: rewrite this, minimizing disc accesses and without using
				  the dangerous c-like strings, i.e. use streams and STL map 
				  and string to store KeyWord:Data pairs
******************************************************************************/

// !! are the external include guards needed in this case, too?
#include "general\debug.hpp"
#include "general\helpmacros.hpp"
#include <stdio.h>      // sscanf, eigentlich wollt ich's ja vermeiden...
#include <string>
#include <memory>       // memset

/*
 * constructor, initializes a cINIFILE object to it's default values, the object
 * is unusable in this state, Open() has to be called to specify an input file first
 */
cINIFILE::cINIFILE( void )
{
    Initialize( );
}

/*
 * constructor, initializes a cINIFILE object by calling Open() with the specified
 * Filename, the object is fully functional after this
 */
cINIFILE::cINIFILE( const char *FileName )
{
	InputFileName = NULL;

    // reset everything to it's default values
    Initialize( );

    // call open() to prepare a file for processing
    Open( FileName );
}

/*
 * Destructor
 */
cINIFILE::~cINIFILE( void )
{
    Close();
	SAFEDELETEARRAY( InputFileName );
}

/*
 * Resets all variables to their default values, the object is unsusable in
 * this state, 'Open()' has to be called
 */
void
cINIFILE::Initialize( void )
{
    // Reset the filehandle to no file
    InputFile = NULL;
	SAFEDELETEARRAY( InputFileName );
   
	// fill the currently loaded string with spaces - there's nothing loaded, yet
    memset( CurrentLine, ' ', MAXLINELENGTH );
	CurrentLine[MAXLINELENGTH - 1] = '\0';

    // fill the currently loaded valuestring (the characters following the equals
    // sign after the keyword in the *.ini file) with spaces
    memset( ValueString, ' ', MAXLINELENGTH );
	ValueString[MAXLINELENGTH - 1] = '\0';
}

/*
 * Opens a file for input, if any error occurs it immediately quits the program with
 * an assert() error message
 */
void
cINIFILE::Open( const char *FileName )
{
    ASSERT( FileName != NULL );
	InputFileName = new char[strlen( FileName ) + 1];
	memcpy( InputFileName, FileName, ( strlen( FileName ) + 1 ) * sizeof( char ));

    // open the file 'FileName' for processing, read only text-mode
    InputFile = new std::ifstream( FileName );

	if ( InputFile->is_open( ) == FALSE )
	{
		*LOG << "Error opening ini file: '" << InputFileName << "'!\n";
		QUIT;
	}

    // check for errors, (check if the filehandle is valid)
    ASSERT( InputFile != NULL );

    // important to check == FALSE and not != TRUE cause it might return values != 1
   // ASSERT( InputFile->fail() == FALSE );
}

/*
 * Closes the input file
 */
void
cINIFILE::Close( void )
{
    //  !! there doesn't seem to be a special close() function -> is that correct? (destructor?)

	SAFEDELETEARRAY( InputFileName );

    // free the file object
    SAFEDELETE ( InputFile );
}

/*
 * Alle nachfolgenden Leseoperationen starten am Dateianfang
 */
void
cINIFILE::Rewind( void )
{
    ASSERT( InputFile != NULL );

    // setzt den FileIndex zurck auf den Anfang der Datei
    InputFile->seekg( 0, std::ios_base::beg );

    //ASSERT( InputFile->fail() == FALSE );
}

/*
 * Read the next Line from the File into the buffer 'CurrentLine'
 */
tBOOL
cINIFILE::ReadLine( void )
{
    ASSERT( InputFile != NULL );

    InputFile->getline( CurrentLine, MAXLINELENGTH, '\n' );

    // if end of file (EOF) was reached return false, else true
    // note the strange way to check for this, read the 'getline()' help for details
    if ( InputFile->fail() == FALSE )
    {   // nominal case, EOF not reached
        return TRUE;
    }
    else
    {   // EOF encountered
        return FALSE;
    }
    // this point is never reached
}

/*
 * Checks if the first characters in 'CurrentLine' match the specified 'StringName'
 *
 * Note: none of the scanf functions is used here cause they would ignore comments
 * in the *.ini file
 */
tBOOL
cINIFILE::FindString( const char *StringName )
{
    ASSERT( StringName != NULL );

    tBOOL MatchFound = FALSE;        // does the first word in CurrentLine match StringName?
    const char *StringPtr = StringName;   // Pointers used to increment through the strings
    const char *LinePtr   = CurrentLine;

    // Solange das gesuchte Wort noch nicht gefunden wurde und die beiden
    // aktuellen Vergleichsbuchstaben bereinstimmen gehe weiter durch
    // den string
    while (( MatchFound == FALSE ) && ( *StringPtr == *LinePtr ))
    {
        // advance one character in both strings
        StringPtr++;
        LinePtr++;

        // wenn der NullCharakter ('\0') erreicht ist, ist das Suchwort zu
        // Ende. Da alle Zeichen bis zu diesem Punkt bereits bereinstimmen
        // knnen wir davon ausgehen, da das Wort gefunden wurde
        if ( *StringPtr == '\0' )
            MatchFound = TRUE;
    }

    return MatchFound;
}

/*
 * Reads through the file line by line and tries to find the keyword 'SectionName'
 * returns TRUE if the keyword was found
 */
tBOOL
cINIFILE::FindSection( const char *SectionName )
{
    ASSERT( SectionName != NULL );

    tBOOL SectionFound = FALSE;

	do
    {
        SectionFound = FindString( SectionName );
    } // continue while there is still more to read and we haven't found the Section yet
    while ((ReadLine()) && (!SectionFound));

    return SectionFound;
}

/*
 * Reads through the file line by line until the keyword is found, or the
 * beginning of a new Section is found. New Sections are marked with "[" because
 * the section name is always enclosed in "[]"
 * returns TRUE if the keyword was found in this Section
 * Note: This function assumes that the file pointer already points to the be-
 * ginning of the right Section, i.e. 'FindSection()' has been called
 */
tBOOL
cINIFILE::FindKeyWord( const char *KeyWord )
{
    ASSERT( KeyWord != NULL );

    tBOOL KeyWordFound = FALSE;
    tBOOL SectionEnd   = FALSE;
    tBOOL ReadOk       = TRUE;

    do
    {
        KeyWordFound = FindString( KeyWord );

        SectionEnd = FindString("[");

        // nur die nchste Zeile lesen wenn das 'KeyWord' noch nicht gefunden wurde
        // ntig, damit spter der Wert hinter dem 'KeyWord=' eingelesen werden kann
        if (!KeyWordFound)
            ReadOk = ReadLine();
    } // continue while there is more to read, we haven't found the keyword and the section didn't end
    while ((ReadOk) && (!KeyWordFound) && (!SectionEnd));

    return KeyWordFound;
}

/*
 * Reads the string after the 'KeyWord=', stops at the end of the line or
 * at the beginning of a comment
 * !! the string is stored in 'ValueString'
 */
void
cINIFILE::ReadValueString( void )
{
    char *ValueStringPtr = ValueString;
    const char *LinePtr  = CurrentLine;

    // clear valuestring
    memset( ValueString, ' ', MAXLINELENGTH );
	ValueString[MAXLINELENGTH - 1] = '\0';

    // advance through the strings until we find the end of the source string
    // or the equals character
    while ((*LinePtr != '=') && (*LinePtr != '\0'))
    {
        LinePtr++;
        ValueStringPtr++;
    }

    tBOOL CopyNextChar = TRUE;

    while ( CopyNextChar )
    {
        LinePtr++;
        ValueStringPtr++;

        // we encountered the beginning of a comment, don't continue to copy characters
        if (( *LinePtr == '/') && ((*(LinePtr+1) == '/')) || (*LinePtr == '\0'))
            CopyNextChar = FALSE;
        else
            *ValueStringPtr = *LinePtr;

        // the source line has ended, stop copying characters
        if ( *LinePtr == '\0' )
            CopyNextChar = FALSE;
    }
}

/*
 * Reads an integer value from the currently opened *.ini file
 * returns TRUE if the read was successfull, else FALSE
 */
tBOOL
cINIFILE::ReadNumber( const char *SectionName, const char *KeyWord, tSINT32 *ReturnValue )
{
    ASSERT( SectionName != NULL );
    ASSERT( KeyWord != NULL );

    // start at the beginning of the file
    Rewind( );

    if ( FindSection( SectionName ) )
        if ( FindKeyWord( KeyWord ) )
            {

            ReadValueString();
            sscanf( ValueString, "%i", ReturnValue ); // !! don't like 'sscanf()'

            return TRUE;
            }

	*LOG << "Trying to read string '" << KeyWord << "' from section '" << 
			 SectionName << "' in file '" << InputFileName << "' failed!\n";
	QUIT;

    return FALSE;
}

/*
 * Reads an unsigned integer value from the currently opened *.ini file
 * returns TRUE if the read was successfull, else FALSE
 */
tBOOL
cINIFILE::ReadNumber( const char *SectionName, const char *KeyWord, tUINT32 *ReturnValue )
{
    ASSERT( SectionName != NULL );
    ASSERT( KeyWord != NULL );

    // start at the beginning of the file
    Rewind( );

    if ( FindSection( SectionName ) )
        if ( FindKeyWord( KeyWord ) )
            {

            ReadValueString();
            sscanf( ValueString, "%u", ReturnValue ); // !! don't like 'sscanf()'

            return TRUE;
            }

	*LOG << "Trying to read string '" << KeyWord << "' from section '" << 
			 SectionName << "' in file '" << InputFileName << "' failed!\n";
	QUIT;

	return FALSE;
}

/*
 * !!! Warning: Extremely dangerous function! Assumes that RetunString already points to
 *              an allocated chunk of memory large enough to hold the string - very bad
 *              Find out a better solution to pass around strings!
 *
 * !!! This function is UGLY! replace asap!
 */
tBOOL
cINIFILE::ReadString( const char *SectionName, const char *KeyWord, const char *ReturnString )
{
    ASSERT( SectionName != NULL );
    ASSERT( KeyWord != NULL );
    ASSERT( ReturnString != NULL );

    // start at the beginning of the file
    Rewind( );

    if ( FindSection( SectionName ) )
        if ( FindKeyWord( KeyWord ) )
            {

            ReadValueString();

            char *ValueStringPtr = ValueString;
            char *LinePtr  = (char *) ReturnString;  // !!! casting away constness - yuck

            while ( *ValueStringPtr != '"' )
            {
                if ( *ValueStringPtr == '\0' )
                    return FALSE;
                ValueStringPtr++;
            }

            ValueStringPtr++;

            while (( *ValueStringPtr != '"') && ( *ValueStringPtr != '\0' ))
            {
                *LinePtr = *ValueStringPtr;
                ValueStringPtr++;
                LinePtr++;
            }

            *LinePtr = '\0';

            return TRUE;
            }

	*LOG << "Trying to read string '" << KeyWord << "' from section '" << 
			 SectionName << "' in file '" << InputFileName << "' failed!\n";
	QUIT;

    return FALSE;

}

