#ifndef __MAP_HPP__
#define __MAP_HPP__

/******************************************************************************
	HawkCreek software

	Header for MapClass

	19.03.2000: file generated by Capella
	07.10.2000: BuschnicK
				this file is a mess, too many confusing accessor functions, each 
				meaning a sliglty different thing... difficult to clean up though,
				since we already use this everywhere
*******************************************************************************/

#ifndef __TYPE_HPP__
#include "general\type.hpp"
#endif

#ifndef __HELPMACROS_HPP__
#include "general\helpmacros.hpp"
#endif

#include <list>
#include <vector>

class cSPRITE;
class cVECTOR;

#define DATAVERSION			3

#define FLAGBLOCKING		2 << 0
#define FLAGTRANSPARENT		2 << 1

typedef enum
{
	NONE		= 0,
	NORTHWEST,
	NORTH,
	NORTHEAST,
	EAST,
	SOUTHEAST,
	SOUTH,
	SOUTHWEST,
	WEST
} tDIRECTION;

/******************************************************************************************/

class // used like a struct, i.e. data members are modified from outside
cWALLINFO
{
public:
	tUINT32 TileSet;
	tUINT32 TileNr;

	cWALLINFO( tUINT32 TileSet = 0, tUINT32 TileNr = 0 ) 
		: TileSet( TileSet ), TileNr( TileNr ) { };
	~cWALLINFO( ) { };
};

/******************************************************************************************/

class // used like a struct, i.e. data members are modified from outside
cCELLINFO
{
public:
	cCELLINFO *NextLayer;
	tUINT32 TileSet;
	tUINT32 TileNr;

	cCELLINFO( tUINT32 TileSet = 0, tUINT32 TileNr = 0 ) 
		: NextLayer( NULL ), TileSet( TileSet ), TileNr( TileNr ) { };
	~cCELLINFO( ) { };
};

/******************************************************************************************/

class
cBASECELL
{
private:
	cCELLINFO			  FirstLayer;
	cWALLINFO			 *Wall;
	tUINT32				  Flags;
	std::list<cSPRITE *>  SpriteList;
public:
	cBASECELL( void );
	~cBASECELL( );

	inline cWALLINFO *GetWall( void );
	inline void SetWall( tUINT32 TileSet, tUINT32 TileNr );
	inline void DeleteWall( );

	inline cCELLINFO *GetFirstLayer( void );
	inline void SetFirstLayer( tUINT32 TileSet, tUINT32 TileNr, cCELLINFO *NextLayer );
	inline void DeleteAllLayers( void );
	inline void AddTopLayer( tUINT32 TileSet, tUINT32 TileNr );
	inline void RemoveTopLayer( void );

	inline tBOOL IsBlocking( void );

	std::list<cSPRITE *> *GetSpriteList( void ) { return &SpriteList; };

	tUINT32 GetFlags( void )			{ return Flags; };
	void SetFlags( tUINT32 Flags ) 		{ this->Flags = Flags; };

};

/******************************************************************************************/

class
cNODE
{
public:
	tSINT32 x, y;
	tSINT32 Cost;
	tSINT32 Heuristic;
	tSINT32 Total;
	cNODE *Parent;

	cNODE( tSINT32 x, tSINT32 y, tSINT32 Cost, tSINT32 Heuristic, cNODE *Parent = NULL )
		: x(x), y(y), Cost(Cost), Heuristic(Heuristic), Parent(Parent), Total( Cost + Heuristic ) 
	{
		ASSERT( Total >= 0 ); // an overflow has occured
	}

	friend bool operator > ( const cNODE &a, const cNODE &b );
};

/******************************************************************************************/

struct
tMAPHEADER
{
public:
	tUINT32 Version;		// 4
	char	Name[64];		// 64
	char	Author[64];		// 64
	tUINT32	Width;			// 4
	tUINT32 Height;			// 4
	tUINT8	Padding[126];	// 126
};							// 256 total

/******************************************************************************************/

class cMAP  
{
private:
	tMAPHEADER Header;
	cBASECELL* Cell;
	std::string FileName;

	void DeleteCells( void );

	inline cBASECELL *GetBaseCell( tUINT32 PosX, tUINT32 PosY );

	enum { COST_ALIGNED = 10, COST_DIAGONAL = 14 };
	enum { MAXITERATIONS = 1000 };	// how long should I try untill giving up? (there is no path to goal)
	tSINT32 GetHeuristic( tSINT32 x1, tSINT32 y1, tSINT32 x2, tSINT32 y2 );
	tSINT32 GetCost( tSINT32 x1, tSINT32 y1, tSINT32 x2, tSINT32 y2 );
	void AddNode( std::vector< cNODE * > &Open, std::vector< cNODE * > &Closed, cNODE *Node );
	void DeleteNodes( std::vector< cNODE * > &Open, std::vector< cNODE * > &Closed );

public:
	cMAP( );
	cMAP( const char *NewFileName );
	cMAP( tMAPHEADER MapHeader );
	~cMAP( );

    inline cCELLINFO *GetFirstLayer( tUINT32 PosX, tUINT32 PosY );
	inline void SetFirstLayer( tUINT32 PosX, tUINT32 PosY, tUINT32 TileSet, tUINT32 TileNr, cCELLINFO *NextLayer );
	inline void DeleteAllLayers( tUINT32 PosX, tUINT32 PosY );
	inline void AddTopLayer( tUINT32 PosX, tUINT32 PosY, tUINT32 TileSet, tUINT32 TileNr );
	inline void RemoveTopLayer( tUINT32 PosX, tUINT32 PosY );

	inline cWALLINFO *GetWall( tUINT32 PosX, tUINT32 PosY );
	inline void SetWall( tUINT32 PosX, tUINT32 PosY, tUINT32 TileSet, tUINT32 TileNr );
	inline void DeleteWall( tUINT32 PosX, tUINT32 PosY ); 

	inline tUINT32 GetFlags( tUINT32 PosX, tUINT32 PosY );
	inline void SetFlags( tUINT32 PosX, tUINT32 PosY, tUINT32 Flags );

	inline tBOOL IsBlocking( tUINT32 StartX, tUINT32 StartY, tUINT32 DestX, tUINT32 DestY );
	inline tBOOL IsValidCell( tSINT32 x, tSINT32 );

	inline std::list<cSPRITE *> *GetSpriteList( tUINT32 PosX, tUINT32 PosY );
	
	inline std::string GetFileName() { return FileName; };
	tUINT32 GetWidth( void )	{ return Header.Width; };
	tUINT32 GetHeight( void )	{ return Header.Height; };

	void Optimize( void );

	void Load( const char *FileName );
	void Save( const char *FileName );
	void SetName( const char* MapName );
	void SetAuthor( const char* Author );
	const char* GetName();
	const char* GetAuthor();

	void InsertSprite( cVECTOR *Position, cSPRITE *Sprite );
	void RemoveSprite( cVECTOR *Position, cSPRITE *Sprite );

	tBOOL FindPath( cVECTOR PointA, cVECTOR PointB, std::list<cVECTOR> *Path );
};

/******************************************************************************************/

/*
 *
 */
tBOOL
cMAP::IsBlocking( tUINT32 StartX, tUINT32 StartY, tUINT32 DestX, tUINT32 DestY )
{
	tBOOL Passable = IsValidCell( DestX, DestY ) && !GetBaseCell( DestX, DestY )->IsBlocking( );

	// if we are moving N, S, E, or W we are moving diagonal in arrayspace and all 
	// four touching cells must be passable, i.e. do not cut corners
	tSINT32 dx = DestX - StartX;
	tSINT32 dy = DestY - StartY;
	if (( dx != 0 ) && ( dy != 0 ))
		Passable &= !GetBaseCell( DestX - dx, DestY )->IsBlocking( )
				 && !GetBaseCell( DestX, DestY - dy )->IsBlocking( );

	return !Passable;
}

/*
 *
 */
tBOOL
cMAP::IsValidCell( tSINT32 x, tSINT32 y )
{
	return ((x >= 0) && (x < GetWidth( )) && (y >= 0) && (y < GetHeight( )));
}

/*
 *
 */
cCELLINFO *
cMAP::GetFirstLayer( tUINT32 PosX, tUINT32 PosY )
{ 
	return GetBaseCell( PosX, PosY )->GetFirstLayer( ); 
}

/*
 *
 */
void 
cMAP::SetFirstLayer( tUINT32 PosX, tUINT32 PosY, tUINT32 TileSet, tUINT32 TileNr, cCELLINFO *NextLayer )
{ 
	GetBaseCell( PosX, PosY )->SetFirstLayer( TileSet, TileNr, NextLayer ); 
}

/*
 *
 */
void
cMAP::DeleteAllLayers( tUINT32 PosX, tUINT32 PosY )
{
	GetBaseCell( PosX, PosY )->DeleteAllLayers( );
}

/*
 *
 */
void
cMAP::AddTopLayer( tUINT32 PosX, tUINT32 PosY, tUINT32 TileSet, tUINT32 TileNr )
{
	GetBaseCell( PosX, PosY )->AddTopLayer( TileSet, TileNr );
}

/*
 *
 */
void
cMAP::RemoveTopLayer( tUINT32 PosX, tUINT32 PosY )
{
	GetBaseCell( PosX, PosY )->RemoveTopLayer( );
}

/*
 *
 */
cWALLINFO *
cMAP::GetWall( tUINT32 PosX, tUINT32 PosY )
{ 
	return GetBaseCell( PosX, PosY )->GetWall( ); 
}

/*
 *
 */
void
cMAP::SetWall( tUINT32 PosX, tUINT32 PosY, tUINT32 TileSet, tUINT32 TileNr )
{
	GetBaseCell( PosX, PosY )->SetWall( TileSet, TileNr );
}

/*
 *
 */
void 
cMAP::DeleteWall( tUINT32 PosX, tUINT32 PosY )
{
	GetBaseCell( PosX, PosY )->DeleteWall( );
}

/*
 *
 */
cBASECELL *
cMAP::GetBaseCell( tUINT32 PosX, tUINT32 PosY )
{ 
	return &Cell[PosY * Header.Width + PosX]; 
}

/*
 *
 */
tUINT32 
cMAP::GetFlags( tUINT32 PosX, tUINT32 PosY )
{ 
	return GetBaseCell( PosX, PosY )->GetFlags( ); 
}

/*
 *
 */
void 
cMAP::SetFlags( tUINT32 PosX, tUINT32 PosY, tUINT32 Flags )
{ 
	GetBaseCell( PosX, PosY )->SetFlags( Flags ); 
}

std::list<cSPRITE *> *
cMAP::GetSpriteList( tUINT32 PosX, tUINT32 PosY )
{ 
	return GetBaseCell( PosX, PosY )->GetSpriteList( ); 
}



/******************************************************************************************/

/*
 * !! add sprite collisions here
 */
tBOOL 
cBASECELL::IsBlocking( void )
{
	return Flags & FLAGBLOCKING;
}

/*
 *
 */
cWALLINFO *
cBASECELL::GetWall( void )			
{ 
	return Wall; 
}

/*
 *
 */
void 
cBASECELL::SetWall( tUINT32 TileSet, tUINT32 TileNr )
{
	DeleteWall( ); // just in case there already is one
	Wall = new cWALLINFO( TileSet, TileNr );
	ASSERT( Wall != NULL );
	Flags |= FLAGBLOCKING;
}

/*
 *
 */
void
cBASECELL::DeleteWall( )
{
	SAFEDELETE( Wall );
	Flags = Flags & ~FLAGBLOCKING;
}

/*
 *
 */
cCELLINFO *
cBASECELL::GetFirstLayer( void ) 	
{ 
	return &FirstLayer; 
}

/*
 *
 */
void 
cBASECELL::SetFirstLayer( tUINT32 TileSet, tUINT32 TileNr, cCELLINFO *NextLayer ) 
{ 
	FirstLayer.TileSet		= TileSet; 
	FirstLayer.TileNr		= TileNr; 
	FirstLayer.NextLayer	= NextLayer; 
}

/*
 *
 */
void
cBASECELL::DeleteAllLayers( )
{
	while ( FirstLayer.NextLayer != NULL )
	{
		cCELLINFO *Layer = FirstLayer.NextLayer;
		cCELLINFO *OldLayer = &FirstLayer;

		while ( Layer->NextLayer != NULL )
		{
			OldLayer	= Layer;
			Layer		= Layer->NextLayer;
		}

		SAFEDELETE( Layer );

		Layer				= NULL;
		OldLayer->NextLayer = NULL;
	}
}

/* 
 * 
 */
void 
cBASECELL::AddTopLayer( tUINT32 TileSet, tUINT32 TileNr )
{
	cCELLINFO *Iterator = &FirstLayer;
	while ( Iterator->NextLayer )
		Iterator = Iterator->NextLayer;

	Iterator->NextLayer = new cCELLINFO( TileSet, TileNr );
}

/* 
 *
 */
void 
cBASECELL::RemoveTopLayer( )
{
	cCELLINFO *Iterator = &FirstLayer;
	while (( Iterator->NextLayer ) && ( Iterator->NextLayer->NextLayer ))
		Iterator = Iterator->NextLayer;

	SAFEDELETE( Iterator->NextLayer );

	Iterator->NextLayer = NULL;
}

#endif 