//--- Specific File Includes ----//
#include "Include File.h"


MAIN_DISPLAY::MAIN_DISPLAY()
{
	window_outline_width = 12;
}

//Is the mouse over the main display when a mouse button is pressed?
bool MAIN_DISPLAY::WindowMouseOver()
{
	if (mouse_x >= GetLeft() && mouse_x <= GetRight() &&
		mouse_y >= GetTop()  && mouse_y <= GetBottom())
		return TRUE;
	else
		return FALSE;
}


//For the main display, the action is executed emmeditately when upon mouse button down
//not when the mouse button is released up
void MAIN_DISPLAY::LMouseButtonDown()
{
	//For the MAIN_DISPLAY, it is devided into 2 things, the main MAIN_DISPLAY WINDOW, AND THE 
	//MAIN_DISPLAY_RESIZE

	//Since we know the mouse IS in the window, is it in either the MAIN_DISPLAY or MAIN_DISPLAY_RESIZE
	//and since it will be mostly likely be in the MAIN_DISPLAY, consider that one first

	if (mouse_x > x_start_pixel_pt && mouse_x < x_end_pixel_pt &&
		mouse_y > y_start_pixel_pt && mouse_y < y_end_pixel_pt)
		Action();
}


void MAIN_DISPLAY::LMouseButtonUp()
{
	//There are no buttons in the Main_Display
}

void MAIN_DISPLAY::RMouseButtonDown()
{
	//the only affect RMouseButtonDown() has is to deselect any that thing (unit/city/structure/tile terrain)
	//that is currently selected
	Unit_Vars.selected_unit = -1;
	City_Vars.selected_city = -1;
	Structure_Vars.selected_structure = -1;
	//tile terrain selection isnt implemented yet.
}

void MAIN_DISPLAY::RMouseButtonUp()
{
	//This does nothing
}


void MAIN_DISPLAY::MouseDoubleClick()
{
	//does nothing
}

void MAIN_DISPLAY::MouseOver()
{
	//Remove the mouse cursor from the tile it was in previously
	int box_x;
	int box_y;


	//(box_x,box_y) is the tile box that the mouse is in
	box_x = (mouse_x + x_screen_pos - x_start_pixel_pt)/tile_width;
	
	if (box_x % 2 == 1) //if box_x is odd
		box_y = (mouse_y + y_screen_pos - y_start_pixel_pt - tile_height/2)/tile_height;
	else
		box_y = (mouse_y + y_screen_pos - y_start_pixel_pt)/tile_height;


	
	
	//is the mouse over one of the edge of map tiles?
	if (box_x < 0 || box_y < 0 || box_x >= map_width || box_y >= map_height)
	{
		tile[mouse_in_x][mouse_in_y].bMouse_In = FALSE; //the mouse is no longer over a tile
								//so remove the cursor from the last tile it was in
		return;
	}
	else //if not, then its on the map, so set the old position to false and find the new position
		tile[mouse_in_x][mouse_in_y].bMouse_In = FALSE;


	
	//(box_x_pos,box_y_pos) is the position in pixels of the box
	int box_x_pos = box_x * tile_width;
	int box_y_pos = box_y * tile_height;


//*********************************************
	//(box_mouse_x,box_mouse_y) is the position of the mouse relative to the box
	int box_mouse_x = mouse_x + x_screen_pos - x_start_pixel_pt - box_x_pos; //  the +x_screen_pos here causes this it to crash
	int box_mouse_y;
	if (box_x % 2 == 1) //if box_x is odd
		box_mouse_y = mouse_y + y_screen_pos - y_start_pixel_pt - tile_height/2 - box_y_pos;
	else
		box_mouse_y = mouse_y + y_screen_pos - y_start_pixel_pt - box_y_pos;
//*********************************************


	if (box_mouse_x > 15) //then the mouse is in the box
	{
		mouse_in_x = box_x;
		mouse_in_y = box_y;
		tile[mouse_in_x][mouse_in_y].bMouse_In = TRUE;
	}
	else //then the mouse is in a possible of 3 tiles, find which one
	{
		double temp1 = tile_height;
		double temp2 = box_mouse_y;
		double temp3 = box_mouse_x;
		double temp  = (temp1/2 - temp2)/temp3;  //(tile_height/2 - box_mouse_y)/box_mouse_x
	
		double angle = atan(temp); //find the angle with arctan


		//set the new tile the mouse is in
		if (angle > 0.7854) //if the angle is greater then 45 degrees
		{
			if (box_x % 2 == 1) //is it odd?
			{
				mouse_in_x = box_x -1;
				mouse_in_y = box_y;
				tile[mouse_in_x][mouse_in_y].bMouse_In = TRUE;
			}
			else  //else if its even
			{
				mouse_in_x = box_x -1;
				mouse_in_y = box_y -1;
				tile[mouse_in_x][mouse_in_y].bMouse_In = TRUE;
			}
		}
		else if (angle < -0.7854) //if the angle is < -45 degrees
		{
			if (box_x % 2 == 1) //is it odd?
			{
				mouse_in_x = box_x -1;
				mouse_in_y = box_y +1;
				tile[mouse_in_x][mouse_in_y].bMouse_In = TRUE;
			}
			else //else if its even
			{
				mouse_in_x = box_x -1;
				mouse_in_y = box_y;
				tile[mouse_in_x][mouse_in_y].bMouse_In = TRUE;
			}
		}
		else
		{
			mouse_in_x = box_x;
			mouse_in_y = box_y;
			tile[mouse_in_x][mouse_in_y].bMouse_In = TRUE;
		}
	}
}









/*----------------------------------------------------------------------------------------------
What the action is gonna do in here is fairly complex. First, chech if a unit has been selected
If a unit has then be selected, then clicking on the map will move that unit.

If a unit has not been selected, then clicking on the map will select what is in that tile in
this order.
Unit, City/Structure, terrain.
For example, there is a city on grass, with a unit on it. Clicking
on the tile once will select the unit. If you click on that tile again, it will deselect
the unit and select the city. If you click on it again, it will select the terrain.
----------------------------------------------------------------------------------------------*/
void MAIN_DISPLAY::Action()
{
	//selected_unit   the number of the currently selected unit, = -1 if no unit selected
	//mouse_in_x	the tile the mouse is currently in.
	//mouse_in_y      
	//tile[mouse_in_x][mouse_in_y].unit_number = the unit number of the unit in the tile, = -1 if no
																	//unit there

	//check if a unit has already been selected
	if (Unit_Vars.selected_unit == -1)
	{					//if = -1, then no unit has been selected, so selected the unit in the tile
		Unit_Vars.selected_unit = tile[mouse_in_x][mouse_in_y].unit_number;
		City_Vars.selected_city = -1;
		Structure_Vars.selected_structure = -1;

		
		//then check if there was a unit in that tile, if there was none, then selected_unit now = -1,
		//so then check if there was a city/structure in that tile, if there is a one, then select it
		if (Unit_Vars.selected_unit == -1)
		{
			City_Vars.selected_city = tile[mouse_in_x][mouse_in_y].city;
			Structure_Vars.selected_structure = -1;
			

			//if selected_city still = -1, then there was no city in that tile, so select the structure
			//that is there
			if (City_Vars.selected_city == -1)
			{
				Structure_Vars.selected_structure = tile[mouse_in_x][mouse_in_y].structure;


				//if selected_structure still = -1, then there was no structure in that tile, so select
				//the terrain of the tile
				if (Structure_Vars.selected_structure == -1)
				{
					//select the terrain in this tile, so you can see what improvements the terrain has ect...
					return;
				}
				return;
			}
			return;
		}
		return;
	}

	else	//this 'else' occurs when is already selected, in which case 2 things can happen
	{		//1: If a unit is selected, and he clicks in a tile other then the tile the unit is in
			//	 then that will move the unit to the destination tile
			//2: If a unit is selected, and he clicks in the same tile as the selected unit, then
			//   that means he desires to the select the next layer, a city/stucture that is in the tile
			//	 if the tile has one, otherwise to select terrain of the tile.

		//unit[selected_unit].x_tile   is the location of the selected unit
		//unit[selected_unit].y_tile
		//mouse_in_x	 the tile the mouse is currently in.
		//mouse_in_y      

		if (unit[Unit_Vars.selected_unit].x_tile == mouse_in_x  &&
			unit[Unit_Vars.selected_unit].y_tile == mouse_in_y)
		{
			//then select the next layer, first checking if a city is in that tile
			City_Vars.selected_city = tile[mouse_in_x][mouse_in_y].city;
			Unit_Vars.selected_unit = -1;      //deselect any selected untis or structures
			Structure_Vars.selected_structure = -1;
			

			//if selected_city still = -1, then there was no city in that tile, so select the structure
			//that is there
			if (City_Vars.selected_city == -1)
			{
				Structure_Vars.selected_structure = tile[mouse_in_x][mouse_in_y].structure;


				//if selected_structure still = -1, then there was no structure in that tile, so select
				//the terrain of the tile
				if (Structure_Vars.selected_structure == -1)
				{
					//select the terrain in this tile, so you can see what improvements the terrain has ect...
					return;
				}
				return;
			}
			return;
		}

		else //then move the unit to the destination tile
		{
			MoveUnit(mouse_in_x, mouse_in_y);   //the move unit function takes in units destination
												  //for arguments
			return;
		}
	}
}







void MAIN_DISPLAY::Render()
{
	int x_pos_on_screen;  //final spot on the monitor
	int y_pos_on_screen;

	//first determine which tile the x_screen_pos and y_screen_pos are in
	int x_ultile = x_screen_pos/tile_width;
	int y_ultile = y_screen_pos/tile_height;
	int x_offset = x_screen_pos - x_ultile*tile_width;
	int y_offset = y_screen_pos - y_ultile*tile_height;

	int x_tile;  //this number is increment with each loop by adding the loop counter to it
	int y_tile;

	int x_loop;



//THE ROW LOOP
	for (int y = -1; y <= y_tiles_on_screen; y++)
	{

		if (x_ultile % 2 == 1) //if the x upper left tile is odd
			x_loop = -1;
		else
			x_loop = 0;


//DRAW THE EVEN COLUMN ROW
		for (int x = x_loop; x <= x_tiles_on_screen; x=x+2)
		{
			//default image dimensions
			rct.left   = 0;
			rct.right  = tile_image_width;
			rct.top    = 0;
			rct.bottom = tile_image_height;


			//the position of image on the screen
			x_pos_on_screen = x * tile_width - x_offset + x_start_pixel_pt; 


			//does the image get cut off on the left side of the screen?
			if(x_pos_on_screen < x_start_pixel_pt)
			{
				rct.left = x_start_pixel_pt - x_pos_on_screen;
				x_pos_on_screen = x_start_pixel_pt;
			}

			//does the image get cut off on the right side of the screen?
			if (x_pos_on_screen + tile_image_width > x_end_pixel_pt +1)
				rct.right = x_end_pixel_pt - x_pos_on_screen +1;


			
			x_tile = x_ultile + x;

			y_tile = y_ultile + y;

						
			//y position on the screen
			if (((x_tile % 2) == 1) || (x_tile == -1))  // if the x column is odd, lower the column
				y_pos_on_screen = y * tile_height - y_offset + tile_height/2 - (tile_image_height - tile_height) + y_start_pixel_pt;
			else
				y_pos_on_screen = y * tile_height - y_offset - (tile_image_height - tile_height) + y_start_pixel_pt;			
				//this is i*32 - offset - (52-32) + 12
		


			//does the image get cut off on the top?
			if (y_pos_on_screen < y_start_pixel_pt)
			{
				rct.top = y_start_pixel_pt - y_pos_on_screen;
				y_pos_on_screen = y_start_pixel_pt;
			}
			

			//does the image get cut off on the bottom?
			if (y_pos_on_screen + tile_image_height > y_end_pixel_pt +1)
				rct.bottom = y_end_pixel_pt - y_pos_on_screen +1;



			/////////////////////////////
			//now finally draw the layers
			cRender_Frame.Tile_Layers(x_pos_on_screen, y_pos_on_screen, x_tile, y_tile);
		}


		//switch the x_loop value
		if (x_loop == 0)
			x_loop = -1;
		else
			x_loop = 0;

//NOW DRAW THE ODD COLUMN ROW, this row is tile_height/2 lower then the even column row
		for (int xb = x_loop; xb <= x_tiles_on_screen; xb=xb+2)
		{
			//default image dimensions
			rct.left   = 0;
			rct.right  = tile_image_width;
			rct.top    = 0;
			rct.bottom = tile_image_height;


			//the position of image on the screen
			x_pos_on_screen = xb * tile_width - x_offset + x_start_pixel_pt; 


			
			//does the image get cut off on the left side of the screen?
			if(x_pos_on_screen < x_start_pixel_pt)
			{
				rct.left = x_start_pixel_pt - x_pos_on_screen;
				x_pos_on_screen = x_start_pixel_pt;
			}

			//does the image get cut off on the right side of the screen?
			if (x_pos_on_screen + tile_image_width > x_end_pixel_pt +1)
				rct.right = x_end_pixel_pt - x_pos_on_screen +1;

		
			
			x_tile = x_ultile + xb;

			y_tile = y_ultile + y;

				
			
			//y position on the screen
			if (((x_tile % 2) == 1) || (x_tile == -1))  // if the x column is odd, lower the column
				y_pos_on_screen = y * tile_height - y_offset + tile_height/2 - (tile_image_height - tile_height) + y_start_pixel_pt;
			else
				y_pos_on_screen = y * tile_height - y_offset - (tile_image_height - tile_height) + y_start_pixel_pt;			
				//this is i*32 - offset - (52-32) + 12
			


			//does the image get cut off on the top?
			if (y_pos_on_screen < y_start_pixel_pt)
			{
				rct.top = y_start_pixel_pt - y_pos_on_screen;
				y_pos_on_screen = y_start_pixel_pt;
			}
			

			//does the image get cut off on the bottom?
			if (y_pos_on_screen + tile_image_height > y_end_pixel_pt +1)
				rct.bottom = y_end_pixel_pt - y_pos_on_screen +1;



			/////////////////////////////
			//now finally draw the layers
			cRender_Frame.Tile_Layers(x_pos_on_screen, y_pos_on_screen, x_tile, y_tile);
		}
	}
}	


void MAIN_DISPLAY::NotInMainDisplay()
{
	tile[mouse_in_x][mouse_in_y].bMouse_In = FALSE;
}

