//--- Specific File Includes ----//
#include "Include File.h"




void RENDER_FRAME::Tile_Layers(int x_pos_on_screen, int y_pos_on_screen, int x, int y)
{
	//NOW SEND THE IMAGE TO BACK BUFFER
	//first consider the blank tiles at the edges of the map
	if ((x <= -1) || (y <= -1) || (x >= map_width) || (y >= map_height))
		lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, blanktile_surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
	else
	{
		//FIRST CHECK IF THIS TILE IS EXPLORED YET, if its not, then display the black unexplored tile
		if (FogOfWar[x][y][Game_Globals.current_player].black)
			lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, UnexploredTile_surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);

		else //otherwise, then the tile might be fully seen or coverd by the fog of war
		{
			//DRAW EACH LAYER IN ORDER
			//1: first layer is the terrain	and the terrain enhancements	
			DrawTerrainEnhancements(x_pos_on_screen, y_pos_on_screen, x, y);

			//2: the second layer is the tile outline, but only if it has one
			if (Game_Globals.bDraw_Tile_Outline)
				DrawTileOutlines(x_pos_on_screen, y_pos_on_screen, x, y);

			//3: check if this tile has the mouse in it, if true, blit the cursor
			if (tile[x][y].bMouse_In)
				Cursor.Cursor(x_pos_on_screen, y_pos_on_screen);

			//4: draw the City or structures
			DrawCitiesStructures(x_pos_on_screen, y_pos_on_screen, x, y);		

			//5: draw the units
			DrawUnits(x_pos_on_screen, y_pos_on_screen, x, y);

			//6: draw the fog of war shading
			if (FogOfWar[x][y][Game_Globals.current_player].fog_of_war)
				lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, FogOfWar_surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
		}
	}
}



void DrawTerrainEnhancements(int x_pos_on_screen, int y_pos_on_screen, int x, int y)
{
	if (FogOfWar[x][y][Game_Globals.current_player].fog_of_war)
	{
		//draw the terrain type
		lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, terrain[FogOfWar[x][y][Game_Globals.current_player].type].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);

		//then draw any terrain enhancements
		if (tile[x][y].terrain_enhancement1 > -1)
		{
			lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, terrain_enhancements[FogOfWar[x][y][Game_Globals.current_player].terrain_enhancement1].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
			if (tile[x][y].terrain_enhancement2 > -1)
			{
				lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, terrain_enhancements[FogOfWar[x][y][Game_Globals.current_player].terrain_enhancement2].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
				if (tile[x][y].terrain_enhancement3 > -1)
				{
					lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, terrain_enhancements[FogOfWar[x][y][Game_Globals.current_player].terrain_enhancement3].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
					if (tile[x][y].terrain_enhancement4 > -1)
					{
						lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, terrain_enhancements[FogOfWar[x][y][Game_Globals.current_player].terrain_enhancement4].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
					}
				}
			}
		}
	}

	//else, then there is no far of war coving this tile, so you can see it all
	else
	{
		//draw the terrain type
		lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, terrain[tile[x][y].type].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);

		//then draw any terrain enhancements
		if (tile[x][y].terrain_enhancement1 > -1)
		{
			lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, terrain_enhancements[tile[x][y].terrain_enhancement1].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
			if (tile[x][y].terrain_enhancement2 > -1)
			{
				lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, terrain_enhancements[tile[x][y].terrain_enhancement2].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
				if (tile[x][y].terrain_enhancement3 > -1)
				{
					lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, terrain_enhancements[tile[x][y].terrain_enhancement3].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
					if (tile[x][y].terrain_enhancement4 > -1)
					{
						lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, terrain_enhancements[tile[x][y].terrain_enhancement4].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
					}
				}
			}
		}
	}
}

void DrawCitiesStructures(int x_pos_on_screen, int y_pos_on_screen, int x, int y)
{
	if (FogOfWar[x][y][Game_Globals.current_player].fog_of_war)
	{
		int city_number = FogOfWar[x][y][Game_Globals.current_player].city;
		//draw a city if there is one

		if (city_number > -1)
		{
			if (city_number == City_Vars.selected_city && Blinking_Selected_Object.bDisplay()) //is the city the selected city? -- and if so,
			{																		//then check weather to display it or not
				//now draw the city
				lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, City_Vars.city_surf[city[city_number].city_image_number], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
			}
			else if (city_number != City_Vars.selected_city) //else draw the city if its not the selected city
			{
				//now draw the city
				lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, City_Vars.city_surf[city[city_number].city_image_number], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
			}
		}
		else
		{
			//draw any structures    they are only drawn if there is no city
			if (tile[x][y].structure1 > -1)
			{
				if (tile[x][y].structure1 == Structure_Vars.selected_structure && Blinking_Selected_Object.bDisplay()) //is the city the selected city? -- and if so,
				{																		//then check weather to display it or not
					//now draw the structure
					lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, structure[tile[x][y].structure1].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
				}
				else if (tile[x][y].structure1 != Structure_Vars.selected_structure)
				{
					//now draw the structure
					lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, structure[tile[x][y].structure1].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
				}

				if (tile[x][y].structure2 > -1)
				{
					if (tile[x][y].structure2 == Structure_Vars.selected_structure && Blinking_Selected_Object.bDisplay()) //is the city the selected city? -- and if so,
					{																		//then check weather to display it or not
						//now draw the structure
						lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, structure[tile[x][y].structure2].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
					}
					else if (tile[x][y].structure2 != Structure_Vars.selected_structure)
					{
						//now draw the structure
						lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, structure[tile[x][y].structure2].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
					}

					if (tile[x][y].structure3 > -1)
					{
						if (tile[x][y].structure3 == Structure_Vars.selected_structure && Blinking_Selected_Object.bDisplay()) //is the city the selected city? -- and if so,
						{																		//then check weather to display it or not
							//now draw the structure
							lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, structure[tile[x][y].structure3].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
						}
						else if (tile[x][y].structure3 != Structure_Vars.selected_structure)
						{
							//now draw the structure
							lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, structure[tile[x][y].structure3].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
						}
					}
				}
			}
		}
	}


	else
	{
		int city_number = tile[x][y].city;
		//draw a city if there is one

		if (city_number > -1)
		{
			if (city_number == City_Vars.selected_city && Blinking_Selected_Object.bDisplay()) //is the city the selected city? -- and if so,
			{																		//then check weather to display it or not
				//now draw the city
				lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, City_Vars.city_surf[city[city_number].city_image_number], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
			}
			else if (city_number != City_Vars.selected_city) //else draw the city if its not the selected city
			{
				//now draw the city
				lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, City_Vars.city_surf[city[city_number].city_image_number], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
			}
		}
		else
		{
			//draw any structures    they are only drawn if there is no city
			if (tile[x][y].structure1 > -1)
			{
				if (tile[x][y].structure1 == Structure_Vars.selected_structure && Blinking_Selected_Object.bDisplay()) //is the city the selected city? -- and if so,
				{																		//then check weather to display it or not
					//now draw the structure
					lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, structure[tile[x][y].structure1].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
				}
				else if (tile[x][y].structure1 != Structure_Vars.selected_structure)
				{
					//now draw the structure
					lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, structure[tile[x][y].structure1].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
				}

				if (tile[x][y].structure2 > -1)
				{
					if (tile[x][y].structure2 == Structure_Vars.selected_structure)
					{
						//then check weather to display it or not
						//now draw the structure
						lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, structure[tile[x][y].structure2].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
					}
					else if (tile[x][y].structure2 != Structure_Vars.selected_structure)
					{
						//now draw the structure
						lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, structure[tile[x][y].structure2].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
					}

					if (tile[x][y].structure3 > -1)
					{
						if (tile[x][y].structure3 == Structure_Vars.selected_structure && Blinking_Selected_Object.bDisplay()) //is the city the selected city? -- and if so,
						{																		//then check weather to display it or not
							//now draw the structure
							lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, structure[tile[x][y].structure3].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
						}
						else if (tile[x][y].structure3 != Structure_Vars.selected_structure)
						{
							//now draw the structure
							lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, structure[tile[x][y].structure3].surf, &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
						}
					}
				}
			}
		}
	}
}


void DrawUnits(int x_pos_on_screen, int y_pos_on_screen, int x, int y)
{
	if (!FogOfWar[x][y][Game_Globals.current_player].fog_of_war)
	{
		int unit_number = tile[x][y].unit_number;

		//is there a unit there?
		if (unit_number > -1)
		{
			if (unit_number == Unit_Vars.selected_unit && Blinking_Selected_Object.bDisplay()) //is the unit the selected unit? -- and if so,
			{																		//then check weather to display it or not
				//now draw the unit
				lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, Unit_Vars.unitImages_surf[unit[unit_number].unit_type], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);


				//draw the hp power bars
				int temp1 = unit[unit_number].player_number;
				lpDDSBack->BltFast (x_pos_on_screen, y_pos_on_screen, Unit_Vars.playerhpbars_surf[temp1], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);

				//draw the unit color indicator
				//the Hp has to be converted to a 10pt scale, so if the units Hp is 23/25, then that is equal to 23*10/25 = 9.2 = the 9Hp power bar is shown
				float Hp = unit[unit_number].hp * 10.0 / unit_types[unit[unit_number].unit_type].hp;
				int temp2 = Hp;//this rounds the decimal down and into an int
				lpDDSBack->BltFast (x_pos_on_screen, y_pos_on_screen, Unit_Vars.hpbars_surf[temp2], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
			}
			else if (unit_number != Unit_Vars.selected_unit) //if its not the selected unit, then display it always
			{																		//then check weather to display it or not
				//now draw the unit
				lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, Unit_Vars.unitImages_surf[unit[unit_number].unit_type], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);


				//draw the hp power bars
				int temp1 = unit[unit_number].player_number;
				lpDDSBack->BltFast (x_pos_on_screen, y_pos_on_screen, Unit_Vars.playerhpbars_surf[temp1], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);

				//draw the unit color indicator
				//the Hp has to be converted to a 10pt scale, so if the units Hp is 23/25, then that is equal to 23*10/25 = 9.2 = the 9Hp power bar is shown
				float Hp = unit[unit_number].hp * 10.0 / unit_types[unit[unit_number].unit_type].hp;
				int temp2 = Hp;//this rounds the decimal down and into an int
				lpDDSBack->BltFast (x_pos_on_screen, y_pos_on_screen, Unit_Vars.hpbars_surf[temp2], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
			}
		}
	}
}


//Function to blit out the tile outline images
void DrawTileOutlines(int x_pos_on_screen, int y_pos_on_screen, int x, int y)
{
	switch (tile[x][y].Hex_Outline)
	{
	case eHex_Outline1:
		lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, City_Vars.Hex_Outline_surf[eHex_Outline1], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
		break;

	case eHex_Outline2:
		lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, City_Vars.Hex_Outline_surf[eHex_Outline2], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
		break;

	case eHex_Outline3:
		lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, City_Vars.Hex_Outline_surf[eHex_Outline3], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
		break;

	case eHex_Outline4:
		lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, City_Vars.Hex_Outline_surf[eHex_Outline4], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
		break;

	case eHex_Outline5:
		lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, City_Vars.Hex_Outline_surf[eHex_Outline5], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
		break;

	case eHex_Outline6:
		lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, City_Vars.Hex_Outline_surf[eHex_Outline6], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
		break;

	case eHex_Outline7:
		lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, City_Vars.Hex_Outline_surf[eHex_Outline7], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
		break;

	case eHex_Outline8:
		lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, City_Vars.Hex_Outline_surf[eHex_Outline8], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
		break;

	case eHex_Outline9:
		lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, City_Vars.Hex_Outline_surf[eHex_Outline9], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
		break;

	case eHex_Outline10:
		lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, City_Vars.Hex_Outline_surf[eHex_Outline10], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
		break;

	case eHex_Outline11:
		lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, City_Vars.Hex_Outline_surf[eHex_Outline11], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
		break;

	case eHex_Outline12:
		lpDDSBack->BltFast(x_pos_on_screen, y_pos_on_screen, City_Vars.Hex_Outline_surf[eHex_Outline12], &rct, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
		break;

	default:
		break;
	}
}



