#ifndef UNIT_H
#define UNIT_H



//------ A STRUCT FOR EACH AND EVERY UNIT IN THE GAME ---------//
class UNIT{
public:
	UNIT();
	void CreateUnit(int unit_type, int player_number, int x_tile, int y_tile);
	//this function is called each time a new unit is created in the game
	void DestroyUnit(int unit_number);

	float GetAttack();
	float GetDefense();
	float GetPower();
	float GetArmor();
	float GetDamage();

	short GetNumAttacks();

	bool  CanRetaliate();	//does the unit still have retaliations left for this turn?

	bool  CanMove();	//Takes in a unit_number, and returns true if the unit can still move
	int	  GetVisionRange();
	LPDIRECTDRAWSURFACE7 GetImageSurf(); //takes in a unit_number and returns the unit image
	

	//UNIT TYPES
	bool Calvary();
	bool Swordsman();
	bool Pikeman();
	bool Archer();
	bool Slinger();
	bool Catapult();
	bool Ram();
	bool Dragon();

	//UNIT SPECIAL ABILITIES
	bool Charge();
	bool Etra_Strike();
	bool CanFly();
	bool CanSwim();
	bool Cold_Breath();
	bool Fire_Breath();
	bool Dominate();
	bool Heal();
	bool Web();
	bool Fire_Touch();
	bool Entangle();
	bool Turn();
	bool Magic();
	//ect....


//These are a must requirement to be stored with each specific unit

	short	unit_type;      //a # for the type of unit it is: like Elven Cav and Halfling Cav are different numbers
	short	player_number;  //the player number of the unit owner
	bool	alive;			//TRUE or FALSE

	int		x_tile;   //unit location on map			
	int		y_tile;

	float	hp;				//current amount of hitpoints
	short	movement;		//current amount of movement points remaining

	short	experience;		//0-4 for green to, discelplined , hardened, verteran. elite only for certain races/situations
							//0 for no bonus, # * .25 upto +1 for damage
							//# * .5  so 0 to + 2 for attack/defense
							//does not affect power/armor

	short	moral;			//1-5 1:very bad 2:poor 3:ok 4:good 5:very high
							//1: -2 attack/defense -1 damage
							//2: -1 attack/defense
							//5: +1 attack/defense/damage

	short	retaliations;	//this number starts at 0 for all units upon entering tactical combat, and 
							//each time a unit retaliates it increases by one, and is reset back to 0 
							//at the begining of each new round in tactical combat
							//the number of retaliations can not exceed the number
							//set for the unit in unit_types
							//--- if this number equals the max number, then the unit can no longer
							//--- retaliate

	short	strike_number;	//this is the number

	//special abilities that must be stored with the unit
	//usually because it can only be used once per combat or once per day, after its been used it must be
	//set to FALSE, and then at the begining of the next turn, it is set back to TRUE;
	bool turn;
	bool charge;
	bool heal;

	//spell affects and other effects that will be particular to this specific unit
	bool cursed;
	bool blessed;
	bool stone_skin;
	bool poisoned;
	bool frozen;
	bool burning;
	bool shocked;
	bool turned;
	bool webbed;
	bool entangled;
	bool enchanted;
	bool hasted;
	bool water_walking;
	bool invisible;
	bool wind_walking;
	//ect.....
};
extern class UNIT unit[MAX_NUM_UNITS];















class UNIT_TYPES
{
public:
	UNIT_TYPES();				//sets up all the unit types

	LPDIRECTDRAWSURFACE7	surf;

//unit stats
	float	hp;					//max amount of hitpoints
	float	attack;
	float	defense;
	float	power;				//power of the sword swing
	float	armor;				//power absorber of the armor
	float	damage;
	short 	movement;
	short	num_attacks;
	short	retaliations;
	int		vision_range;


	//UNIT TYPES
	bool calvary;
	bool swordsman;
	bool pikeman;
	bool archer;
	bool slinger;
	bool catapult;
	bool ram;
	bool dragon;

	//UNIT SPECIAL ABILITIES
	short	BuildCityMethod;   // = -1 if cant build city, otherwise enum {eBuildCity, eBuildCitySpell, eBuildUnderGroundCity, eBuildMtCity, eAnyTerrainBuildCitySpell};
	bool turn;
	bool charge;
	bool heal;

	bool etra_strike;
	bool can_fly;
	bool can_swim;
	bool cold_breath;
	bool fire_breath;
	bool fire_touch;
	bool dominate;
	bool web;
	bool fire_Touch;
	bool entangle;
	bool magic;
	//ect....
};
extern class UNIT_TYPES unit_types[eNUM_UNIT_TYPES];   //the tile[][].unit_number goes in here
				 // first unit made is #0 second unit made is #1 ect....

#endif